@@ -17,7 +17,6 @@
 
 """Handles branch-specific operations."""
 
-
 import os
 from subvertpy import (
     ERR_FS_NO_SUCH_REVISION,
@@ -30,7 +29,6 @@ from bzrlib import (
     osutils,
     tag,
     trace,
-    ui,
     urlutils,
     )
 from bzrlib.branch import (
@@ -48,9 +46,12 @@ from bzrlib.bzrdir import (
     )
 from bzrlib.errors import (
     DivergedBranches,
+    IncompatibleFormat,
     LocalRequiresBoundBranch,
+    LossyPushToSameVCS,
     NoSuchRevision,
     NotBranchError,
+    TokenLockingNotSupported,
     UnstackableBranchFormat,
     )
 from bzrlib.foreign import (
@@ -69,7 +70,6 @@ from bzrlib.plugins.svn import (
     util,
     )
 from bzrlib.plugins.svn.fetch import (
-    FetchRevisionFinder,
     InterFromSvnRepository,
     )
 from bzrlib.plugins.svn.push import (
@@ -82,12 +82,11 @@ from bzrlib.plugins.svn.config import (
 from bzrlib.plugins.svn.errors import (
     NotSvnBranchPath,
     PushToEmptyBranch,
-    )
-from bzrlib.plugins.svn.repository import (
-    SvnRepository,
+    SubversionBranchDiverged,
     )
 from bzrlib.plugins.svn.tags import (
     SubversionTags,
+    resolve_tags_svn_ancestry,
     )
 from bzrlib.plugins.svn.transport import (
     bzr_to_svn_url,
@@ -112,16 +111,17 @@ class SubversionSourcePullResult(PullResult):
                 if self.new_revmeta is None:
                     self.new_revmeta, _ = self.source_branch.repository._get_revmeta(self.new_revid)
                 to_file.write('Now on revision %d (svn revno: %d).\n' %
-                        (self.new_revno, self.new_revmeta.revnum))
+                        (self.new_revno, self.new_revmeta.metarev.revnum))
         self._show_tag_conficts(to_file)
 
 
 class SubversionWriteLock(object):
     """A (dummy) write lock on a Subversion object."""
 
-    __slots__ = ('unlock')
+    __slots__ = ('unlock', 'branch_token')
 
     def __init__(self, unlock):
+        self.branch_token = None
         self.unlock = unlock
 
     def __repr__(self):
@@ -156,10 +156,9 @@ class SubversionTargetBranchPushResult(BranchPushResult):
         try:
             return self._lookup_revno(self.old_revid)
         except NoSuchRevision:
-            # Last resort
-            return len(
-                self.target_branch.repository.iter_reverse_revision_history(
-                    self.old_revid))
+            graph = self.target_branch.repository.get_graph(
+                self.source_branch.repository)
+            return graph.find_distance_to_null(self.old_revid, [])
 
     @property
     def new_revno(self):
@@ -181,10 +180,9 @@ class SubversionTargetPullResult(PullResult):
         try:
             return self._lookup_revno(self.old_revid)
         except NoSuchRevision:
-            # Last resort
-            return len(
-                list(self.target_branch.repository.iter_reverse_revision_history(
-                    self.old_revid)))
+            graph = self.target_branch.repository.get_graph(
+                self.source_branch.repository)
+            return graph.find_distance_to_null(self.old_revid, [])
 
     @property
     def new_revno(self):
@@ -194,11 +192,12 @@ class SubversionTargetPullResult(PullResult):
 class SvnBranch(ForeignBranch):
     """Maps to a Branch in a Subversion repository """
 
-    def __init__(self, repository, branch_path, revnum=None, _skip_check=False,
-                 mapping=None):
+    def __init__(self, repository, controldir, branch_path, mapping, revnum=None,
+            _skip_check=False):
         """Instantiate a new SvnBranch.
 
-        :param repos: SvnRepository this branch is part of.
+        :param repository: SvnRepository this branch is part of.
+        :param controldir: Control dir this branch was opened on
         :param branch_path: Relative path inside the repository this
             branch is located at.
         :param revnum: Subversion revision number of the branch to
@@ -206,16 +205,15 @@ class SvnBranch(ForeignBranch):
         :param _skip_check: If True, don't check if the branch actually exists.
         """
         self.repository = repository
-        self.bzrdir = repository.bzrdir
+        self.bzrdir = controldir
         self._format = SvnBranchFormat()
         self.layout = self.repository.get_layout()
-        assert isinstance(self.repository, SvnRepository)
-        super(SvnBranch, self).__init__(mapping or self.repository.get_mapping())
-        self._lock_mode = None
-        self._lock_count = 0
         self._branch_path = branch_path.strip("/")
         self.base = urlutils.join(self.repository.base,
                         self._branch_path).rstrip("/")
+        super(SvnBranch, self).__init__(mapping)
+        self._lock_mode = None
+        self._lock_count = 0
         self._revmeta_cache = None
         self._revnum = revnum
         assert isinstance(self._branch_path, str)
@@ -234,11 +232,18 @@ class SvnBranch(ForeignBranch):
         if type not in ('branch', 'tag') or ip != '':
             raise NotBranchError(branch_path)
 
+    def leave_lock_in_place(self):
+        raise NotImplementedError(self.leave_lock_in_place)
+
+    def dont_leave_lock_in_place(self):
+        raise NotImplementedError(self.dont_leave_lock_in_place)
+
     def _push_should_merge_tags(self):
         return self.supports_tags()
 
     def check_path(self):
-        return self.repository.transport.check_path(self._branch_path, self._revnum or self.repository.get_latest_revnum())
+        return self.repository.transport.check_path(self._branch_path,
+            self._revnum or self.repository.get_latest_revnum())
 
     def supports_tags(self):
         """See Branch.supports_tags()."""
@@ -264,13 +269,19 @@ class SvnBranch(ForeignBranch):
         assert revnum >= 0
 
         last_revmeta, _ = self.last_revmeta()
-        if revnum == last_revmeta.revnum:
-            return last_revmeta.branch_path
+        if revnum > last_revmeta.metarev.revnum:
+            # Apparently a commit happened in the mean time
+            self._clear_cached_state()
+            last_revmeta, _ = self.last_revmeta()
+        if revnum == last_revmeta.metarev.revnum:
+            return last_revmeta.metarev.branch_path
+
+        locations = self.repository.transport.get_locations(
+                last_revmeta.metarev.branch_path, last_revmeta.metarev.revnum,
+                [revnum])
 
         # Use revnum - this branch may have been moved in the past
-        return self.repository.transport.get_locations(
-                    last_revmeta.branch_path, last_revmeta.revnum,
-                    [revnum])[revnum].strip("/")
+        return locations[revnum].strip("/")
 
     def get_revnum(self):
         """Obtain the Subversion revision number this branch was
@@ -278,7 +289,7 @@ class SvnBranch(ForeignBranch):
 
         :return: Revision number
         """
-        return self.last_revmeta()[0].revnum
+        return self.last_revmeta()[0].metarev.revnum
 
     def get_child_submit_format(self):
         """Return the preferred format of submissions to this branch."""
@@ -294,7 +305,7 @@ class SvnBranch(ForeignBranch):
             return revmeta, mapping
         return None, None
 
-    def check(self):
+    def check(self, refs=None):
         """See Branch.Check.
 
         Doesn't do anything for Subversion repositories at the moment (yet).
@@ -349,10 +360,10 @@ class SvnBranch(ForeignBranch):
         else:
             revnum = self.get_revnum()
 
-        svn_url = bzr_to_svn_url(self.base)
+        svn_url, readonly = bzr_to_svn_url(self.base)
         os.mkdir(to_location.encode(osutils._fs_enc))
         wc.ensure_adm(to_location.encode("utf-8"), self.repository.uuid, svn_url,
-                      bzr_to_svn_url(self.repository.base), revnum)
+                      bzr_to_svn_url(self.repository.base)[0], revnum)
         adm = wc.WorkingCopy(None, to_location.encode("utf-8"), write_lock=True)
         try:
             conn = self.repository.transport.connections.get(svn_url)
@@ -380,9 +391,9 @@ class SvnBranch(ForeignBranch):
         assert isinstance(revnum, int)
         revmeta_history = self._revision_meta_history()
         for revmeta, mapping in revmeta_history:
-            if revmeta.revnum == revnum:
+            if revmeta.metarev.revnum == revnum:
                 return revmeta.get_revision_id(mapping)
-            if revmeta.revnum < revnum:
+            if revmeta.metarev.revnum < revnum:
                 break
         raise NoSuchRevision(self, revnum)
 
@@ -403,66 +414,37 @@ class SvnBranch(ForeignBranch):
 
     def set_revision_history(self, rev_history):
         """See Branch.set_revision_history()."""
-        if (rev_history == [] or
-            not self.repository.has_revision(rev_history[-1],
-                project=self.project)):
-            raise NotImplementedError("set_revision_history can't add ghosts")
-        rev = self.repository.get_revision(rev_history[-1])
-        if rev.parent_ids:
-            base_revid = rev.parent_ids[0]
+        if rev_history == []:
+            self._set_last_revision(NULL_REVISION)
         else:
-            base_revid = NULL_REVISION
-        interrepo = InterToSvnRepository(self.repository, self.repository)
-        interrepo.push(self.get_branch_path(), self.get_config(), rev)
-        self._clear_cached_state()
+            self._set_last_revision(rev_history[-1])
 
     def set_last_revision_info(self, revno, revid):
         """See Branch.set_last_revision_info()."""
+        # FIXME: 
 
-    def _missing_revisions(self, other_repo, stop_revision, overwrite=False):
-        todo = self._mainline_missing_revisions(other_repo, stop_revision)
-        if todo is not None:
-            return todo
-        # Not possible to add cleanly onto mainline, perhaps need a replace
-        # operation
-        return self._otherline_missing_revisions(other_repo, stop_revision,
-                                                 overwrite)
-
-    def _mainline_missing_revisions(self, other_repo, stop_revision):
-        """Find the revisions missing on the mainline.
-
-        :param other: Other branch to retrieve revisions from.
-        :param stop_revision: Revision to stop fetching at.
-        """
-        missing = []
-        lastrevid = self.last_revision()
-        for revid in other_repo.iter_reverse_revision_history(stop_revision):
-            if lastrevid == revid:
-                missing.reverse()
-                return missing
-            missing.append(revid)
-        return None
-
-    def _otherline_missing_revisions(self, other_repo, stop_revision,
-                                     overwrite=False):
-        """Find the revisions missing on the mainline.
-
-        :param other: Other branch to retrieve revisions from.
-        :param stop_revision: Revision to stop fetching at.
-        :param overwrite: Whether or not the existing data should be
-            overwritten
-        """
-        missing = []
-        for revid in other_repo.iter_reverse_revision_history(stop_revision):
-            if self.repository.has_revision(revid, project=self.project):
-                missing.reverse()
-                return missing
-            missing.append(revid)
-        if not overwrite:
-            return None
+    def _set_last_revision(self, revid):
+        if revid == NULL_REVISION:
+            create_branch_with_hidden_commit(
+                self.repository,
+                self.get_branch_path(), NULL_REVISION,
+                set_metadata=True, deletefirst=True)
         else:
-            missing.reverse()
-            return missing
+            try:
+                rev = self.repository.get_revision(revid)
+            except NoSuchRevision:
+                raise NotImplementedError("set_last_revision_info can't add ghosts")
+            if rev.parent_ids:
+                base_revid = rev.parent_ids[0]
+            else:
+                base_revid = NULL_REVISION
+            interrepo = InterToSvnRepository(self.repository, self.repository)
+            base_foreign_info = interrepo._get_foreign_revision_info(base_revid,
+                self.get_branch_path())
+            interrepo.push_single_revision(self.get_branch_path(), self.get_config(), rev,
+                push_metadata=True, root_action=("replace", self.get_revnum()),
+                base_foreign_info=base_foreign_info)
+        self._clear_cached_state()
 
     def last_revision_info(self):
         """See Branch.last_revision_info()."""
@@ -495,7 +477,7 @@ class SvnBranch(ForeignBranch):
         return None
 
     def _iter_revision_meta_ancestry(self, pb=None):
-        return self.repository._iter_reverse_revmeta_mapping_ancestry(
+        return self.repository._revmeta_provider._iter_reverse_revmeta_mapping_ancestry(
             self.get_branch_path(),
             self._revnum or self.repository.get_latest_revnum(), self.mapping,
             lhs_history=self._revision_meta_history(), pb=pb)
@@ -503,7 +485,7 @@ class SvnBranch(ForeignBranch):
     def _revision_meta_history(self):
         if self._revmeta_cache is None:
             self._revmeta_cache = util.lazy_readonly_list(
-                self.repository._iter_reverse_revmeta_mapping_history(
+                self.repository._revmeta_provider._iter_reverse_revmeta_mapping_history(
                     self.get_branch_path(),
                     self._revnum or self.repository.get_latest_revnum(),
                     to_revnum=0, mapping=self.mapping))
@@ -521,7 +503,7 @@ class SvnBranch(ForeignBranch):
             if revmeta.is_hidden(mapping):
                 continue
             if count == 0:
-                assert revmeta.get_revno(mapping) == revno
+                assert revmeta.get_revno(mapping) == revno, "Expected %d, was (%r,%r) %d" % (revno, revmeta, mapping, revmeta.get_revno(mapping))
                 return revmeta.get_revision_id(mapping)
             count -= 1
         raise AssertionError
@@ -537,21 +519,33 @@ class SvnBranch(ForeignBranch):
         # Shortcut for finding the tip. This avoids expensive generation time
         # on large branches.
         last_revmeta, mapping = self.last_revmeta()
+        if last_revmeta is None:
+            return NULL_REVISION
         return last_revmeta.get_revision_id(mapping)
 
     def get_push_merged_revisions(self):
         return (self.layout.push_merged_revisions(self.project) and
                 self.get_config().get_push_merged_revisions())
 
-    def import_last_revision_info(self, source_repo, revno, revid):
+    def import_last_revision_info(self, source_repo, revno, revid, lossy=False):
         interrepo = InterToSvnRepository(source_repo, self.repository)
-        todo = self._missing_revisions(source_repo, revid, overwrite=False)
-        if todo is None:
-            raise DivergedBranches(self, None)
-        assert todo != []
-        interrepo.push_branch(todo, self.layout, self.project,
-            self.get_branch_path(), self.get_config(),
-            self.get_push_merged_revisions(), overwrite=False)
+        last_revmeta, mapping = self.last_revmeta()
+        if last_revmeta is None:
+            last_revid = NULL_REVISION
+        else:
+            last_revid = last_revmeta.get_revision_id(mapping)
+        revidmap = interrepo.push_todo(last_revid,
+            last_revmeta.metarev.get_foreign_revid(), mapping, revid, self.layout,
+            self.project, self.get_branch_path(), self.get_config(),
+            push_merged=self.get_push_merged_revisions(),
+            overwrite=False, push_metadata=not lossy, append_revisions_only=True)
+        return (revno, revidmap[revid][0])
+
+    def import_last_revision_info_and_tags(self, source, revno, revid, lossy=False):
+        (revno, revid) = self.import_last_revision_info(source.repository,
+            revno, revid, lossy=lossy)
+        self.tags.merge_to(source.tags, overwrite=False)
+        return (revno, revid)
 
     def generate_revision_history(self, revision_id, last_rev=None,
         other_branch=None):
@@ -588,9 +582,11 @@ class SvnBranch(ForeignBranch):
     def break_lock(self):
         pass
 
-    def lock_write(self):
+    def lock_write(self, token=None):
         """See Branch.lock_write()."""
         # TODO: Obtain lock on the remote server?
+        if token is not None:
+            raise TokenLockingNotSupported(self)
         if self._lock_mode:
             assert self._lock_mode == 'w'
             self._lock_count += 1
@@ -665,17 +661,16 @@ class SvnBranchFormat(BranchFormat):
         """See BranchFormat.get_format_description."""
         return 'Subversion Smart Server'
 
-    def get_format_string(self):
-        """See BranchFormat.get_format_string()."""
-        return 'Subversion Smart Server'
-
     def get_foreign_tests_branch_factory(self):
         from bzrlib.plugins.svn.tests.test_branch import ForeignTestsBranchFactory
         return ForeignTestsBranchFactory()
 
-    def initialize(self, to_bzrdir):
+    def initialize(self, to_bzrdir, name=None, repository=None):
         """See BranchFormat.initialize()."""
-        raise NotImplementedError(self.initialize)
+        from bzrlib.plugins.svn.remote import SvnRemoteAccess
+        if not isinstance(to_bzrdir, SvnRemoteAccess):
+            raise IncompatibleFormat(self, to_bzrdir._format)
+        return to_bzrdir.create_branch(name)
 
     def supports_tags(self):
         return True
@@ -687,7 +682,7 @@ class SvnBranchFormat(BranchFormat):
             return tag.DisabledTags(branch)
 
 
-class InterSvnOtherBranch(GenericInterBranch):
+class InterFromSvnBranch(GenericInterBranch):
     """InterBranch implementation that is optimized for copying from
     Subversion.
 
@@ -698,23 +693,55 @@ class InterSvnOtherBranch(GenericInterBranch):
        history browsing operations
     """
 
-    def update_revisions(self, stop_revision=None, overwrite=False,
-                         graph=None):
-        self._update_revisions(stop_revision, overwrite, graph)
+    @staticmethod
+    def _get_branch_formats_to_test():
+        from bzrlib.branch import format_registry as branch_format_registry
+        return [(SvnBranchFormat(), branch_format_registry.get_default())]
+
+    def fetch(self, stop_revision=None, fetch_tags=None, find_ghosts=False, limit=None):
+        """See InterBranch.fetch."""
+        # we fetch here so that we don't process data twice in the
+        # common case of having something to pull, and so that the
+        # check for already merged can operate on the just fetched
+        # graph, which will be cached in memory.
+        (revmeta, mapping) = self.source.last_revmeta()
+        if stop_revision is None:
+            todo = [self.source.last_revmeta()]
+        elif stop_revision == NULL_REVISION:
+            todo = []
+        else:
+            todo = [self.source.repository._get_revmeta(stop_revision)]
+        if limit is not None and len(todo) > limit:
+            # No need to fetch tags if there are already up to 'limit' revisions
+            # missing in mainline.
+            fetch_tags = False
+        if fetch_tags is None:
+            c = self.source.get_config()
+            fetch_tags = c.get_user_option_as_bool('branch.fetch_tags')
+        if fetch_tags and self.source.supports_tags():
+            tag_revmetas = self.source.tags._get_tag_dict_revmeta()
+            d = resolve_tags_svn_ancestry(self.source, tag_revmetas)
+            for name, (revmeta, mapping, revid) in d.iteritems():
+                todo.append((revmeta, mapping))
+        self._fetch_revmetas(todo, find_ghosts=find_ghosts, limit=limit)
+
+    def _fetch_revmetas(self, revmetas, find_ghosts=False, limit=None):
+        interrepo = InterFromSvnRepository(self.source.repository,
+            self.target.repository)
+        revisionfinder = interrepo.get_revision_finder()
+        for revmeta, mapping in revmetas:
+            revisionfinder.find_until(revmeta.metarev.get_foreign_revid(), mapping,
+                find_ghosts=find_ghosts)
+        interrepo.fetch(needed=revisionfinder.get_missing(limit=limit),
+            project=self.source.project, mapping=self.source.mapping)
 
     def _update_revisions(self, stop_revision=None, overwrite=False,
-            graph=None, limit=None):
-        """Like InterBranch.update_revisions, but with a few additional
-            parameters.
-
-        :param limit: Optional maximum number of revisions to fetch
-        :return: Last revision
-        """
+                         graph=None, fetch_tags=None):
+        "See InterBranch.update_revisions."""
         self.source.lock_read()
         try:
-            other_last_revision = self.source.last_revision()
             if stop_revision is None:
-                stop_revision = other_last_revision
+                stop_revision = self.source.last_revision()
                 if is_null(stop_revision):
                     # if there are no commits, we're done.
                     return self.target.last_revision_info()
@@ -722,27 +749,7 @@ class InterSvnOtherBranch(GenericInterBranch):
             # what's the current last revision, before we fetch [and
             # change it possibly]
             last_rev = self.target.last_revision()
-            # we fetch here so that we don't process data twice in the
-            # common case of having something to pull, and so that the
-            # check for already merged can operate on the just fetched
-            # graph, which will be cached in memory.
-            revisionfinder = FetchRevisionFinder(self.source.repository,
-                self.target.repository)
-            pb = ui.ui_factory.nested_progress_bar()
-            try:
-                revisionfinder.find_until(
-                    self.source.last_revmeta()[0].get_foreign_revid(),
-                    self.source.mapping,
-                    find_ghosts=False, pb=pb)
-            finally:
-                pb.finished()
-            interrepo = InterFromSvnRepository(self.source.repository,
-                self.target.repository)
-            missing = revisionfinder.get_missing(limit=limit)
-            if limit is not None and len(missing) > 0:
-                stop_revision = missing[-1][0].get_revision_id(missing[-1][1])
-            interrepo.fetch(needed=missing,
-                project=self.source.project, mapping=self.source.mapping)
+            self.fetch(stop_revision=stop_revision, fetch_tags=fetch_tags)
             # Check to see if one is an ancestor of the other
             if not overwrite:
                 if graph is None:
@@ -767,14 +774,14 @@ class InterSvnOtherBranch(GenericInterBranch):
         result.target_branch = self.target
         graph = self.target.repository.get_graph(self.source.repository)
         result.old_revno, result.old_revid = self.target.last_revision_info()
-        self.update_revisions(stop_revision, overwrite=overwrite, graph=graph)
+        self._update_revisions(stop_revision, overwrite=overwrite, graph=graph)
         # FIXME: Tags
         result.new_revno, result.new_revid = self.target.last_revision_info()
         return result
 
     def pull(self, overwrite=False, stop_revision=None,
              _hook_master=None, run_hooks=True, possible_transports=None,
-             _override_hook_target=None, local=False, limit=None):
+             _override_hook_target=None, local=False):
         """See InterBranch.pull()."""
         if local:
             raise LocalRequiresBoundBranch()
@@ -797,7 +804,7 @@ class InterSvnOtherBranch(GenericInterBranch):
                 try:
                     result.old_revmeta, _ = \
                         self.source.repository._get_revmeta(result.old_revid)
-                    tags_since_revnum = result.old_revmeta.revnum
+                    tags_since_revnum = result.old_revmeta.metarev.revnum
                 except NoSuchRevision:
                     result.old_revmeta = None
                     tags_since_revnum = None
@@ -810,7 +817,7 @@ class InterSvnOtherBranch(GenericInterBranch):
             else:
                 result.new_revmeta = None
             (result.new_revno, result.new_revid) = self._update_revisions(
-                stop_revision, overwrite, limit=limit)
+                stop_revision, overwrite)
             if self.source.supports_tags():
                 result.tag_conflicts = self.source.tags.merge_to(
                     self.target.tags, overwrite,
@@ -831,13 +838,16 @@ class InterSvnOtherBranch(GenericInterBranch):
 
     @classmethod
     def is_compatible(self, source, target):
-        return (isinstance(source, SvnBranch) and
-                not isinstance(target, SvnBranch))
+        if not isinstance(source, SvnBranch):
+            return False
+        if isinstance(target, SvnBranch):
+            return False
+        return True
 
-InterBranch.register_optimiser(InterSvnOtherBranch)
+InterBranch.register_optimiser(InterFromSvnBranch)
 
 
-class InterOtherSvnBranch(InterBranch):
+class InterToSvnBranch(InterBranch):
     """InterBranch implementation that is optimized for copying to
     Subversion.
 
@@ -845,12 +855,8 @@ class InterOtherSvnBranch(InterBranch):
 
     @staticmethod
     def _get_branch_formats_to_test():
-        return []
-
-    def update_revisions(self, stop_revision=None, overwrite=False, graph=None):
-        """See Branch.update_revisions()."""
-        self._update_revisions(stop_revision=stop_revision,
-            overwrite=overwrite, graph=graph)
+        from bzrlib.branch import format_registry as branch_format_registry
+        return [(branch_format_registry.get_default(), SvnBranchFormat())]
 
     def _target_is_empty(self, graph, revid):
         parent_revids = tuple(graph.get_parent_map([revid])[revid])
@@ -860,72 +866,40 @@ class InterOtherSvnBranch(InterBranch):
             self.target.get_branch_path(), self.target.get_revnum())[0]
         return tree_contents == {}
 
-    def _todo(self, graph, stop_revision, overwrite=False):
-        old_last_revid = self.target.last_revision()
-        if not graph.is_ancestor(old_last_revid, stop_revision):
-            if graph.is_ancestor(stop_revision, old_last_revid):
-                return []
-            if not overwrite:
-                if self._target_is_empty(graph, old_last_revid):
-                    raise PushToEmptyBranch(self.target, self.source)
-                raise DivergedBranches(self.target, self.source)
-        todo = self.target._missing_revisions(self.source.repository,
-            stop_revision, overwrite)
-        if todo is None:
-            raise DivergedBranches(self.target, self.source)
-        return todo
+    def copy_content_into(self, revision_id=None):
+        if revision_id is None:
+            revision_id = self.source.last_revision()
+        self._push(revision_id, overwrite=True, push_metadata=True)
 
-    def _get_interrepo(self, graph):
-        interrepo = InterToSvnRepository(self.source.repository,
-                self.target.repository, graph)
-        # Add the current branch path as a hint, since it's very likely we'll
-        # need it as base for new revisions that will be pushed, and
-        # push should preferably use it over other paths in the
-        # repository that contain the same data.
-        last_branch_path = self.target.get_branch_path()
-        last_revnum = self.target.repository._log.find_latest_change(
-            last_branch_path, self.target.repository.get_latest_revnum())
-        interrepo.add_path_info(self.target.last_revision(),
-            last_branch_path,
-            ((self.target.repository.uuid, last_branch_path, last_revnum), self.target.mapping))
-        return interrepo
+    def _push(self, stop_revision, overwrite, push_metadata):
+        old_last_revid = self.target.last_revision()
+        if old_last_revid == stop_revision:
+            return { old_last_revid: (old_last_revid, None) }
+        push_merged = self.target.get_push_merged_revisions()
+        interrepo = InterToSvnRepository(
+            self.source.repository, self.target.repository)
+        try:
+            return interrepo.push_branch(self.target.get_branch_path(),
+                    self.target.get_config(), self.target.last_revmeta(),
+                    stop_revision=stop_revision, overwrite=overwrite,
+                    push_metadata=push_metadata, push_merged=push_merged,
+                    layout=self.target.layout, project=self.target.project)
+        except SubversionBranchDiverged, e:
+            if self._target_is_empty(interrepo.get_graph(), e.target_revid):
+                raise PushToEmptyBranch(self.target, self.source)
+            raise DivergedBranches(self.target, self.source)
 
     def _update_revisions(self, stop_revision=None, overwrite=False,
-            graph=None, push_merged=False, override_svn_revprops=None):
+            graph=None):
         old_last_revid = self.target.last_revision()
         if stop_revision is None:
             stop_revision = ensure_null(self.source.last_revision())
-        if old_last_revid == stop_revision:
-            return old_last_revid, old_last_revid, None
-        # Request graph from other repository, since it's most likely faster
-        # than Subversion
-        graph = self.source.repository.get_graph(self.target.repository)
-        todo = self._todo(graph, stop_revision, overwrite=overwrite)
-        if todo == []:
-            return old_last_revid, old_last_revid, None
-        if push_merged is None:
-            push_merged = self.target.get_push_merged_revisions()
-        if (self.target.mapping.supports_hidden and
-            self.target.repository.has_revision(stop_revision)):
-            # Revision is already present in the repository, so just
-            # copy from there.
-            new_last_revid, new_foreign_info = \
-                create_branch_with_hidden_commit(self.target.repository,
-                    self.target.get_branch_path(), stop_revision,
-                    set_metadata=True, deletefirst=True)
-        else:
-            interrepo = self._get_interrepo(graph)
-            assert todo != []
-            (count, (new_last_revid, new_foreign_info)) = interrepo.push_branch(
-                todo, self.target.layout, self.target.project,
-                self.target.get_branch_path(), self.target.get_config(),
-                push_merged, overwrite=overwrite)
+        self._push(stop_revision, overwrite=overwrite, push_metadata=True)
         self.target._clear_cached_state()
-        assert isinstance(new_last_revid, str)
         assert isinstance(old_last_revid, str)
-        return (old_last_revid, new_last_revid, new_foreign_info)
+        return (old_last_revid, self.target.last_revision())
 
-    def _update_revisions_lossy(self, stop_revision=None):
+    def _update_revisions_lossy(self, stop_revision=None, overwrite=False):
         """Push derivatives of the revisions missing from target from source
         into target.
 
@@ -934,82 +908,67 @@ class InterOtherSvnBranch(InterBranch):
         :param stop_revision: If not None, stop at this revision.
         :return: Map of old revids to new revids.
         """
-        self.source.lock_write()
-        try:
-            if stop_revision is None:
-                stop_revision = ensure_null(self.source.last_revision())
-            graph = self.source.repository.get_graph(self.target.repository)
-            todo = self._todo(graph, stop_revision)
-            if todo == []:
-                return { self.source.last_revision(): self.source.last_revision() }
-            revid_map = {}
-            target_branch_path = self.target.get_branch_path()
-            target_config = self.target.get_config()
-            # Request graph from other repository, since it's most likely faster
-            # than Subversion
-            interrepo = self._get_interrepo(graph)
-            pb = ui.ui_factory.nested_progress_bar()
-            try:
-                # FIXME: Call create_branch_with_hidden_commit if the revision
-                # is already present in the target repository ?
-                for rev in self.source.repository.get_revisions(todo):
-                    pb.update("pushing revisions", todo.index(rev.revision_id),
-                              len(todo))
-                    if len(rev.parent_ids) == 0:
-                        base_revid = NULL_REVISION
-                    elif rev.parent_ids[0] in revid_map:
-                        base_revid = revid_map[rev.parent_ids[0]]
-                    else:
-                        base_revid = rev.parent_ids[0]
-                    revid_map[rev.revision_id], _ = interrepo.push(
-                        target_branch_path, target_config, rev,
-                        push_metadata=False, base_revid=base_revid)
-            finally:
-                pb.finished()
-            interrepo = InterFromSvnRepository(self.target.repository,
-                                               self.source.repository)
-            interrepo.fetch(revision_id=revid_map[rev.revision_id],
-                mapping=self.target.mapping, project=self.target.project)
-            self.target._clear_cached_state()
-            assert stop_revision in revid_map
-            assert len(revid_map.keys()) > 0
-            return revid_map
-        finally:
-            self.source.unlock()
+        if stop_revision is None:
+            stop_revision = ensure_null(self.source.last_revision())
+        revid_map = self._push(stop_revision, overwrite=overwrite,
+            push_metadata=False)
+        reverse_inter = InterFromSvnBranch(self.target, self.source)
+        self.target._clear_cached_state()
+        reverse_inter.fetch(stop_revision=revid_map[stop_revision][0])
+        assert stop_revision in revid_map
+        assert len(revid_map.keys()) > 0
+        return dict([(k, v[0]) for (k, v) in revid_map.iteritems()])
 
     def lossy_push(self, stop_revision=None):
         """See InterBranch.lossy_push()."""
-        result = SubversionTargetBranchPushResult()
-        result.target_branch = self.target
-        result.master_branch = None
-        result.source_branch = self.source
-        self.source.lock_write()
-        try:
-            result.old_revid = self.target.last_revision()
-            result.revidmap = self._update_revisions_lossy(stop_revision)
-            result.new_revid = self.target.last_revision()
-            # FIXME: Tags ?
-            return result
-        finally:
-            self.source.unlock()
+        # For compatibility with bzr < 2.4
+        return self.push(lossy=True, stop_revision=stop_revision)
+
+    def fetch(self, stop_revision=None, fetch_tags=None, find_ghosts=False,
+            limit=None):
+        """Fetch into a subversion repository."""
+        # For the moment, this method is disabled. Merge uses it to fetch
+        # revisions before doing its work, but then fails later on.
+        # Having this method fail means the user won't end up with
+        # spurious revisions in their repository for an operation that's
+        # not going to work anyway.
+        raise NotImplementedError(self.fetch)
+        # FIXME: Handle limit
+        # FIXME: Handle fetch_tags
+        # FIXME: Handle find_ghosts
+        interrepo = InterToSvnRepository(
+            self.source.repository, self.target.repository)
+        interrepo.fetch(revision_id=stop_revision)
 
     def update_tags(self, overwrite=False):
         return self.source.tags.merge_to(self.target.tags, overwrite)
 
+    def _basic_push(self, overwrite=False, stop_revision=None):
+        # Wrapper for the benefit of GenericInterBranch, which
+        # calls it for bound branches
+        return self.push(overwrite=overwrite, stop_revision=stop_revision)
+
     def push(self, overwrite=False, stop_revision=None,
-            _push_merged=None, _override_svn_revprops=None,
-            _override_hook_source_branch=None):
+            lossy=False, _override_hook_source_branch=None):
         """See InterBranch.push()."""
+        if lossy and isinstance(self.source, SvnBranch):
+            raise LossyPushToSameVCS(self.source, self.target)
         result = SubversionTargetBranchPushResult()
         result.target_branch = self.target
         result.master_branch = None
         result.source_branch = self.source
-        self.source.lock_read()
+        if lossy:
+            self.source.lock_write()
+        else:
+            self.source.lock_read()
         try:
-            (result.old_revid, result.new_revid, result.new_foreign_info) = \
-                self._update_revisions(stop_revision, overwrite,
-                    push_merged=_push_merged,
-                    override_svn_revprops=_override_svn_revprops)
+            if lossy:
+                result.old_revid = self.target.last_revision()
+                result.revidmap = self._update_revisions_lossy(stop_revision)
+                result.new_revid = self.target.last_revision()
+            else:
+                (result.old_revid, result.new_revid) = \
+                    self._update_revisions(stop_revision, overwrite)
             result.tag_conflicts = self.update_tags(overwrite)
             return result
         finally:
@@ -1017,8 +976,7 @@ class InterOtherSvnBranch(InterBranch):
 
     def pull(self, overwrite=False, stop_revision=None,
              _hook_master=None, run_hooks=True, possible_transports=None,
-             _push_merged=None, _override_svn_revprops=None, local=False,
-             limit=None):
+             local=False):
         """See InterBranch.pull()."""
         if local:
             raise LocalRequiresBoundBranch()
@@ -1028,10 +986,8 @@ class InterOtherSvnBranch(InterBranch):
         result.target_branch = self.target
         self.source.lock_read()
         try:
-            (result.old_revid, result.new_revid, result.new_foreign_info) = \
-                self._update_revisions(stop_revision, overwrite,
-                    push_merged=_push_merged,
-                    override_svn_revprops=_override_svn_revprops)
+            (result.old_revid, result.new_revid) = \
+                self._update_revisions(stop_revision, overwrite)
             result.tag_conflicts = self.update_tags(overwrite)
             return result
         finally:
@@ -1039,7 +995,9 @@ class InterOtherSvnBranch(InterBranch):
 
     @classmethod
     def is_compatible(self, source, target):
-        return isinstance(target, SvnBranch)
+        if not isinstance(target, SvnBranch):
+            return False
+        return True
 
 
-InterBranch.register_optimiser(InterOtherSvnBranch)
+InterBranch.register_optimiser(InterToSvnBranch)
