@@ -72,7 +72,7 @@ def find_prev_location(paths, branch_path, revnum):
 
             revnum = paths[p][2]
             branch_path = paths[p][1].encode("utf-8") + branch_path[len(p):]
-            return (branch_path, revnum)
+            return (branch_path.lstrip("/"), revnum)
 
     return (branch_path, revnum)
 
@@ -163,3 +163,24 @@ def changes_outside_branch_path(branch_path, paths):
         if not path_is_child(branch_path, p):
             return True
     return False
+
+
+def under_prefixes(path, prefixes):
+    """Check if path is under one of prefixes."""
+    if prefixes is None or "" in prefixes:
+        return True
+    return any([x for x in prefixes if path_is_child(x, path)])
+
+
+def common_prefix(paths):
+    prefix = ""
+    if paths == []:
+        return ""
+    # Find a common prefix
+    parts = paths[0].split("/")
+    for i in range(len(parts)+1):
+        for j in paths:
+            if j.split("/")[:i] != parts[:i]:
+                return prefix
+        prefix = "/".join(parts[:i])
+    return prefix
