@@ -75,8 +75,9 @@ class cmd_svn_import(Command):
                               'Default: auto.'),
                      Option('keep',
                          help="Don't delete branches removed in Subversion."),
-                     Option('incremental',
-                         help="Import revisions incrementally."),
+                     Option('restore',
+                         help="Restore branches that were removed but have "
+                              "not been changed since the last import."),
                      Option('prefix', type=str,
                          hidden=True,
                          help='Only consider branches of which path starts '
@@ -87,7 +88,7 @@ class cmd_svn_import(Command):
 
     def run(self, from_location, to_location=None, format=None, trees=False,
             standalone=False, layout=None, all=False, prefix=None, keep=False,
-            incremental=False, until=None):
+            restore=False, until=None):
         from bzrlib import (
             osutils,
             trace,
@@ -150,7 +151,6 @@ class cmd_svn_import(Command):
 
         from_repos.lock_read()
         try:
-
             if prefix is not None:
                 if layout is None:
                     overall_layout = from_repos.get_guessed_layout()
@@ -179,18 +179,18 @@ class cmd_svn_import(Command):
                     return False
                 return True
 
-            trace.note("Using repository layout: %s" % (layout or from_repos.get_layout(),))
+            trace.note("Using repository layout: %s",
+                       layout or from_repos.get_layout())
             convert_repository(from_repos, to_location, layout,
-                               not standalone, trees, all,
-                               format=format,
-                               filter_branch=filter_branch,
-                               keep=keep, incremental=incremental,
-                               to_revnum=to_revnum, prefix=prefix)
+                not standalone, trees, all, format=format,
+                filter_branch=filter_branch, keep=keep,
+                incremental=not restore, to_revnum=to_revnum, prefix=prefix)
 
             if tmp_repos is not None:
                 osutils.rmtree(tmp_repos)
             if not trees:
-                trace.note("Use 'bzr checkout' to create a working tree in the newly created branches.")
+                trace.note("Use 'bzr checkout' to create a working tree in "
+                           "the newly created branches.")
         finally:
             from_repos.unlock()
 
@@ -211,15 +211,17 @@ class cmd_svn_layout(Command):
             )
         from bzrlib.branch import Branch
         from bzrlib.repository import Repository
+        from bzrlib.plugins.svn import errors as bzrsvn_errors
 
         try:
             branch, _ = Branch.open_containing(path)
             repos = branch.repository
-        except errors.NotBranchError:
+        except (errors.NotBranchError, errors.NoColocatedBranchSupport):
             repos = Repository.open(path)
             branch = None
         if getattr(repos, "uuid", None) is None:
-            raise errors.BzrCommandError("Not a Subversion branch or repository.")
+            raise errors.BzrCommandError(
+                "Not a Subversion branch or repository.")
         layout = repos.get_layout()
         self.outf.write("Repository root: %s\n" % repos.base)
         self.outf.write("Layout: %s\n" % str(layout))
@@ -227,16 +229,71 @@ class cmd_svn_layout(Command):
             self.outf.write("Branch path: %s\n" % branch.get_branch_path())
             if branch.project:
                 self.outf.write("Project: %s\n" % branch.project)
-            test_tag_path = layout.get_tag_path("test", branch.project)
-            if test_tag_path:
-                self.outf.write("Tag container directory: %s\n" %
-                        urlutils.dirname(test_tag_path))
-            test_branch_path = layout.get_branch_path("test", branch.project)
-            if test_branch_path:
-                self.outf.write("Branch container directory: %s\n" %
-                        urlutils.dirname(test_branch_path))
+            try:
+                test_tag_path = layout.get_tag_path("test", branch.project)
+            except bzrsvn_errors.NoLayoutTagSetSupport:
+                self.outf.write("No tag support\n")
+            else:
+                if test_tag_path:
+                    self.outf.write("Tag container directory: %s\n" %
+                            urlutils.dirname(test_tag_path))
+            try:
+                test_branch_path = layout.get_branch_path("test",
+                    branch.project)
+            except bzrsvn_errors.NoCustomBranchPaths:
+                self.outf.write("No custom branch support\n")
+            else:
+                if test_branch_path:
+                    self.outf.write("Branch container directory: %s\n" %
+                            urlutils.dirname(test_branch_path))
             self.outf.write("Push merged revisions: %s\n" %
                     branch.get_push_merged_revisions())
 
 
+class cmd_svn_branches(Command):
+    """Print the branches and tags in a repository.
 
+    """
+
+    takes_args = ["location"]
+
+    takes_options = [
+        Option('layout', type=get_layout,
+               help='Repository layout (none, trunk, etc). '
+                    'Default: auto.')]
+
+    hidden = True
+
+    def run(self, location, layout=None):
+        from bzrlib import errors
+        from bzrlib.bzrdir import BzrDir
+        from bzrlib.plugins.svn.remote import SvnRemoteAccess
+        from bzrlib.plugins.svn.workingtree import SvnCheckout
+
+        dir = BzrDir.open(location)
+
+        if not (isinstance(dir, SvnRemoteAccess) or
+                isinstance(dir, SvnCheckout)):
+            raise errors.BzrCommandError(
+                "Source repository is not a Subversion repository.")
+
+        try:
+            repository = dir.open_repository()
+        except errors.NoRepositoryPresent, e:
+            repository = dir.find_repository(_ignore_branch_path=True)
+            assert dir.root_transport.base.startswith(repository.base)
+            prefix = dir.root_transport.base[len(repository.base):].strip("/")
+            prefix = prefix.encode("utf-8")
+        else:
+            prefix = None
+
+        revnum = repository.get_latest_revnum()
+        if layout is None:
+            layout = repository.get_guessed_layout()
+
+        self.outf.write("Branches:\n")
+        for (project, path, name, has_props, revnum) in layout.get_branches(repository, revnum, prefix):
+            self.outf.write("%s (%s)\n" % (path, name))
+        self.outf.write("Tags:\n")
+        for (project, path, name, has_props, revnum) in layout.get_tags(repository, revnum, prefix):
+            self.outf.write("%s (%s)\n" % (path, name))
