@@ -31,7 +31,6 @@ from bzrlib import (
     )
 from bzrlib.config import (
     ConfigObj,
-    IniBasedConfig,
     config_dir,
     ensure_config_dir_exists,
     GlobalConfig,
@@ -45,6 +44,8 @@ from bzrlib.errors import (
     BzrError,
     )
 
+from bzrlib.config import LockableConfig
+
 
 def as_bool(str):
     """Parse a string as a boolean.
@@ -69,24 +70,20 @@ def as_bool(str):
 # Data stored includes default branching scheme and locations the repository
 # was seen at.
 
-def subversion_config_filename():
-    """Return per-user configuration ini file filename."""
-    return osutils.pathjoin(config_dir(), 'subversion.conf')
 
 
-config_base_class = getattr(bzrlib.config, "LockableConfig", IniBasedConfig)
-
-
-class SubversionUUIDConfig(config_base_class):
+class SubversionUUIDConfig(LockableConfig):
     """UUID-based Subversion configuration."""
 
     def __init__(self, uuid):
-        name_generator = subversion_config_filename
-        super(SubversionUUIDConfig, self).__init__(name_generator)
+        super(SubversionUUIDConfig, self).__init__(self._get_filename())
         self.uuid = uuid
         if not self.uuid in self._get_parser():
             self._get_parser()[self.uuid] = {}
 
+    def _get_filename(self):
+        return osutils.pathjoin(config_dir(), 'subversion.conf')
+
     def __getitem__(self, name):
         return self._get_parser()[self.uuid][name]
 
@@ -248,14 +245,6 @@ class SvnRepositoryConfig(Config):
             return BranchingScheme.find_scheme(schemename.encode('ascii'))
         return None
 
-    def get_supports_change_revprop(self):
-        """Check whether or not the repository supports changing existing
-        revision properties."""
-        try:
-            return self.get_bool("supports-change-revprop")
-        except KeyError:
-            return None
-
     def get_use_cache(self):
         try:
             if self.get_bool("use-cache"):
@@ -301,6 +290,11 @@ class SvnRepositoryConfig(Config):
         locations.add(location.rstrip("/"))
         self.set_user_option('locations', ";".join(list(locations)))
 
+    def get_default_stack_on(self):
+        return None
+
+    def set_default_stack_on(self, value):
+        raise BzrError("Cannot set configuration")
 
 
 class BranchConfig(Config):
@@ -372,6 +366,15 @@ class BranchConfig(Config):
                 return value
         return None
 
+    def get_allow_metadata_in_fileprops(self):
+        """Check whether file properties may be used to store bzr-svn metadata.
+
+        """
+        try:
+            return self.get_bool("allow_metadata_in_file_properties")
+        except KeyError:
+            return False
+
     def get_append_revisions_only(self, default=None):
         """Check whether it is possible to remove revisions from the mainline.
         """
@@ -417,6 +420,9 @@ class BranchConfig(Config):
         except KeyError:
             return None
 
+    def has_explicit_nickname(self):
+        return False
+
 
 class PropertyConfig(object):
     """ConfigObj-like class that looks at Subversion file ids."""
@@ -456,7 +462,9 @@ class SubversionBuildPackageConfig(object):
             self.wt_layout_path = os.path.join(tree.abspath("."), ".svn", "svn-layout")
             self.option_source = ConfigObj(self.wt_layout_path, encoding="utf-8")
         elif tree.has_filename("debian/svn-layout"):
-            self.option_source = ConfigObj(tree.get_file(tree.path2id("debian/svn-layout"), "debian/svn-layout"), encoding="utf-8")
+            file_id = tree.path2id("debian/svn-layout")
+            layout_file = tree.get_file(file_id, "debian/svn-layout")
+            self.option_source = ConfigObj(layout_file, encoding="utf-8")
         elif isinstance(tree, SubversionTree) and tree.has_filename("debian"):
             self.option_source = PropertyConfig(tree, "debian", "svn-bp:")
         else:
