@@ -96,14 +96,20 @@ class NotDumpFile(BzrError):
         self.dumpfile = dumpfile
 
 
-def load_dumpfile(dumpfile, outputdir):
+def load_dumpfile(dumpfile, outputdir, feedback_stream=None):
     """Load a Subversion dump file.
 
     :param dumpfile: Path to dump file.
     :param outputdir: Directory in which Subversion repository should be
         created.
     """
-    from cStringIO import StringIO
+    if feedback_stream is None:
+        class NullStream(object):
+
+            def read(self): return
+            def write(self, data): return
+            def close(self): return
+        feedback_stream = NullStream()
     r = repos.create(outputdir)
     if dumpfile.endswith(".gz"):
         import gzip
@@ -115,7 +121,7 @@ def load_dumpfile(dumpfile, outputdir):
         file = open(dumpfile)
     try:
         try:
-            r.load_fs(file, StringIO(), repos.LOAD_UUID_DEFAULT)
+            r.load_fs(file, feedback_stream, repos.LOAD_UUID_DEFAULT)
         except SubversionException, (_, num):
             if num == ERR_STREAM_MALFORMED_DATA:
                 raise NotDumpFile(dumpfile)
@@ -126,6 +132,9 @@ def load_dumpfile(dumpfile, outputdir):
 
 
 def contains_parent_path(s, p):
+    """See if one of the parent paths of p exists in s.
+
+    """
     while p != "":
         if p in s:
             return p
@@ -175,7 +184,8 @@ class RepositoryConverter(object):
                 if not layout.is_branch("") and not target_repos.is_shared():
                     raise BzrError("Repository %r is not shared." % target_repos)
             except NoRepositoryPresent:
-                target_repos = self.get_dir(prefix, prefix).create_repository(shared=True)
+                target_dir = self.get_dir(prefix, prefix)
+                target_repos = target_dir.create_repository(shared=True)
                 target_repos_is_empty = True
             target_repos.set_make_working_trees(working_trees)
         else:
@@ -197,8 +207,8 @@ class RepositoryConverter(object):
             existing_branches = {}
             deleted = set()
             it = source_repos._revmeta_provider.iter_all_changes(layout,
-                    mapping.is_branch_or_tag, to_revnum, from_revnum,
-                    prefix=prefix)
+                mapping.is_branch_or_tag, to_revnum, from_revnum,
+                prefix=prefix)
             if create_shared_repo:
                 revfinder = FetchRevisionFinder(source_repos, target_repos,
                                                 target_repos_is_empty)
@@ -213,14 +223,14 @@ class RepositoryConverter(object):
             try:
                 for kind, item in it:
                     if kind == "revision":
-                        pb.update("finding branches", to_revnum-item.revnum,
+                        pb.update("finding branches", to_revnum-item.metarev.revnum,
                                   to_revnum-from_revnum)
-                        if (not item.branch_path in existing_branches and
-                            layout.is_branch(item.branch_path) and
-                            not contains_parent_path(deleted, item.branch_path)):
-                            existing_branches[item.branch_path] = SvnBranch(
-                                source_repos, item.branch_path,
-                                revnum=item.revnum, _skip_check=True,
+                        if (not item.metarev.branch_path in existing_branches and
+                            layout.is_branch(item.metarev.branch_path) and
+                            not contains_parent_path(deleted, item.metarev.branch_path)):
+                            existing_branches[item.metarev.branch_path] = SvnBranch(
+                                source_repos, None, item.metarev.branch_path,
+                                revnum=item.metarev.revnum, _skip_check=True,
                                 mapping=mapping)
                             if heads is not None:
                                 heads.add(item)
@@ -258,11 +268,11 @@ class RepositoryConverter(object):
                               revfinder, mapping, heads):
         def needs_manual_check(revmeta):
             if (prefix is not None and
-                not changes.path_is_child(prefix, revmeta.branch_path)):
+                not changes.path_is_child(prefix, revmeta.metarev.branch_path)):
                 # Parent branch path is outside of prefix; we need to
                 # check manually
                 return True
-            if revmeta.revnum < from_revnum:
+            if revmeta.metarev.revnum < from_revnum:
                 return True
             return False
 
@@ -271,11 +281,12 @@ class RepositoryConverter(object):
         try:
             pb.update("checking revisions to fetch", 0,
                       len(revmetas))
-            revfinder.find_iter_revisions(revmetas, mapping, needs_manual_check,
-                                          pb=pb, heads=heads)
+            revfinder.find_iter_revisions(revmetas, mapping,
+                needs_manual_check, pb=pb, heads=heads)
         finally:
             pb.finished()
-        inter.fetch(needed=revfinder.get_missing())
+        missing = revfinder.get_missing()
+        inter.fetch(needed=missing)
 
     def _create_branches(self, existing_branches, prefix, shared,
                          working_trees):
