@@ -20,17 +20,22 @@
 import subvertpy
 import urllib
 
+from bzrlib import (
+    trace,
+    )
 import bzrlib.errors
 from bzrlib.errors import (
     BzrError,
     ConnectionError,
     ConnectionReset,
+    DependencyNotPresent,
     DivergedBranches,
     InvalidRevisionSpec,
     LockActive,
     PermissionDenied,
     NoRepositoryPresent,
     NoSuchRevision,
+    TagsNotSupported,
     TipChangeRejected,
     TransportError,
     UnexpectedEndOfContainerError,
@@ -38,11 +43,6 @@ from bzrlib.errors import (
     )
 
 
-ERR_RA_DAV_PROPPATCH_FAILED = getattr(subvertpy, "ERR_RA_DAV_PROPPATCH_FAILED", 175008)
-ERR_APR_OS_START_EAIERR = getattr(subvertpy, "ERR_APR_OS_START_EAIERR", 670000)
-ERR_APR_OS_ERRSPACE_SIZE = getattr(subvertpy, "ERR_APR_OS_ERRSPACE_SIZE", 50000)
-
-
 class InvalidBzrSvnRevision(NoSuchRevision):
     _fmt = """Revision id %(revid)s was added incorrectly"""
 
@@ -69,7 +69,7 @@ class NoSvnRepositoryPresent(NoRepositoryPresent):
 
 
 class ChangesRootLHSHistory(BzrError):
-    _fmt = """changing lhs branch history not possible on repository root"""
+    _fmt = """Changing lhs branch history not possible on repository root"""
 
 
 class MissingPrefix(BzrError):
@@ -118,15 +118,15 @@ def convert_error(err):
         return TransportError("Malformed data", msg)
     elif num == subvertpy.ERR_RA_NOT_IMPLEMENTED:
         return NotImplementedError("Function not implemented in remote server")
-    elif num == subvertpy.ERR_UNKNOWN_HOSTNAME:
-        return ConnectionError(msg=msg)
     elif num == subvertpy.ERR_RA_DAV_REQUEST_FAILED:
         return DavRequestFailed(msg)
     elif num == subvertpy.ERR_REPOS_HOOK_FAILURE:
         return TipChangeRejected(msg)
-    if num == ERR_RA_DAV_PROPPATCH_FAILED:
+    if num == subvertpy.ERR_RA_DAV_PROPPATCH_FAILED:
         return PropertyChangeFailed(msg)
-    if num > ERR_APR_OS_START_EAIERR and num < ERR_APR_OS_START_EAIERR + ERR_APR_OS_ERRSPACE_SIZE:
+    if (num > subvertpy.ERR_APR_OS_START_EAIERR and
+        num < subvertpy.ERR_APR_OS_START_EAIERR + subvertpy.ERR_CATEGORY_SIZE):
+        # Newer versions of subvertpy (>= 0.7.6) do this for us.
         return ConnectionError(msg=msg)
     else:
         return err
@@ -139,8 +139,14 @@ def convert_svn_error(unbound):
     def convert(*args, **kwargs):
         try:
             return unbound(*args, **kwargs)
-        except subvertpy.SubversionException, e:
-            raise convert_error(e)
+        except subvertpy.SubversionException, svn_err:
+            mapped_err = convert_error(svn_err)
+            if svn_err is mapped_err:
+                # Bare 'raise' preserves the original traceback, whereas
+                # 'raise e' would not.
+                raise
+            else:
+                raise mapped_err
 
     convert.__doc__ = unbound.__doc__
     convert.__name__ = unbound.__name__
@@ -268,6 +274,11 @@ class PropertyChangeFailed(BzrError):
         BzrError.__init__(self, msg=msg)
 
 
+class RequiresMetadataInFileProps(BzrError):
+
+    _fmt = """This operation requires storing bzr-svn metadata in Subversion file properties. These file properties may cause spurious conflicts for other Subversion users during merges. To allow this, set `allow_metadata_in_file_properties = True` in your configuration and try again."""
+
+
 class TextChecksumMismatch(VersionedFileInvalidChecksum):
 
     _fmt = """checksum mismatch: %(expected_checksum)r != %(actual_checksum)r in %(path)s:%(revnum)d"""
@@ -278,3 +289,49 @@ class TextChecksumMismatch(VersionedFileInvalidChecksum):
         self.path = path
         self.revnum = revnum
 
+
+class SubversionBranchDiverged(DivergedBranches):
+
+    _fmt = "Subversion branch at %(branch_path)s has diverged from %(source_repo)r."
+
+    def __init__(self, source_repo, source_revid, target_repo, branch_path, target_revid):
+        self.branch_path = branch_path
+        self.target_repo = target_repo
+        self.source_repo = source_repo
+        self.source_revid = source_revid
+        self.target_revid = target_revid
+
+
+class NoLayoutTagSetSupport(TagsNotSupported):
+
+    _fmt = "Creating tags is not possible with the current layout %(layout)r%(extra)s"
+
+    def __init__(self, layout, extra=None):
+        self.layout = layout
+        if extra is None:
+            self.extra = ""
+        else:
+            self.extra = ": %s" % extra
+
+
+class IncompleteRepositoryHistory(BzrError):
+
+    _fmt = "Unable to fetch revision info; %(msg)s"
+
+    def __init__(self, msg):
+        self.msg = msg
+
+
+_reuse_uuids_warned = set()
+def warn_uuid_reuse(uuid, location):
+    """Warn that a UUID is being reused for different repositories."""
+    global _reuse_uuids_warned
+    if uuid in _reuse_uuids_warned:
+        return
+    trace.warning("Repository with UUID %s at %s contains fewer revisions "
+         "than cache. This either means that this repository contains an out "
+         "of date mirror of another repository (harmless), or that the UUID "
+         "is being used for two different Subversion repositories ("
+         "potential repository corruption).",
+         uuid, location)
+    _reuse_uuids_warned.add(uuid)
