@@ -16,9 +16,11 @@
 """Repository layouts."""
 
 import subvertpy
-from subvertpy.ra import DIRENT_KIND, DIRENT_HAS_PROPS
-
-ERR_RA_DAV_FORBIDDEN = getattr(subvertpy, "ERR_RA_DAV_FORBIDDEN", 175013)
+from subvertpy.ra import (
+    DIRENT_KIND,
+    DIRENT_HAS_PROPS,
+    DIRENT_CREATED_REV,
+    )
 
 from bzrlib import (
     registry,
@@ -27,14 +29,14 @@ from bzrlib import (
     )
 from bzrlib.trace import mutter
 
-from bzrlib.plugins.svn.errors import NotSvnBranchPath, NoCustomBranchPaths
+from bzrlib.plugins.svn.errors import (
+    NotSvnBranchPath,
+    NoCustomBranchPaths,
+    )
 
 class RepositoryLayout(object):
     """Describes a repository layout."""
 
-    def __init__(self):
-        pass
-
     def get_project_prefixes(self, project):
         return [project]
 
@@ -45,7 +47,8 @@ class RepositoryLayout(object):
         """Return the path at which the tag with specified name should be found.
 
         :param name: Name of the tag.
-        :param project: Optional name of the project the tag is for. Can include slashes.
+        :param project: Optional string identifier for the project the tag is
+            for. Can include slashes.
         :return: Path of the tag.
         """
         raise NotImplementedError
@@ -58,7 +61,7 @@ class RepositoryLayout(object):
         raise NotImplementedError
 
     def push_merged_revisions(self, project=""):
-        """Determine whether or not right hand side (merged) revisions should be pushed.
+        """Determine whether or not right hand side revisions should be pushed.
 
         Defaults to False.
 
@@ -67,10 +70,12 @@ class RepositoryLayout(object):
         return False
 
     def get_branch_path(self, name, project=""):
-        """Return the path at which the branch with specified name should be found.
+        """Return the path at which the branch with specified name should be
+        found.
 
-        :param name: Name of the branch.
-        :param project: Optional name of the project the branch is for. Can include slashes.
+        :param name: Name of the branch, None for the default branch.
+        :param project: Optional name of the project the branch is for. Can
+            include slashes.
         :return: Path of the branch.
         """
         raise NoCustomBranchPaths(self)
@@ -124,19 +129,20 @@ class RepositoryLayout(object):
         return self.is_branch(path, project) or self.is_tag(path, project)
 
     def is_branch_or_tag_parent(self, path, project=None):
-        return self.is_branch_parent(path, project) or self.is_tag_parent(path, project)
+        return (self.is_branch_parent(path, project) or
+                self.is_tag_parent(path, project))
 
     def get_branches(self, repository, revnum, project="", pb=None):
         """Retrieve a list of paths that refer to branches in a specific revision.
 
-        :result: Iterator over tuples with (project, branch path)
+        :return: Iterator over tuples with (project, branch path, branch name, has_props, revnum)
         """
         raise NotImplementedError(self.get_branches)
 
     def get_tags(self, repository, revnum, project="", pb=None):
         """Retrieve a list of paths that refer to tags in a specific revision.
 
-        :result: Iterator over tuples with (project, branch path)
+        :return: Iterator over tuples with (project, branch path, branch name, has_props, revnum)
         """
         raise NotImplementedError(self.get_tags)
 
@@ -152,50 +158,82 @@ def wildcard_matches(path, pattern):
     return True
 
 
-def expand_branch_pattern(begin, todo, check_path, get_children, project=None):
+class BranchPatternExpander(object):
     """Find the paths in the repository that match the expected branch pattern.
-
-    :param begin: List of path elements currently opened.
-    :param todo: List of path elements to still evaluate (including wildcards)
-    :param check_path: Function for checking a path exists
-    :param get_children: Function for retrieving the children of a path
     """
-    mutter('expand branches: %r, %r', begin, todo)
-    path = "/".join(begin)
-    if (project is not None and
-        not project.startswith(path) and
-        not path.startswith(project)):
-        return []
-    # If all elements have already been handled, just check the path exists
-    if len(todo) == 0:
-        if check_path(path):
-            return [(path, None)]
-        else:
+
+    def __init__(self, transport, revnum, project=None):
+        self.transport = transport
+        self.revnum = revnum
+        self.project = project
+
+    def get_latest_change(self, path):
+        try:
+            return self.transport.get_dir(path, self.revnum, 0)[1]
+        except subvertpy.SubversionException, (msg, num):
+            if num in (subvertpy.ERR_FS_NOT_DIRECTORY,
+                       subvertpy.ERR_FS_NOT_FOUND,
+                       subvertpy.ERR_RA_DAV_PATH_NOT_FOUND,
+                       subvertpy.ERR_RA_DAV_FORBIDDEN):
+                return None
+            raise
+
+    def get_children(self, path):
+        try:
+            assert not path.startswith("/")
+            dirents = self.transport.get_dir(path, self.revnum,
+                DIRENT_KIND|DIRENT_HAS_PROPS|DIRENT_CREATED_REV)[0]
+        except subvertpy.SubversionException, (msg, num):
+            if num in (subvertpy.ERR_FS_NOT_DIRECTORY,
+                       subvertpy.ERR_FS_NOT_FOUND,
+                       subvertpy.ERR_RA_DAV_PATH_NOT_FOUND,
+                       subvertpy.ERR_RA_DAV_FORBIDDEN):
+                return None
+            raise
+        return [(d, dirents[d]['has_props'], int(dirents[d]['created_rev']))
+            for d in dirents if dirents[d]['kind'] == subvertpy.NODE_DIR]
+
+    def expand(self, begin, todo):
+        """Expand
+
+        :param begin: List of path elements currently opened.
+        :param todo: List of path elements to still evaluate (including wildcards)
+        """
+        mutter('expand branches: %r, %r', begin, todo)
+        path = "/".join(begin)
+        if (self.project is not None and
+            not self.project.startswith(path) and
+            not path.startswith(self.project)):
             return []
-    # Not a wildcard? Just expand next bits
-    if todo[0] != "*":
-        return expand_branch_pattern(begin+[todo[0]], todo[1:], check_path,
-                                     get_children, project)
-    children = get_children(path)
-    if children is None:
-        return []
-    ret = []
-    pb = ui.ui_factory.nested_progress_bar()
-    try:
-        for idx, (c, has_props) in enumerate(children):
-            pb.update("browsing branches", idx, len(children))
-            if len(todo) == 1:
-                # Last path element, so return directly
-                ret.append(("/".join(begin+[c]), has_props))
+        # If all elements have already been handled, just check the path exists
+        if len(todo) == 0:
+            revnum = self.get_latest_change(path)
+            if revnum is not None:
+                return [(path, None, int(revnum))]
             else:
-                ret += expand_branch_pattern(begin+[c], todo[1:], check_path,
-                                             get_children, project)
-    finally:
-        pb.finished()
-    return ret
+                return []
+        # Not a wildcard? Just expand next bits
+        if todo[0] != "*":
+            return self.expand(begin+[todo[0]], todo[1:])
+        children = self.get_children(path)
+        if children is None:
+            return []
+        ret = []
+        pb = ui.ui_factory.nested_progress_bar()
+        try:
+            for idx, (c, has_props, revnum) in enumerate(children):
+                pb.update("browsing branches", idx, len(children))
+                if len(todo) == 1:
+                    # Last path element, so return directly
+                    ret.append(("/".join(begin+[c]), has_props, revnum))
+                else:
+                    ret += self.expand(begin+[c], todo[1:])
+        finally:
+            pb.finished()
+        return ret
 
 
-def get_root_paths(repository, itemlist, revnum, verify_fn, project=None, pb=None):
+def get_root_paths(repository, itemlist, revnum, verify_fn, project=None):
     """Find all the paths in the repository matching a list of items.
 
     :param repository: Repository to search in.
@@ -204,31 +242,21 @@ def get_root_paths(repository, itemlist, revnum, verify_fn, project=None, pb=Non
     :param verify_fn: Function that checks if a path is acceptable.
     :param project: Optional project branch/tag should be in.
     :param pb: Optional progress bar.
-    :return: Iterator over project, branch path, nick, has_props
+    :return: Iterator over project, branch path, nick, has_props, revnum
     """
-    def check_path(path):
-        return repository.transport.check_path(path, revnum) == subvertpy.NODE_DIR
-    def find_children(path):
-        try:
-            assert not path.startswith("/")
-            dirents = repository.transport.get_dir(path, revnum, DIRENT_KIND|DIRENT_HAS_PROPS)[0]
-        except subvertpy.SubversionException, (msg, num):
-            if num in (subvertpy.ERR_FS_NOT_DIRECTORY,
-                       subvertpy.ERR_FS_NOT_FOUND,
-                       subvertpy.ERR_RA_DAV_PATH_NOT_FOUND,
-                       ERR_RA_DAV_FORBIDDEN):
-                return None
-            raise
-        return [(d, dirents[d]['has_props']) for d in dirents if dirents[d]['kind'] == subvertpy.NODE_DIR]
+    expander = BranchPatternExpander(repository.transport, revnum, project)
 
-    for idx, pattern in enumerate(itemlist):
-        assert isinstance(pattern, str)
-        if pb is not None:
+    pb = ui.ui_factory.nested_progress_bar()
+    try:
+        for idx, pattern in enumerate(itemlist):
+            assert isinstance(pattern, str)
             pb.update("finding branches", idx, len(itemlist))
-        for bp, has_props in expand_branch_pattern([], pattern.strip("/").split("/"), check_path,
-                find_children, project):
-            if verify_fn(bp, project):
-                yield project, bp, bp.split("/")[-1], has_props
+            for bp, has_props, revnum in expander.expand([],
+                    pattern.strip("/").split("/")):
+                if verify_fn(bp, project):
+                    yield project, bp, bp.split("/")[-1], has_props, revnum
+    finally:
+        pb.finished()
 
 
 help_layout = """Subversion repository layouts.
@@ -263,20 +291,23 @@ branches = path/to/*/bla;path/to/trunk
 This would consider paths path/to/foo/bla, path/to/blie/bla and path/to/trunk
 branches, if they existed. The key used (203a...) is the UUID of the Subversion
 repository. The UUID for a repository can be found by running "svn info <url>"
-or "bzr info <url>".
+or "bzr info -v <url>".
 
 """
 
 
 layout_registry = registry.Registry()
-layout_registry.register_lazy("root", "bzrlib.plugins.svn.layout.standard", "RootLayout")
-layout_registry.register_lazy("none", "bzrlib.plugins.svn.layout.standard", "RootLayout")
-layout_registry.register_lazy("trunk", "bzrlib.plugins.svn.layout.standard", "TrunkLayout0")
-layout_registry.register_lazy("trunk0", "bzrlib.plugins.svn.layout.standard", "TrunkLayout0")
-layout_registry.register_lazy("trunk1", "bzrlib.plugins.svn.layout.standard", "TrunkLayout1")
-layout_registry.register_lazy("trunk2", "bzrlib.plugins.svn.layout.standard", "TrunkLayout2")
-layout_registry.register_lazy("trunk3", "bzrlib.plugins.svn.layout.standard", "TrunkLayout3")
-layout_registry.register_lazy("trunk-variable", "bzrlib.plugins.svn.layout.standard", "TrunkLayoutVariable")
+layout_registry.register_lazy("root", "bzrlib.plugins.svn.layout.standard",
+    "RootLayout")
+layout_registry.register_lazy("none", "bzrlib.plugins.svn.layout.standard",
+    "RootLayout")
+layout_registry.register_lazy("trunk", "bzrlib.plugins.svn.layout.standard",
+    "TrunkLayout0")
+for i in range(10):
+    layout_registry.register_lazy("trunk%d" % i,
+        "bzrlib.plugins.svn.layout.standard", "TrunkLayout%d" % i)
+layout_registry.register_lazy("trunk-variable",
+    "bzrlib.plugins.svn.layout.standard", "TrunkLayoutVariable")
 
 layout_registry.register_lazy("itrunk1", "bzrlib.plugins.svn.layout.standard",
     "InverseTrunkLayout1")
@@ -294,7 +325,5 @@ class RepositoryRegistry(registry.Registry):
             return None
 
 repository_registry = RepositoryRegistry()
-repository_registry.register_lazy("283d02a7-25f6-0310-bc7c-ecb5cbfe19da",
-        "bzrlib.plugins.svn.layout.custom", "KDELayout")
 repository_registry.register_lazy("13f79535-47bb-0310-9956-ffa450edef68",
-        "bzrlib.plugins.svn.layout.custom", "ApacheLayout")
+        "bzrlib.plugins.svn.layout.standard", "TrunkLayout")
