@@ -13,14 +13,21 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+from subvertpy import SubversionException, ERR_FS_NOT_FOUND
+import urllib
+
 from bzrlib import ui
 from bzrlib.trace import mutter
 
+from bzrlib.plugins.svn.changes import (
+    common_prefix,
+    )
+
 from bzrlib.plugins.svn.layout.standard import (
-        CustomLayout,
-        RootLayout,
-        TrunkLayout,
-        )
+    CustomLayout,
+    RootLayout,
+    TrunkLayout,
+    )
 
 # Number of revisions to evaluate when guessing the repository layout
 GUESS_SAMPLE_SIZE = 300
@@ -32,19 +39,7 @@ def find_commit_paths(changed_paths):
     :return: List of potential commit paths.
     """
     for changes in changed_paths:
-        yield _find_common_prefix(changes.keys())
-
-
-def _find_common_prefix(paths):
-    prefix = ""
-    # Find a common prefix
-    parts = paths[0].split("/")
-    for i in range(len(parts)+1):
-        for j in paths:
-            if j.split("/")[:i] != parts[:i]:
-                return prefix
-        prefix = "/".join(parts[:i])
-    return prefix
+        yield common_prefix(changes.keys())
 
 
 def guess_layout_from_branch_path(relpath):
@@ -93,8 +88,8 @@ def guess_layout_from_path(relpath):
 def guess_layout_from_history(changed_paths, last_revnum, relpath=None):
     """Try to determine the best fitting layout.
 
-    :param changed_paths: Iterator over (branch_path, changes, revnum, revprops)
-        as returned from LogWalker.iter_changes().
+    :param changed_paths: Iterator over (branch_path, changes, revnum,
+        revprops) as returned from LogWalker.iter_changes().
     :param last_revnum: Number of entries in changed_paths.
     :param relpath: Branch path that should be accepted by the branching
                     scheme as a branch.
@@ -110,7 +105,7 @@ def guess_layout_from_history(changed_paths, last_revnum, relpath=None):
             assert isinstance(revpaths, dict)
             pb.update("analyzing repository layout", last_revnum-revnum,
                       last_revnum)
-            if revpaths == {}:
+            if revnum == 0 or revpaths == {}:
                 continue
             for path in find_commit_paths([revpaths]):
                 layout = guess_layout_from_path(path)
@@ -145,10 +140,13 @@ def guess_layout_from_history(changed_paths, last_revnum, relpath=None):
 
 
 def repository_guess_layout(repository, revnum, branch_path=None):
+    return logwalker_guess_layout(repository._log, revnum, branch_path=branch_path)
+
+def logwalker_guess_layout(logwalker, revnum, branch_path=None):
     pb = ui.ui_factory.nested_progress_bar()
     try:
         (guessed_layout, layout) = guess_layout_from_history(
-            repository._log.iter_changes(None, revnum, max(0, revnum-GUESS_SAMPLE_SIZE)), revnum, branch_path)
+            logwalker.iter_changes(None, revnum, max(0, revnum-GUESS_SAMPLE_SIZE)), revnum, branch_path)
     finally:
         pb.finished()
     mutter("Guessed repository layout: %r, guess layout to use: %r" %
@@ -156,4 +154,22 @@ def repository_guess_layout(repository, revnum, branch_path=None):
     return (guessed_layout, layout)
 
 
+def is_likely_branch_url(url):
+    """Check whether url refers to a likely branch URL.
 
+    """
+    from bzrlib.plugins.svn.transport import SvnRaTransport
+    from bzrlib.plugins.svn.logwalker import LogWalker
+    transport = SvnRaTransport(url)
+    lw = LogWalker(transport=transport)
+    svn_root_url = transport.get_repos_root()
+    branch_path = urllib.unquote(url[len(svn_root_url):])
+    try:
+        (guessed_layout, _) = logwalker_guess_layout(lw, transport.get_latest_revnum())
+    except SubversionException, (msg, num):
+        if num == ERR_FS_NOT_FOUND:
+            return False # path doesn't exist
+        raise
+    if guessed_layout is None:
+        return None
+    return guessed_layout.is_branch(branch_path)
