@@ -26,11 +26,16 @@ from bzrlib import (
     trace,
     )
 from bzrlib.errors import (
+    BzrError,
     NoSuchRevision,
     )
 
 from bzrlib.plugins.svn import (
     changes,
+    errors as bzrsvn_errors,
+    )
+from bzrlib.plugins.svn.cache import (
+    CacheConcurrencyError,
     )
 from bzrlib.plugins.svn.transport import (
     SvnRaTransport,
@@ -45,7 +50,7 @@ from subvertpy import NODE_UNKNOWN
 # Maximum number of extra revisions to fetch in caching logwalker
 MAX_OVERHEAD_FETCH = 1000
 
-def iter_all_changes(from_revnum, to_revnum, get_revision_paths, 
+def iter_all_changes(from_revnum, to_revnum, get_revision_paths,
     revprop_list, limit=0):
     revnum = from_revnum
     ascending = (to_revnum > from_revnum)
@@ -82,7 +87,10 @@ def iter_changes(prefixes, from_revnum, to_revnum, get_revision_paths,
     """
     revnum = from_revnum
 
-    if prefixes in (None, [""], []):
+    if prefixes is not None:
+        prefixes = set(prefixes)
+
+    if prefixes in (None, set([""]), set()):
         return iter_all_changes(from_revnum, to_revnum, get_revision_paths,
                 revprop_list, limit)
     else:
@@ -90,21 +98,20 @@ def iter_changes(prefixes, from_revnum, to_revnum, get_revision_paths,
             get_revision_paths, revprop_list, limit)
 
 
-def iter_prefixes_changes(prefixes, from_revnum, to_revnum, 
+def iter_prefixes_changes(from_prefixes, from_revnum, to_revnum,
     get_revision_paths, revprop_list, limit=0):
-    prefixes = [p.strip("/") for p in prefixes]
+    assert type(from_prefixes) is set
+    from_prefixes = set([p.strip("/") for p in from_prefixes])
 
     assert from_revnum >= to_revnum, "path: %r, %d >= %d" % (
-            prefixes, from_revnum, to_revnum)
+            from_prefixes, from_revnum, to_revnum)
 
     revnum = from_revnum
-    todo_prefixes = { revnum: set(prefixes) }
+    todo_prefixes = { revnum: from_prefixes }
 
     i = 0
     while revnum >= to_revnum:
         prefixes = todo_prefixes.pop(revnum)
-        if not (revnum > 0):
-            raise AssertionError("Inconsistent revnum, prefixes: %r,%r" % (revnum, prefixes))
         revpaths = get_revision_paths(revnum)
 
         # Report this revision if any affected paths are changed
@@ -134,12 +141,17 @@ def iter_prefixes_changes(prefixes, from_revnum, to_revnum,
 
 
 class LogCache(object):
-    """Log browser cache. """
+    """Log browser cache manager. The methods of this class
+    encapsulate the data access patterns used by CachingLogWalker to
+    access the log cache."""
 
     def find_latest_change(self, path, revnum):
         """Find the last revision in which a particular path
         was changed.
 
+        This is an optional operation: cache backends may choose not to
+        implement it if they cannot efficiently perform this kind of lookup.
+
         :param path: Path to check
         :param revnum: Revision in which to check
         """
@@ -152,7 +164,7 @@ class LogCache(object):
         """
         raise NotImplementedError(self.get_revision_paths)
 
-    def insert_paths(self, rev, orig_paths):
+    def insert_paths(self, rev, orig_paths, revprops, all_revprops):
         """Insert new history information into the cache.
 
         :param rev: Revision number of the revision
@@ -173,8 +185,38 @@ class LogCache(object):
     def insert_revprops(self, revision, revprops, all):
         raise NotImplementedError(self.insert_revprops)
 
-    def last_revnum(self):
-        raise NotImplementedError(self.last_revnum)
+    def max_revnum(self):
+        raise NotImplementedError(self.max_revnum)
+
+    def min_revnum(self):
+        raise NotImplementedError(self.min_revnum)
+
+
+class CachingLogWalkerUpdater(object):
+    """Function that can update a logwalker."""
+
+    def __init__(self, logwalker, all_revprops):
+        self.count = 0
+        self.total = None
+        self.pb = ui.ui_factory.nested_progress_bar()
+        self.all_revprops = all_revprops
+        self.logwalker = logwalker
+
+    def __call__(self, orig_paths, revision, revprops, has_children=None):
+        self.count += 1
+        self.pb.update('fetching svn revision info', self.count, self.total)
+        self.logwalker.cache.insert_paths(revision, orig_paths,
+            revprops, self.all_revprops)
+        self.logwalker.saved_maxrevnum = max(revision,
+            self.logwalker.saved_maxrevnum)
+        if self.logwalker.saved_minrevnum is None:
+            self.logwalker.saved_minrevnum = revision
+        else:
+            self.logwalker.saved_minrevnum = min(revision,
+                self.logwalker.saved_minrevnum)
+
+    def finished(self):
+        self.pb.finished()
 
 
 class CachingLogWalker(object):
@@ -185,7 +227,7 @@ class CachingLogWalker(object):
         self.actual = actual
         self.quick_revprops = actual.quick_revprops
         self._transport = actual._transport
-        self.saved_revnum = self.cache.last_revnum()
+        self.saved_maxrevnum = self.cache.max_revnum()
         self.saved_minrevnum = self.cache.min_revnum()
         self._latest_revnum = None
 
@@ -193,6 +235,9 @@ class CachingLogWalker(object):
         if "logwalker" in debug.debug_flags:
             trace.mutter(text, *args, **kwargs)
 
+    def _warn_busy_cache(self):
+        trace.warning("Cache for repository %s busy, ignoring")
+
     def find_latest_change(self, path, revnum):
         """Find latest revision that touched path.
 
@@ -201,7 +246,11 @@ class CachingLogWalker(object):
         """
         assert isinstance(path, str)
         assert isinstance(revnum, int) and revnum >= 0
-        self._fetch_revisions(revnum)
+        try:
+            self._fetch_revisions(revnum)
+        except CacheConcurrencyError:
+            self._warn_busy_cache()
+            return self.actual.find_latest_change(path, revnum)
 
         self.mutter("latest change: %r:%r", path, revnum)
         try:
@@ -214,10 +263,10 @@ class CachingLogWalker(object):
         return revnum
 
     def iter_changes(self, prefixes, from_revnum, to_revnum=0, limit=0, pb=None):
-        """Return iterator over all the revisions between from_revnum and to_revnum named path or inside path.
+        """Return iterator over all the revisions in a range.
 
-        :param prefixes:    Prefixes of paths to report about
-        :param from_revnum:  Start revision.
+        :param prefixes: Prefixes of paths to report about
+        :param from_revnum: Start revision.
         :param to_revnum: End revision.
         :return: An iterator that yields tuples with (paths, revnum, revprops)
             where paths is a dictionary with all changes that happened
@@ -225,20 +274,33 @@ class CachingLogWalker(object):
         """
         assert from_revnum >= 0 and to_revnum >= 0, "%r -> %r" % (from_revnum, to_revnum)
         self.mutter("iter changes %r->%r (%r)", from_revnum, to_revnum, prefixes)
-        self._fetch_revisions(max(from_revnum, to_revnum), pb=pb)
+        try:
+            self._fetch_revisions(max(from_revnum, to_revnum), pb=pb)
+        except CacheConcurrencyError:
+            self._warn_busy_cache()
+            return self.actual.iter_changes(prefixes, from_revnum, to_revnum,
+                    limit, pb)
         return iter(iter_changes(prefixes, from_revnum, to_revnum,
             self.get_revision_paths, self.revprop_list, limit))
 
     def get_revision_paths(self, revnum):
         if revnum == 0:
             return changes.REV0_CHANGES
-        self._fetch_revisions(revnum)
-
-        return self.cache.get_revision_paths(revnum)
+        try:
+            self._fetch_revisions(revnum)
+        except CacheConcurrencyError:
+            self._warn_busy_cache()
+            return self.actual.get_revision_paths(revnum)
+        else:
+            return self.cache.get_revision_paths(revnum)
 
     def revprop_list(self, revnum):
         self.mutter('revprop list: %d' % revnum)
-        self._fetch_revisions(revnum)
+        try:
+            self._fetch_revisions(revnum)
+        except CacheConcurrencyError:
+            self._warn_busy_cache()
+            return self.actual.revprop_list(revnum)
 
         if revnum > 0:
             (known_revprops, has_all_revprops) = self.cache.get_revprops(revnum)
@@ -253,74 +315,72 @@ class CachingLogWalker(object):
 
     def _caching_revprop_list(self, revnum):
         revprops = self._transport.revprop_list(revnum)
-        self.cache.insert_revprops(revnum, revprops, True)
-        self.cache.commit()
+        try:
+            self.cache.insert_revprops(revnum, revprops, True)
+            self.cache.commit()
+        except CacheConcurrencyError:
+            pass
         return revprops
 
-    def _fetch_revisions(self, to_revnum=None, pb=None):
+    def _fetch_revisions(self, to_revnum, pb=None):
         """Fetch information about all revisions in the remote repository
         until to_revnum.
 
         :param to_revnum: End of range to fetch information for
         """
-        assert isinstance(self.saved_revnum, int)
+        assert isinstance(self.saved_maxrevnum, int)
 
-        if to_revnum <= self.saved_revnum and self.saved_minrevnum == 0:
+        if to_revnum <= self.saved_maxrevnum and self.saved_minrevnum == 0:
             return
 
         # Try to fetch log data in lumps, if possible.
         if self._latest_revnum is None:
             self._latest_revnum = self.actual._transport.get_latest_revnum()
         assert isinstance(self._latest_revnum, int)
-        to_revnum = max(min(self._latest_revnum, to_revnum+MAX_OVERHEAD_FETCH), to_revnum)
+        to_revnum = max(min(self._latest_revnum, to_revnum+MAX_OVERHEAD_FETCH),
+                        to_revnum)
 
-        # Subversion 1.4 clients and servers can only deliver a limited set of revprops
+        # Subversion 1.4 clients and servers can only deliver a limited set of
+        # revprops
         if self._transport.has_capability("log-revprops"):
             todo_revprops = None
         else:
             todo_revprops = ["svn:author", "svn:log", "svn:date"]
 
-        def rcvr(orig_paths, revision, revprops, has_children=None):
-            nested_pb.update('fetching svn revision info',
-                             to_revnum - revision, to_revnum)
-            self.cache.insert_paths(revision, orig_paths)
-            self.cache.insert_revprops(revision, revprops,
-                                       todo_revprops is None)
-            self.saved_revnum = max(revision, self.saved_revnum)
-            if self.saved_minrevnum is None:
-                self.saved_minrevnum = revision
-            else:
-                self.saved_minrevnum = min(revision, self.saved_minrevnum)
-
-        if pb is None:
-            nested_pb = ui.ui_factory.nested_progress_bar()
-        else:
-            nested_pb = pb
-
+        rcvr = CachingLogWalkerUpdater(self, todo_revprops is None)
         try:
-            nested_pb.update('fetching svn revision info', 0, to_revnum)
             try:
+                # The get_log bounds are inclusive at both ends, so the total
+                # number of revisions requested is A - B.
+                rcvr.total = to_revnum - self.saved_maxrevnum
                 # Try to keep the cache consistent by closing any holes early
                 # in the history
-                if self.saved_minrevnum is None:
-                    self.saved_minrevnum = 0
-                elif self.saved_minrevnum != 0:
+                if self.saved_minrevnum:
+                    rcvr.total += self.saved_minrevnum
                     self.mutter("get_log %d->%d", self.saved_minrevnum, 0)
                     self.actual._transport.get_log(rcvr, [""],
-                        self.saved_minrevnum, 0, 0, True, True, False,
+                        self.saved_minrevnum - 1, 0, 0, True, True, False,
                         todo_revprops)
-                self.mutter("get_log %d->%d", to_revnum, self.saved_revnum)
-                self.actual._transport.get_log(rcvr, [""], to_revnum,
-                    self.saved_revnum, 0, True, True, False, todo_revprops)
-            except subvertpy.SubversionException, (_, num):
+                    if self.saved_minrevnum:
+                        raise bzrsvn_errors.IncompleteRepositoryHistory(
+                            "first available revision: %d" % self.saved_minrevnum)
+                if to_revnum > self.saved_maxrevnum:
+                    self.mutter("get_log %d->%d", to_revnum,
+                            self.saved_maxrevnum)
+                    self.actual._transport.get_log(rcvr, [""], to_revnum,
+                        self.saved_maxrevnum, 0, True, True, False,
+                        todo_revprops)
+                    if to_revnum > self.saved_maxrevnum and to_revnum > 0:
+                        raise bzrsvn_errors.IncompleteRepositoryHistory(
+                            "last available revision: %d" % self.saved_maxrevnum)
+            except subvertpy.SubversionException, (msg, num):
                 if num == subvertpy.ERR_FS_NO_SUCH_REVISION:
                     raise NoSuchRevision(branch=self,
-                        revision="Revision number %d" % to_revnum)
+                        revision="Revision number %d: %s" % (to_revnum, msg))
                 raise
         finally:
+            rcvr.finished()
             self.cache.commit()
-            if pb is None:
-                nested_pb.finished()
 
 
 def strip_slashes(changed_paths):
@@ -383,7 +443,8 @@ class LogWalker(object):
         return lazy_dict({}, self._transport.revprop_list, revnum)
 
     def iter_changes(self, prefixes, from_revnum, to_revnum=0, limit=0, pb=None):
-        """Return iterator over all the revisions between revnum and 0 named path or inside path.
+        """Return iterator over all the revisions between revnum and 0 named
+        path or inside path.
 
         :param prefixes:    Prefixes to report about (in from_revnum)
         :param from_revnum:  Start revision.
@@ -412,8 +473,11 @@ class LogWalker(object):
                     revpaths = {}
                 if todo_revprops is None:
                     revprops = known_revprops
+                    if revprops is None:
+                        revprops = {}
                 else:
-                    revprops = lazy_dict(known_revprops, self._transport.revprop_list, revnum)
+                    revprops = lazy_dict(known_revprops, self._transport.revprop_list,
+                        revnum)
                 yield (revpaths, revnum, revprops)
         except subvertpy.SubversionException, (_, num):
             if num == subvertpy.ERR_FS_NO_SUCH_REVISION:
@@ -433,8 +497,9 @@ class LogWalker(object):
             return changes.REV0_CHANGES
 
         try:
-            return strip_slashes(
-                self._transport.iter_log([""], revnum, revnum, 1, True, True, False, []).next()[0])
+            log_iter = self._transport.iter_log([""], revnum, revnum, 1, True,
+                True, False, [])
+            return strip_slashes(log_iter.next()[0])
         except subvertpy.SubversionException, (_, num):
             if num == subvertpy.ERR_FS_NO_SUCH_REVISION:
                 raise NoSuchRevision(branch=self,
