@@ -18,6 +18,7 @@
 
 
 import calendar
+from cStringIO import StringIO
 from subvertpy import properties
 import time
 import urllib
@@ -49,6 +50,7 @@ SVN_PROP_BZR_FILEIDS = 'bzr:file-ids'
 SVN_PROP_BZR_REVISION_INFO = 'bzr:revision-info'
 SVN_PROP_BZR_REVISION_ID = 'bzr:revision-id:'
 SVN_PROP_BZR_TEXT_REVISIONS = 'bzr:text-revisions'
+SVN_PROP_BZR_TEXT_PARENTS = 'bzr:text-parents'
 SVN_PROP_BZR_LOG = 'bzr:log'
 SVN_PROP_BZR_REQUIRED_FEATURES = 'bzr:required-features'
 SVN_PROP_BZR_HIDDEN = 'bzr:hidden'
@@ -66,13 +68,14 @@ SVN_REVPROP_BZR_ROOT = 'bzr:root'
 SVN_REVPROP_BZR_SIGNATURE = 'bzr:gpg-signature'
 SVN_REVPROP_BZR_TIMESTAMP = 'bzr:timestamp'
 SVN_REVPROP_BZR_LOG = 'bzr:log'
+SVN_REVPROP_BZR_TEXT_PARENTS = 'bzr:text-parents'
 SVN_REVPROP_BZR_TEXT_REVISIONS = 'bzr:text-revisions'
 SVN_REVPROP_BZR_REQUIRED_FEATURES = 'bzr:required-features'
 SVN_REVPROP_BZR_BASE_REVISION = 'bzr:base-revision'
 SVN_REVPROP_BZR_SKIP = 'bzr:skip'
 SVN_REVPROP_BZR_HIDDEN = 'bzr:hidden'
 SVN_REVPROP_BZR_REPOS_UUID = 'bzr:repository-uuid'
-SVN_REVPROP_BZR_POINTLESS = 'bzr:pointless'
+SVN_REVPROP_BZR_TESTAMENT = 'bzr:testament'
 
 
 def find_new_lines((oldvalue, newvalue)):
@@ -113,7 +116,7 @@ def format_highres_date(t, offset=0):
     :param offset:  The timezone offset in integer seconds
     :type offset: int
     """
-    assert isinstance(t, float)
+    assert isinstance(t, float) or isinstance(t, int)
 
     # This has to be formatted for "original" date, so that the
     # revision XML entry will be reproduced faithfully.
@@ -191,15 +194,27 @@ def parse_svn_dateprop(date):
 
 
 def parse_svn_log(log):
+    """Parse a Subversion commit ("svn:log") message.
+
+    :param log: Log message as bytestring or None
+    :return: Unicode string with log message
+    """
     if log is None:
         return u""
     return log.decode("utf-8", "replace")
 
 
 def parse_svn_revprops(svn_revprops, rev):
-    rev.committer = svn_revprops.get(properties.PROP_REVISION_AUTHOR, "").decode("utf-8")
+    """Fill in a Revision object from a set of svn properties.
+
+    :param svn_revprops: Subversion revision properties
+    :param rev: Revision object
+    """
+    rev.committer = svn_revprops.get(properties.PROP_REVISION_AUTHOR, "")
     if rev.committer is None:
         rev.committer = u""
+    else:
+        rev.committer = rev.committer.decode("utf-8")
     rev.message = parse_svn_log(svn_revprops.get(properties.PROP_REVISION_LOG))
     assert type(rev.message) is unicode
 
@@ -294,7 +309,8 @@ def parse_bzr_svn_revprops(props, rev):
     """
     changed = False
     if props.has_key(SVN_REVPROP_BZR_TIMESTAMP):
-        (rev.timestamp, rev.timezone) = unpack_highres_date(props[SVN_REVPROP_BZR_TIMESTAMP])
+        (rev.timestamp, rev.timezone) = unpack_highres_date(
+            props[SVN_REVPROP_BZR_TIMESTAMP])
         changed = True
 
     if props.has_key(SVN_REVPROP_BZR_COMMITTER):
@@ -308,7 +324,7 @@ def parse_bzr_svn_revprops(props, rev):
 
     for name, value in props.iteritems():
         if name.startswith(SVN_REVPROP_BZR_REVPROP_PREFIX):
-            rev.properties[name[len(SVN_REVPROP_BZR_REVPROP_PREFIX):]] = value
+            rev.properties[name[len(SVN_REVPROP_BZR_REVPROP_PREFIX):]] = value.decode("utf-8")
         changed = True
 
     return changed
@@ -333,9 +349,11 @@ class BzrSvnMapping(foreign.VcsMapping):
 
     def __init__(self):
         super(BzrSvnMapping, self).__init__(foreign_vcs_svn)
-        if (version_info[3] == 'exp' or self.experimental) and not BzrSvnMapping._warned_experimental:
+        if ((version_info[3] == 'exp' or self.experimental) and
+            not BzrSvnMapping._warned_experimental):
             from bzrlib.trace import warning
-            warning("using experimental bzr-svn mappings; may break existing branches in the most horrible ways")
+            warning("using experimental bzr-svn mappings; may break existing "
+                    "branches in the most horrible ways")
             BzrSvnMapping._warned_experimental = True
 
     @classmethod
@@ -410,7 +428,8 @@ class BzrSvnMapping(foreign.VcsMapping):
 
         :param uuid: UUID of the repository
         :param revnum: Revision number at which the file was introduced.
-        :param branch: Branch path of the branch in which the file was introduced.
+        :param branch: Branch path of the branch in which the file was
+            introduced.
         :param inv_path: Original path of the file within the inventory
         """
         raise NotImplementedError
@@ -509,6 +528,36 @@ class BzrSvnMapping(foreign.VcsMapping):
         """
         raise NotImplementedError(self.export_fileid_map_fileprops)
 
+    def import_text_parents_revprops(self, revprops):
+        """Obtain a text parent map from properties.
+
+        :param revprops: Subversion revision properties.
+        """
+        raise NotImplementedError(self.import_text_parents_revprops)
+
+    def import_text_parents_fileprops(self, fileprops):
+        """Obtain a text parent map from properties.
+
+        :param fileprops: File properties.
+        """
+        raise NotImplementedError(self.import_text_parents_fileprops)
+
+    def export_text_parents_revprops(self, text_parents, revprops):
+        """Store a text parent map.
+
+        :param text_parents: Text parent map
+        :param revprops: Revision properties
+        """
+        raise NotImplementedError(self.export_text_parents_revprops)
+
+    def export_text_parents_fileprops(self, text_parents, fileprops):
+        """Store a text parent map.
+
+        :param text_parents: Text parent map
+        :param fileprops: File properties
+        """
+        raise NotImplementedError(self.export_text_parents_fileprops)
+
     def export_text_revisions_revprops(self, text_revisions, revprops):
         """Store a text revisions map.
 
@@ -531,12 +580,15 @@ class BzrSvnMapping(foreign.VcsMapping):
     def import_text_revisions_fileprops(self, fileprops):
         raise NotImplementedError(self.import_text_revisions_fileprops)
 
-    def export_revision_revprops(self, uuid, branch_root, timestamp, timezone, committer, revprops, revision_id, revno, parent_ids, svn_revprops):
+    def export_revision_revprops(self, svn_revprops, uuid, branch_root,
+            timestamp, timezone, committer, revprops, revision_id, revno,
+            parent_ids, testament):
         """Determines the revision properties.
         """
         raise NotImplementedError(self.export_revision_revprops)
 
-    def export_revision_fileprops(self, timestamp, timezone, committer, revprops, revision_id, revno, parent_ids, svn_fileprops):
+    def export_revision_fileprops(self, svn_fileprops, timestamp, timezone,
+            committer, revprops, revision_id, revno, parent_ids, testament):
         """Determines the branch root file properties.
         """
         raise NotImplementedError(self.export_revision_fileprops)
@@ -583,8 +635,8 @@ class BzrSvnMapping(foreign.VcsMapping):
 
     def newer_than(self, other_mapping):
         """Check whether other_mapping precedes this mapping, and that
-        revisions mapped with other_mapping can exist in the ancestry of revisions
-        mapped with this mapping.
+        revisions mapped with other_mapping can exist in the ancestry of
+        revisions mapped with this mapping.
         """
         return False
 
@@ -608,19 +660,41 @@ def generate_fileid_property(fileids):
     :param fileids: Map of path -> fileid
     :return: Property value
     """
-    return "".join(["%s\t%s\n" % (urllib.quote(path.encode("utf-8")), fileids[path]) for path in sorted(fileids.keys())])
+    return "".join(["%s\t%s\n" % (urllib.quote(path.encode("utf-8")),
+        fileids[path]) for path in sorted(fileids.keys())])
+
+
+def parse_text_parents_property(text):
+    ret = {}
+    for line in text.splitlines():
+        parts = line.split("\t")
+        entry = parts[0]
+        ret[urllib.unquote(entry).decode("utf-8")] = filter(lambda x: x != "", [osutils.safe_revision_id(parent_revid) for parent_revid in parts[1:]])
+    return ret
 
 
 def parse_text_revisions_property(text):
     ret = {}
     for line in text.splitlines():
         (entry, revid) = line.split("\t", 1)
-        ret[urllib.unquote(entry).decode("utf-8")] = osutils.safe_revision_id(revid)
+        ret[urllib.unquote(entry).decode("utf-8")] = osutils.safe_revision_id(
+            revid)
     return ret
 
 
+def generate_text_parents_property(text_parents):
+    return "".join(["%s\t%s\n" % (
+        urllib.quote(path.encode("utf-8")), "\t".join(text_parents[path]))
+        for path in sorted(text_parents.keys())
+        if text_parents[path] is not None])
+
+
 def generate_text_revisions_property(text_revisions):
-    return "".join(["%s\t%s\n" % (urllib.quote(path.encode("utf-8")), text_revisions[path]) for path in sorted(text_revisions.keys())])
+    return "".join([
+        "%s\t%s\n" % (urllib.quote(path.encode("utf-8")), text_revisions[path])
+        for path in sorted(text_revisions.keys())
+        if text_revisions[path] is not None
+        ])
 
 
 class BzrSvnMappingFileProps(object):
@@ -639,6 +713,13 @@ class BzrSvnMappingFileProps(object):
             return True
         return False
 
+    def import_text_parents_fileprops(self, fileprops):
+        metadata = fileprops.get(SVN_PROP_BZR_TEXT_PARENTS)
+        if metadata is None:
+            return {}
+        assert isinstance(metadata, tuple)
+        return parse_text_parents_property(metadata[1])
+
     def import_text_revisions_fileprops(self, fileprops):
         metadata = fileprops.get(SVN_PROP_BZR_TEXT_REVISIONS)
         if metadata is None:
@@ -646,6 +727,12 @@ class BzrSvnMappingFileProps(object):
         assert isinstance(metadata, tuple)
         return parse_text_revisions_property(metadata[1])
 
+    def export_text_parents_fileprops(self, text_parents, fileprops):
+        if text_parents != {}:
+            fileprops[SVN_PROP_BZR_TEXT_PARENTS] = generate_text_parents_property(text_parents)
+        elif SVN_PROP_BZR_TEXT_PARENTS in fileprops:
+            fileprops[SVN_PROP_BZR_TEXT_PARENTS] = ""
+
     def export_text_revisions_fileprops(self, text_revisions, fileprops):
         if text_revisions != {}:
             fileprops[SVN_PROP_BZR_TEXT_REVISIONS] = generate_text_revisions_property(text_revisions)
@@ -661,7 +748,8 @@ class BzrSvnMappingFileProps(object):
                 mutter(str(e))
                 return ()
             if len(new_lines) != 1:
-                mutter("unexpected number of lines in bzr merge property: %r" % new_lines)
+                mutter("unexpected number of lines in bzr merge property: %r",
+                    new_lines)
                 return ()
             return parse_merge_property(new_lines[0])
 
@@ -697,15 +785,16 @@ class BzrSvnMappingFileProps(object):
 
         return svnprops
 
-    def export_revision_fileprops(self, timestamp, timezone, committer,
-        revprops, revision_id, revno, parent_ids, svn_fileprops):
+    def export_revision_fileprops(self, svn_fileprops, timestamp, timezone,
+            committer, revprops, revision_id, revno, parent_ids, testament):
 
         # Keep track of what Subversion properties to set later on
         svn_fileprops[SVN_PROP_BZR_REVISION_INFO] = generate_revision_metadata(
             timestamp, timezone, committer, revprops)
 
         if len(parent_ids) > 1:
-            svn_fileprops.update(self._record_merges(parent_ids[1:], svn_fileprops))
+            svn_fileprops.update(self._record_merges(parent_ids[1:],
+                svn_fileprops))
 
         # Set appropriate property if revision id was specified by
         # caller
@@ -732,14 +821,17 @@ class BzrSvnMappingFileProps(object):
 
         if len(new_lines) != 1:
             mutter("unexpected number of lines: %r" % new_lines)
-            return (None, None, self.is_bzr_revision_hidden_fileprops(fileprops))
+            return (None, None,
+                    self.is_bzr_revision_hidden_fileprops(fileprops))
 
         try:
             (revno, revid) = parse_revid_property(new_lines[0])
-            return (revno, revid, self.is_bzr_revision_hidden_fileprops(fileprops))
+            return (revno, revid,
+                    self.is_bzr_revision_hidden_fileprops(fileprops))
         except errors.InvalidPropertyValue, e:
             mutter(str(e))
-            return (None, None, self.is_bzr_revision_hidden_fileprops(fileprops))
+            return (None, None,
+                    self.is_bzr_revision_hidden_fileprops(fileprops))
 
     def export_fileid_map_fileprops(self, fileids, fileprops):
         if fileids != {}:
@@ -750,7 +842,8 @@ class BzrSvnMappingFileProps(object):
 
     def check_fileprops(self, changed_fileprops, checkresult):
         # bzr:revision-id:*
-        text = changed_fileprops.get(SVN_PROP_BZR_REVISION_ID+self.name, ("", ""))
+        text = changed_fileprops.get(SVN_PROP_BZR_REVISION_ID+self.name,
+            ("", ""))
         try:
             new_lines = find_new_lines(text)
         except ValueError, e:
@@ -784,14 +877,25 @@ class BzrSvnMappingRevProps(object):
             return {}
         return parse_fileid_property(svn_revprops[SVN_REVPROP_BZR_FILEIDS])
 
+    def import_text_parents_revprops(self, svn_revprops):
+        if not svn_revprops.has_key(SVN_REVPROP_BZR_TEXT_PARENTS):
+            return {}
+        return parse_text_parents_property(svn_revprops[SVN_REVPROP_BZR_TEXT_PARENTS])
+
+    def export_text_parents_revprops(self, text_parents, svn_revprops):
+        if text_parents != {}:
+            svn_revprops[SVN_REVPROP_BZR_TEXT_PARENTS] = generate_text_parents_property(text_parents)
+
     def import_text_revisions_revprops(self, svn_revprops):
         if not svn_revprops.has_key(SVN_REVPROP_BZR_TEXT_REVISIONS):
             return {}
-        return parse_text_revisions_property(svn_revprops[SVN_REVPROP_BZR_TEXT_REVISIONS])
+        return parse_text_revisions_property(
+            svn_revprops[SVN_REVPROP_BZR_TEXT_REVISIONS])
 
     def export_text_revisions_revprops(self, text_revisions, svn_revprops):
         if text_revisions != {}:
-            svn_revprops[SVN_REVPROP_BZR_TEXT_REVISIONS] = generate_text_revisions_property(text_revisions)
+            value = generate_text_revisions_property(text_revisions)
+            svn_revprops[SVN_REVPROP_BZR_TEXT_REVISIONS] = value
 
     def get_lhs_parent_revprops(self, svn_revprops):
         return svn_revprops.get(SVN_REVPROP_BZR_BASE_REVISION)
@@ -816,18 +920,21 @@ class BzrSvnMappingRevProps(object):
     def export_message_revprops(self, message, revprops):
         revprops[SVN_REVPROP_BZR_LOG] = message.encode("utf-8")
 
-    def export_revision_revprops(self, uuid, branch_root, timestamp, timezone,
-        committer, revprops, revision_id, revno, parent_ids, svn_revprops):
+    def export_revision_revprops(self, svn_revprops, uuid, branch_root,
+            timestamp, timezone, committer, revprops, revision_id, revno,
+            parent_ids, testament):
         svn_revprops[SVN_REVPROP_BZR_MAPPING_VERSION] = self.name
         if timestamp is not None:
-            svn_revprops[SVN_REVPROP_BZR_TIMESTAMP] = format_highres_date(timestamp, timezone)
+            svn_revprops[SVN_REVPROP_BZR_TIMESTAMP] = format_highres_date(
+                timestamp, timezone)
 
         if committer is not None:
             svn_revprops[SVN_REVPROP_BZR_COMMITTER] = committer.encode("utf-8")
 
         if revprops is not None:
             for name, value in revprops.iteritems():
-                svn_revprops[SVN_REVPROP_BZR_REVPROP_PREFIX+name] = value.encode("utf-8")
+                svn_name = SVN_REVPROP_BZR_REVPROP_PREFIX+name
+                svn_revprops[svn_name] = value.encode("utf-8")
 
         svn_revprops[SVN_REVPROP_BZR_ROOT] = branch_root
         svn_revprops[SVN_REVPROP_BZR_REPOS_UUID] = uuid
@@ -836,7 +943,8 @@ class BzrSvnMappingRevProps(object):
             svn_revprops[SVN_REVPROP_BZR_REVISION_ID] = revision_id
 
         if len(parent_ids) > 1:
-            svn_revprops[SVN_REVPROP_BZR_MERGE] = "".join([x+"\n" for x in parent_ids[1:]])
+            svn_revprops[SVN_REVPROP_BZR_MERGE] = "".join(
+                [x+"\n" for x in parent_ids[1:]])
         if len(parent_ids) == 0:
             svn_revprops[SVN_REVPROP_BZR_BASE_REVISION] = NULL_REVISION
         else:
@@ -844,10 +952,13 @@ class BzrSvnMappingRevProps(object):
 
         svn_revprops[SVN_REVPROP_BZR_REVNO] = str(revno)
         svn_revprops[SVN_REVPROP_BZR_USER_AGENT] = get_client_string()
+        if testament is not None:
+            svn_revprops[SVN_REVPROP_BZR_TESTAMENT] = testament.as_short_text()
 
     def export_fileid_map_revprops(self, fileids, revprops):
         if fileids != {}:
-            revprops[SVN_REVPROP_BZR_FILEIDS] = generate_fileid_property(fileids)
+            revprops[SVN_REVPROP_BZR_FILEIDS] = generate_fileid_property(
+                fileids)
 
     def check_revprops(self, revprops, check_result):
         pass
@@ -874,6 +985,8 @@ class SubversionMappingRegistry(foreign.VcsMappingRegistry):
         :param revid: Revision id to parse
         :return: tuple with (uuid, branch_path, revno), mapping
         """
+        if type(revid) is not str:
+            raise TypeError("revision id is not a bytestring")
         if not revid.startswith("svn-"):
             raise InvalidRevisionId(revid, None)
         mapping_version = revid[len("svn-"):len("svn-vx")]
@@ -907,7 +1020,8 @@ def find_mapping_revprops(revprops):
     """
     if SVN_REVPROP_BZR_MAPPING_VERSION in revprops:
         try:
-            return mapping_registry.parse_mapping_name("svn-" + revprops[SVN_REVPROP_BZR_MAPPING_VERSION])
+            return mapping_registry.parse_mapping_name(
+                "svn-%s" % revprops[SVN_REVPROP_BZR_MAPPING_VERSION])
         except KeyError:
             return None
     return None
@@ -924,7 +1038,8 @@ def find_mappings_fileprops(changed_fileprops):
                 pass
             else:
                 try:
-                    mapping = mapping_registry.parse_mapping_name("svn-" + k[len(SVN_PROP_BZR_REVISION_ID):])
+                    mapping = mapping_registry.parse_mapping_name(
+                        "svn-" + k[len(SVN_PROP_BZR_REVISION_ID):])
                 except KeyError:
                     pass
                 else:
@@ -960,8 +1075,7 @@ def is_bzr_revision_fileprops(fileprops):
 
 
 def estimate_bzr_ancestors(fileprops):
-    """Estimate the number of bzr ancestors that a revision has based on file properties.
-
+    """Estimate the number of bzr ancestors of a revision from file properties.
     """
     found = []
     for k, v in fileprops.iteritems():
@@ -1001,6 +1115,16 @@ def revprops_complete(revprops):
             SVN_REVPROP_BZR_HIDDEN in revprops)
 
 
+def get_svn_file_contents(tree, kind, file_id):
+    if kind == "file":
+        return tree.get_file(file_id)
+    elif kind == "symlink":
+        target = tree.get_symlink_target(file_id)
+        return StringIO("link %s" % target.encode("utf-8"))
+    else:
+        raise AssertionError
+
+
 class ForeignSubversion(foreign.ForeignVcs):
 
     @property
@@ -1018,7 +1142,7 @@ class ForeignSubversion(foreign.ForeignVcs):
         self.abbreviation = "svn"
 
     def show_foreign_revid(self, (uuid, bp, revnum)):
-        return { "svn revno": "%d (on /%s)" % (revnum, bp)}
+        return { u"svn revno": u"%d (on /%s)" % (revnum, bp.decode("utf-8"))}
 
     def serialize_foreign_revid(self, (uuid, bp, revnum)):
         return "%s:%d:%s" % (uuid, revnum, urllib.quote(bp))
