@@ -16,11 +16,13 @@
 """Custom revision specifier for Subversion."""
 
 from bzrlib.errors import (
-    BzrError,
     InvalidRevisionId,
     InvalidRevisionSpec,
     NoSuchRevision,
     )
+from bzrlib.revision import (
+    NULL_REVISION,
+    )
 from bzrlib.revisionspec import (
     RevisionSpec,
     RevisionInfo,
@@ -35,11 +37,11 @@ class RevisionSpec_svn(RevisionSpec):
 
     help_txt = """Selects a revision using a Subversion revision number (revno).
 
-    Subversion revision numbers are per-repository whereas Bazaar revision 
-    numbers are per-branch. This revision specifier allows specifying 
+    Subversion revision numbers are per-repository whereas Bazaar revision
+    numbers are per-branch. This revision specifier allows specifying
     a Subversion revision number.
     """
-    
+
     prefix = 'svn:'
 
     def _get_revnum(self):
@@ -48,7 +50,10 @@ class RevisionSpec_svn(RevisionSpec):
 
     def _match_on_foreign(self, branch):
         ret = set()
-        revnum = self._get_revnum()
+        try:
+            revnum = self._get_revnum()
+        except ValueError:
+            raise InvalidRevisionSpec(self.user_spec, branch)
         branch.lock_read()
         try:
             graph = branch.repository.get_graph()
@@ -62,7 +67,8 @@ class RevisionSpec_svn(RevisionSpec):
                     continue
             if len(ret) == 1:
                 revid = ret.pop()
-                history = list(branch.repository.iter_reverse_revision_history(revid))
+                history = list(graph.iter_lefthand_ancestry(
+                    revid, (NULL_REVISION,)))
                 history.reverse()
                 return RevisionInfo.from_revision_id(branch, revid, history)
             elif len(ret) == 0:
