@@ -1,5 +1,5 @@
 # Copyright (C) 2009 Lukas Lalinsky <lalinsky@gmail.com>
-# Copyright (C) 2009 Jelmer Vernooij <jelmer@samba.org>
+# Copyright (C) 2009-2010 Jelmer Vernooij <jelmer@samba.org>
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -31,12 +31,6 @@ from subvertpy import (
     )
 
 
-try:
-    BaseMergeDirective = merge_directive.BaseMergeDirective
-except AttributeError:
-    BaseMergeDirective = merge_directive._BaseMergeDirective
-
-
 class SvnDiffTree(_mod_diff.DiffTree):
     """Provides a text representation between two trees, formatted for svn."""
 
@@ -51,9 +45,12 @@ class SvnDiffTree(_mod_diff.DiffTree):
 
     def _get_svn_rev_info(self, tree, file_id):
         try:
-            rev = tree.inventory[file_id].revision
+            rev = tree.get_file_revision(file_id)
         except errors.NoSuchId:
             return '(revision 0)'
+        except AttributeError:
+            # Not a revision tree
+            rev = None
         if rev is None:
             return '(working copy)'
         repo = self._get_repository()
@@ -66,16 +63,20 @@ class SvnDiffTree(_mod_diff.DiffTree):
                 return '(revision %d)' % info[0][2]
         return '(working copy)'
 
-    def _write_contents_diff(self, path, old_version, old_contents, new_version, new_contents):
+    def _write_contents_diff(self, path, old_version, old_contents,
+            new_version, new_contents):
         if path is None:
             return
         self.to_file.write("Index: %s\n" % path)
         self.to_file.write("=" * 67 + "\n")
         old_label = '%s\t%s' % (path, old_version)
         new_label = '%s\t%s' % (path, new_version)
-        _mod_diff.internal_diff(old_label, old_contents,
-                                new_label, new_contents,
-                                self.to_file)
+        try:
+            _mod_diff.internal_diff(old_label, old_contents,
+                                    new_label, new_contents,
+                                    self.to_file)
+        except errors.BinaryFile:
+            self.to_file.write("Cannot display: file contains binary data.\n")
 
     def _write_properties_diff(self, path, old_properties, new_properties):
         if new_properties is None:
@@ -98,7 +99,8 @@ class SvnDiffTree(_mod_diff.DiffTree):
             elif new_value is None:
                 self.to_file.write("Removed: %s\n\t-%s\n" % (name, old_value))
             else:
-                self.to_file.write("Changed: %s\n\t-%s\n\t+%s\n" % (name, old_value, new_value))
+                self.to_file.write("Changed: %s\n\t-%s\n\t+%s\n" % (
+                    name, old_value, new_value))
 
     def _get_file_properties(self, tree, path, kind, executable):
         if kind in (None, "directory"):
@@ -113,9 +115,9 @@ class SvnDiffTree(_mod_diff.DiffTree):
     def _get_file_contents(self, tree, file_id, path, kind):
         if kind in (None, "directory"):
             return None
-        if kind == "symlink":
-            return ["link %s" % tree.get_symlink_target(file_id)]
-        return tree.get_file(file_id).readlines()
+        from bzrlib.plugins.svn.mapping import get_svn_file_contents
+        f = get_svn_file_contents(tree, kind, file_id)
+        return f.readlines()
 
     def _show_diff(self, specific_files, extra_trees):
         iterator = self.new_tree.iter_changes(self.old_tree,
@@ -136,22 +138,31 @@ class SvnDiffTree(_mod_diff.DiffTree):
             oldpath_encoded = get_encoded_path(paths[0])
             newpath_encoded = get_encoded_path(paths[1])
             renamed = (parent[0], name[0]) != (parent[1], name[1])
-            old_properties = self._get_file_properties(self.old_tree, oldpath_encoded, kind[0], executable[0])
-            new_properties = self._get_file_properties(self.new_tree, newpath_encoded, kind[1], executable[1])
+            old_properties = self._get_file_properties(self.old_tree,
+                    oldpath_encoded, kind[0], executable[0])
+            new_properties = self._get_file_properties(self.new_tree,
+                    newpath_encoded, kind[1], executable[1])
             old_version = self._get_svn_rev_info(self.old_tree, file_id)
             new_version = self._get_svn_rev_info(self.new_tree, file_id)
 
             if oldpath_encoded == newpath_encoded:
                 if changed_content:
-                    old_contents = self._get_file_contents(self.old_tree, file_id, oldpath_encoded, kind[0])
-                    new_contents = self._get_file_contents(self.new_tree, file_id, newpath_encoded, kind[1])
-                    self._write_contents_diff(oldpath_encoded, old_version, old_contents, new_version, new_contents)
+                    old_contents = self._get_file_contents(self.old_tree,
+                            file_id, oldpath_encoded, kind[0])
+                    new_contents = self._get_file_contents(self.new_tree,
+                            file_id, newpath_encoded, kind[1])
+                    self._write_contents_diff(oldpath_encoded, old_version,
+                            old_contents, new_version, new_contents)
                 self._write_properties_diff(oldpath_encoded, old_properties, new_properties)
             else:
-                old_contents = self._get_file_contents(self.old_tree, file_id, oldpath_encoded, kind[0])
-                new_contents = self._get_file_contents(self.new_tree, file_id, newpath_encoded, kind[1])
-                self._write_contents_diff(oldpath_encoded, old_version, old_contents, new_version, [])
-                self._write_contents_diff(newpath_encoded, old_version, [], new_version, new_contents)
+                old_contents = self._get_file_contents(self.old_tree, file_id,
+                        oldpath_encoded, kind[0])
+                new_contents = self._get_file_contents(self.new_tree, file_id,
+                        newpath_encoded, kind[1])
+                self._write_contents_diff(oldpath_encoded, old_version,
+                        old_contents, new_version, [])
+                self._write_contents_diff(newpath_encoded, old_version, [],
+                        new_version, new_contents)
                 self._write_properties_diff(newpath_encoded, {}, new_properties)
 
             has_changes = (changed_content or renamed)
@@ -159,7 +170,7 @@ class SvnDiffTree(_mod_diff.DiffTree):
         return has_changes
 
 
-class SvnMergeDirective(BaseMergeDirective):
+class SvnMergeDirective(merge_directive.BaseMergeDirective):
 
     def to_lines(self):
         return self.patch.splitlines(True)
