@@ -1,5 +1,5 @@
 # Copyright (C) 2005-2009 Jelmer Vernooij <jelmer@samba.org>
- 
+
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
@@ -35,25 +35,29 @@ from bzrlib.trace import mutter
 
 from bzrlib.plugins.svn import (
     errors as svn_errors,
-    mapping,
+    )
+from bzrlib.plugins.svn.mapping import (
+    SVN_REVPROP_BZR_SKIP,
+    mapping_registry,
     )
 from bzrlib.plugins.svn.transport import (
     check_dirs_exist,
     create_branch_prefix,
     )
 
+
 def reverse_dict(orig):
     ret = {}
     for k, v in orig.iteritems():
         ret.setdefault(v, []).append(k)
     return ret
 
+
 def _resolve_reverse_tags_fallback(branch, reverse_tag_revmetas):
-    """Determine the revids for tags that were not found in the branch 
+    """Determine the revids for tags that were not found in the branch
     ancestry.
     """
-    ret = {}
-    # For anything that's not in the branches' ancestry, just use 
+    # For anything that's not in the branches' ancestry, just use
     # the latest mapping
     for (revmeta, names) in reverse_tag_revmetas.iteritems():
         mapping = revmeta.get_original_mapping() or branch.mapping
@@ -63,19 +67,19 @@ def _resolve_reverse_tags_fallback(branch, reverse_tag_revmetas):
             continue
         for name in names:
             assert isinstance(name, str)
-            ret[name.decode("utf-8")] = revid
-    return ret
+            yield (name.decode("utf-8"), (revmeta, mapping, revid))
 
-def _resolve_tags_svn_ancestry(branch, tag_revmetas):
+
+def resolve_tags_svn_ancestry(branch, tag_revmetas):
     """Resolve a name -> revmeta dictionary to a name -> revid dict.
 
-    The tricky bit here is figuring out what mapping to use. Preferably, 
-    we should be using whatever mapping makes the tag useful to the user, 
-    which generally means using the revid that is in the ancestry of the 
+    The tricky bit here is figuring out what mapping to use. Preferably,
+    we should be using whatever mapping makes the tag useful to the user,
+    which generally means using the revid that is in the ancestry of the
     branch.
 
-    As fallback, we will use the mapping used by the branch. That will 
-    however cause question marks to show up rather than revno's in 
+    As fallback, we will use the mapping used by the branch. That will
+    however cause question marks to show up rather than revno's in
     "bzr tags".
     """
     if len(tag_revmetas) == 0:
@@ -95,7 +99,7 @@ def _resolve_tags_svn_ancestry(branch, tag_revmetas):
                 return ret
             for name in reverse_tag_revmetas[revmeta]:
                 assert isinstance(name, str)
-                ret[name.decode("utf-8")] = revmeta.get_revision_id(mapping)
+                ret[name.decode("utf-8")] = (revmeta, mapping, revmeta.get_revision_id(mapping))
             del reverse_tag_revmetas[revmeta]
     finally:
         pb.finished()
@@ -112,10 +116,11 @@ class ReverseTagDict(object):
         self._by_foreign_revid = {}
         self._tags = tags
         for name, revmeta in tags.iteritems():
-            self._by_foreign_revid.setdefault(revmeta.get_foreign_revid(), []).append(name)
+            self._by_foreign_revid.setdefault(revmeta.metarev.get_foreign_revid(), []).append(name)
 
     def _lookup_revid(self, revid):
-        return self.repository.lookup_bzr_revision_id(revid, project=self.project)
+        return self.repository.lookup_bzr_revision_id(revid,
+            project=self.project)
 
     def has_key(self, revid):
         foreign_revid, mapping = self._lookup_revid(revid)
@@ -125,12 +130,29 @@ class ReverseTagDict(object):
         foreign_revid, mapping = self._lookup_revid(revid)
         return self._by_foreign_revid.get(foreign_revid, default)
 
+    def __getitem__(self, key):
+        foreign_revid, mapping = self._lookup_revid(key)
+        return self._by_foreign_revid[foreign_revid]
+
     def items(self):
-        return _resolve_tags_svn_ancestry(self.branch, self._tags).items()
+        d = resolve_tags_svn_ancestry(self.branch, self._tags)
+        rev = {}
+        for key, (revmeta, mapping, revid) in d.iteritems():
+            rev.setdefault(revid, []).append(key)
+        return rev.items()
 
     def iteritems(self):
         return iter(self.items())
 
+    def __iter__(self):
+        return self.iterkeys()
+
+    def iterkeys(self):
+        return (k for (k, v) in self.iteritems())
+
+    def keys(self):
+        return list(self.iterkeys())
+
 
 class SubversionTags(BasicTags):
     """Subversion tags object."""
@@ -144,7 +166,7 @@ class SubversionTags(BasicTags):
         """Make sure that the container of these tags exists.
 
         :param parent: Parent path
-        :return: True if the container already existed, False if it had to 
+        :return: True if the container already existed, False if it had to
             be created.
         """
         if parent in self._parent_exists:
@@ -152,12 +174,12 @@ class SubversionTags(BasicTags):
         assert isinstance(parent, str)
         bp_parts = parent.split("/")
         existing_bp_parts = check_dirs_exist(
-                self.repository.transport, 
+                self.repository.transport,
                 bp_parts, self.repository.get_latest_revnum())
         if existing_bp_parts == bp_parts:
             self._parent_exists.add(parent)
             return True
-        create_branch_prefix(self.repository.transport, 
+        create_branch_prefix(self.repository.transport,
                 self._revprops("Add tags base directory."),
                 bp_parts, existing_bp_parts)
         self._parent_exists.add(parent)
@@ -175,7 +197,7 @@ class SubversionTags(BasicTags):
             return
         self._ensure_tag_parent_exists(parent)
         try:
-            current_from_foreign_revid = self._lookup_tag_revmeta(path).get_foreign_revid()
+            current_from_foreign_revid = self._lookup_tag_revmeta(path).metarev.get_foreign_revid()
             deletefirst = True
         except KeyError:
             current_from_foreign_revid = None
@@ -183,7 +205,8 @@ class SubversionTags(BasicTags):
         if current_from_foreign_revid == (from_uuid, from_bp, from_revnum):
             # Already present
             return
-        mutter("setting tag %s from %r (deletefirst: %r)", path, (from_uuid, from_bp, from_revnum), deletefirst)
+        mutter("setting tag %s from %r (deletefirst: %r)", path,
+               (from_uuid, from_bp, from_revnum), deletefirst)
         conn = self.repository.transport.get_connection(parent)
         try:
             ci = svn_errors.convert_svn_error(conn.get_commit_editor)(
@@ -193,7 +216,8 @@ class SubversionTags(BasicTags):
                 root = ci.open_root()
                 if deletefirst:
                     root.delete_entry(urlutils.basename(path))
-                tag_dir = root.add_directory(urlutils.basename(path), urlutils.join(self.repository.base, from_bp), from_revnum)
+                tag_dir = root.add_directory(urlutils.basename(path),
+                    urlutils.join(self.repository.base, from_bp), from_revnum)
                 tag_dir.close()
                 root.close()
             except:
@@ -206,18 +230,19 @@ class SubversionTags(BasicTags):
     def _revprops(self, message, tags_dict=None):
         """Create a revprops dictionary.
 
-        Optionally sets bzr:skip to slightly optimize fetching of this revision later.
+        Optionally sets bzr:skip to slightly optimize fetching of this
+        revision later.
         """
         revprops = {properties.PROP_REVISION_LOG: message, }
         if self.repository.transport.has_capability("commit-revprops"):
-            revprops[mapping.SVN_REVPROP_BZR_SKIP] = ""
+            revprops[SVN_REVPROP_BZR_SKIP] = ""
         return revprops
 
     def _lookup_tag_revmeta(self, path):
         revnum = self.repository.get_latest_revnum()
         if self.repository.transport.check_path(path, revnum) == NODE_NONE:
             raise KeyError
-        tip, mapping = self.repository._iter_reverse_revmeta_mapping_history(
+        tip, mapping = self.repository._revmeta_provider._iter_reverse_revmeta_mapping_history(
             path, revnum, to_revnum=0, mapping=self.branch.mapping).next()
         return tip.get_tag_revmeta(mapping)
 
@@ -232,10 +257,11 @@ class SubversionTags(BasicTags):
     def _get_tag_dict_revmeta(self, from_revnum=None, to_revnum=None):
         """Get a name -> revmeta dictionary."""
         if from_revnum is None or from_revnum == 0:
-            return self.repository.find_tags(project=self.branch.project, 
+            return self.repository.find_tags(
+                    project=self.branch.project,
                     layout=self.branch.layout,
                     mapping=self.branch.mapping,
-                    revnum=self.branch._revnum)
+                    revnum=to_revnum)
         elif from_revnum <= to_revnum:
             return self.repository.find_tags_between(
                     project=self.branch.project,
@@ -248,18 +274,23 @@ class SubversionTags(BasicTags):
 
     def _resolve_tags_ancestry(self, tag_revmetas, graph, last_revid):
         """Resolve a name -> revmeta dictionary using the ancestry of a branch.
+
+        :param tag_revmetas: Dictionary mapping names to revmeta objects
+        :param graph: Graph object
+        :param last_revid: Branch last revid
+        :return: Dictionary mapping unicode tag names to revision ids
         """
         ret = {}
         reverse_tag_revmetas = reverse_dict(tag_revmetas)
         foreign_revid_map = {}
         for revmeta in reverse_tag_revmetas:
-            foreign_revid_map[revmeta.get_foreign_revid()] = revmeta
+            foreign_revid_map[revmeta.metarev.get_foreign_revid()] = revmeta
         for revid, _ in graph.iter_ancestry([last_revid]):
             if len(reverse_tag_revmetas) == 0:
                 # No more tag revmetas to resolve, just return immediately
                 return ret
             try:
-                foreign_revid, m = mapping.mapping_registry.parse_revision_id(revid)
+                foreign_revid, m = mapping_registry.parse_revision_id(revid)
             except InvalidRevisionId:
                 continue
             if not foreign_revid in foreign_revid_map:
@@ -267,19 +298,21 @@ class SubversionTags(BasicTags):
             revmeta = foreign_revid_map[foreign_revid]
             for name in reverse_tag_revmetas[revmeta]:
                 assert isinstance(name, str)
-                ret[name.decode("utf-8")] = revid
+                ret[name.decode("utf-8")] = (revmeta, m, revid)
             del reverse_tag_revmetas[revmeta]
-        ret.update(_resolve_reverse_tags_fallback(self.branch, reverse_tag_revmetas))
+        ret.update(_resolve_reverse_tags_fallback(self.branch,
+                                                  reverse_tag_revmetas))
         return ret
 
     def get_tag_dict(self):
         tag_revmetas = self._get_tag_dict_revmeta()
-        return _resolve_tags_svn_ancestry(self.branch, tag_revmetas)
+        d = resolve_tags_svn_ancestry(self.branch, tag_revmetas)
+        return dict([(k, v[2]) for (k, v) in d.iteritems()])
 
     def get_reverse_tag_dict(self):
         """Returns a dict with revisions as keys
            and a list of tags for that revision as value"""
-        return ReverseTagDict(self.branch, self.repository, 
+        return ReverseTagDict(self.branch, self.repository,
                               self._get_tag_dict_revmeta(),
                               self.branch.project)
 
@@ -288,10 +321,12 @@ class SubversionTags(BasicTags):
         parent = urlutils.dirname(path)
         conn = self.repository.transport.get_connection(parent)
         try:
-            if conn.check_path(urlutils.basename(path), self.repository.get_latest_revnum()) != subvertpy.NODE_DIR:
+            if conn.check_path(urlutils.basename(path),
+                    self.repository.get_latest_revnum()) != subvertpy.NODE_DIR:
                 raise NoSuchTag(tag_name)
-            ci = svn_errors.convert_svn_error(conn.get_commit_editor)(self._revprops("Remove tag %s" % tag_name.encode("utf-8"),
-                                        {tag_name: ""}))
+            ci = svn_errors.convert_svn_error(conn.get_commit_editor)(
+                    self._revprops("Remove tag %s" % tag_name.encode("utf-8"),
+                    {tag_name: ""}))
             try:
                 root = ci.open_root()
                 root.delete_entry(urlutils.basename(path))
@@ -313,10 +348,11 @@ class SubversionTags(BasicTags):
             if k not in dest_dict:
                 self.delete_tag(k)
 
-    def merge_to(self, to_tags, overwrite=False, _from_revnum=None, _to_revnum=None):
+    def merge_to(self, to_tags, overwrite=False, _from_revnum=None,
+                 _to_revnum=None):
         """Copy tags between repositories if necessary and possible.
-        
-        This method has common command-line behaviour about handling 
+
+        This method has common command-line behaviour about handling
         error cases.
 
         All new definitions are copied across, except that tags that already
@@ -325,7 +361,7 @@ class SubversionTags(BasicTags):
         :param to_tags: Branch to receive these tags
         :param overwrite: Overwrite conflicting tags in the target branch
 
-        :returns: A list of tags that conflicted, each of which is 
+        :returns: A list of tags that conflicted, each of which is
             (tagname, source_target, dest_target), or None if no copying was
             done.
         """
@@ -342,11 +378,12 @@ class SubversionTags(BasicTags):
         to_tags.branch.lock_write()
         try:
             graph = to_tags.branch.repository.get_graph()
-            source_dict = self._resolve_tags_ancestry(tag_revmetas, 
+            source_dict = self._resolve_tags_ancestry(tag_revmetas,
                 graph, to_tags.branch.last_revision())
             dest_dict = to_tags.get_tag_dict()
-            result, conflicts = self._reconcile_tags(source_dict, dest_dict,
-                                                     overwrite)
+            result, conflicts = self._reconcile_tags(
+                dict([(k, v[2]) for (k, v) in source_dict.items()]),
+                dest_dict, overwrite)
             if result != dest_dict:
                 to_tags._set_tag_dict(result)
         finally:
