@@ -1,5 +1,7 @@
+# Tests for repository
+
+# Copyright (C) 2010 Jelmer Vernooij <jelmer@samba.org>
 
- 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
@@ -17,13 +19,13 @@ from bzrlib import urlutils
 from bzrlib.branch import Branch
 from bzrlib.bzrdir import BzrDir
 from bzrlib.errors import NoSuchRevision
-from bzrlib.inventory import Inventory
 from bzrlib.repository import Repository
 from bzrlib.revision import NULL_REVISION
 from bzrlib.tests import TestSkipped, TestNotApplicable
 
-from bzrlib.plugins.svn import errors as svn_errors, format
+from bzrlib.plugins.svn import errors as svn_errors
 from subvertpy import ra
+from bzrlib.plugins.svn.repository import find_branches_between
 from bzrlib.plugins.svn.layout.standard import (
     TrunkLayout, RootLayout, CustomLayout,)
 from bzrlib.plugins.svn.mapping import mapping_registry
@@ -38,12 +40,9 @@ class TestSubversionMappingRepositoryWorks(SubversionTestCase):
     def setUp(self):
         super(TestSubversionMappingRepositoryWorks, self).setUp()
         self._old_mapping = mapping_registry._get_default_key()
+        self.addCleanup(mapping_registry.set_default, self._old_mapping)
         mapping_registry.set_default(self.mapping_name)
 
-    def tearDown(self):
-        super(TestSubversionMappingRepositoryWorks, self).tearDown()
-        mapping_registry.set_default(self._old_mapping)
-
     def test_get_branch_log(self):
         repos_url = self.make_repository("a")
         cb = self.get_commit_editor(repos_url)
@@ -58,27 +57,6 @@ class TestSubversionMappingRepositoryWorks(SubversionTestCase):
             ('', {'': ('A', None, -1, NODE_DIR)}, 0)],
             repos._revmeta_provider.iter_reverse_branch_changes("", 1, 0))
 
-    def test_iter_changes_parent_rename(self):
-        repos_url = self.make_repository("a")
-
-        dc = self.get_commit_editor(repos_url)
-        foo = dc.add_dir("foo")
-        foo.add_dir("foo/bar")
-        dc.close()
-
-        dc = self.get_commit_editor(repos_url)
-        dc.add_dir("bla", "foo", 1)
-        dc.close()
-
-        repos = Repository.open(repos_url)
-        try:
-            repos.set_layout(CustomLayout(["bla/bar"]))
-        except svn_errors.LayoutUnusable:
-            raise TestNotApplicable
-        ret = list(repos._revmeta_provider.iter_changes('bla/bar', 2, 0))
-        self.assertEquals(1, len(ret))
-        self.assertEquals("foo/bar", ret[0][0])
-
     def test_set_make_working_trees(self):
         repos_url = self.make_repository("a")
         repos = Repository.open(repos_url)
@@ -92,14 +70,18 @@ class TestSubversionMappingRepositoryWorks(SubversionTestCase):
         rm_provider = repos._revmeta_provider
         ret = repos.get_fileid_map(rm_provider.get_revision("", 0), mapping).as_dict()
         if mapping.is_branch(""):
-            self.assertEqual({u"": (mapping.generate_file_id((repos.uuid, "", 0), u""), mapping.revision_id_foreign_to_bzr((repos.uuid, "", 0)), None)}, ret)
+            self.assertEqual(
+                {u"": (mapping.generate_file_id((repos.uuid, "", 0), u""),
+                       mapping.revision_id_foreign_to_bzr((repos.uuid, "", 0)),
+                       None)},
+                ret)
         else:
             self.assertEquals({}, ret)
 
     def test_add_revision(self):
         repos_url = self.make_repository("a")
         repos = Repository.open(repos_url)
-        self.assertRaises(NotImplementedError, repos.add_revision, "revid", 
+        self.assertRaises(NotImplementedError, repos.add_revision, "revid",
                 None)
 
     def test_has_signature_for_revision_id_no(self):
@@ -122,7 +104,7 @@ class TestSubversionMappingRepositoryWorks(SubversionTestCase):
         repos_url = self.make_repository("a")
         repos = Repository.open(repos_url)
         repos.set_layout(RootLayout())
-        self.assertRaises(NoSuchRevision, list, 
+        self.assertRaises(NoSuchRevision, list,
                repos._revmeta_provider.iter_reverse_branch_changes("/", 20, 0))
 
     def test_follow_branch_switched_parents(self):
@@ -133,7 +115,7 @@ class TestSubversionMappingRepositoryWorks(SubversionTestCase):
         trunk = pykleur.add_dir("pykleur/trunk")
         nested = trunk.add_dir("pykleur/trunk/pykleur")
         cb.close() #1
-        
+
         cb = self.get_commit_editor(repos_url)
         pykleur = cb.open_dir("pykleur")
         trunk = pykleur.open_dir("pykleur/trunk")
@@ -194,7 +176,7 @@ class TestSubversionMappingRepositoryWorks(SubversionTestCase):
             repos.set_layout(CustomLayout(["pygments"]))
         except svn_errors.LayoutUnusable:
             raise TestNotApplicable
-        changes = [c for c,m in repos._iter_reverse_revmeta_mapping_history("pygments", 2, 0, repos.get_mapping())]
+        changes = [c for c,m in repos._revmeta_provider._iter_reverse_revmeta_mapping_history("pygments", 2, 0, repos.get_mapping())]
         if repos.get_mapping().is_branch("pykleur"):
             self.assertRevmetaLogEquals([('pygments',
                   {'pygments': ('A', 'pykleur', 1, NODE_DIR)},
@@ -217,8 +199,8 @@ class TestSubversionMappingRepositoryWorks(SubversionTestCase):
         repos = Repository.open(repos_url)
         repos.set_layout(TrunkLayout())
 
-        self.assertEqual(1, 
-                   len(set(repos.all_revision_ids(TrunkLayout()))))
+        self.assertEqual(1,
+               len(set(repos.all_revision_ids(TrunkLayout()))))
 
     def test_all_revs_empty(self):
         repos_url = self.make_repository("a")
@@ -251,7 +233,7 @@ class TestSubversionMappingRepositoryWorks(SubversionTestCase):
         repos.set_layout(TrunkLayout(0))
         mapping = repos.get_mapping()
         self.assertEqual(set([
-            repos.generate_revision_id(1, "trunk", mapping), 
+            repos.generate_revision_id(1, "trunk", mapping),
             repos.generate_revision_id(2, "branches/somebranch", mapping)]),
             set(repos.all_revision_ids()))
 
@@ -259,7 +241,7 @@ class TestSubversionMappingRepositoryWorks(SubversionTestCase):
         repos_url = self.make_repository("a")
         repos = Repository.open(repos_url)
         repos.set_layout(RootLayout())
-        self.assertEqual(set([repos.generate_revision_id(0, '', repos.get_mapping())]), 
+        self.assertEqual(set([repos.generate_revision_id(0, '', repos.get_mapping())]),
               set(repos.all_revision_ids(repos.get_layout())))
 
     def test_follow_history_empty_branch(self):
@@ -273,7 +255,7 @@ class TestSubversionMappingRepositoryWorks(SubversionTestCase):
 
         repos = Repository.open(repos_url)
         repos.set_layout(TrunkLayout(0))
-        self.assertEqual(set([repos.generate_revision_id(1, 'trunk', repos.get_mapping())]), 
+        self.assertEqual(set([repos.generate_revision_id(1, 'trunk', repos.get_mapping())]),
                 set(repos.all_revision_ids(repos.get_layout())))
 
     def test_follow_history_follow(self):
@@ -304,7 +286,7 @@ class TestSubversionMappingRepositoryWorks(SubversionTestCase):
             'dc/branches': None,
             'dc/branches/brancha': None,
             'dc/branches/branchab': None,
-            'dc/branches/brancha/data': "data", 
+            'dc/branches/brancha/data': "data",
             "dc/branches/branchab/data":"data"})
         self.client_add("dc/branches")
         self.client_commit("dc", "My Message")
@@ -322,7 +304,7 @@ class TestSubversionMappingRepositoryWorks(SubversionTestCase):
         self.build_tree({
             'dc/branches/brancha': None,
             'dc/branches/branchab': None,
-            'dc/branches/brancha/data': "data", 
+            'dc/branches/brancha/data': "data",
             "dc/branches/branchab/data":"data"})
         self.client_add("dc/branches/brancha")
         self.client_commit("dc", "My Message")
@@ -341,7 +323,7 @@ class TestSubversionMappingRepositoryWorks(SubversionTestCase):
         self.build_tree({
             'dc/branches/brancha': None,
             'dc/branches/branchab': None,
-            'dc/branches/brancha/data': "data", 
+            'dc/branches/brancha/data': "data",
             "dc/branches/branchab/data":"data"})
         self.client_add("dc/branches")
         self.client_commit("dc", "My Message")
@@ -349,9 +331,9 @@ class TestSubversionMappingRepositoryWorks(SubversionTestCase):
         repos.set_layout(TrunkLayout(0))
         branches = repos.find_branches()
         self.assertEquals(2, len(branches))
-        self.assertEquals(urlutils.join(repos.base, "branches/brancha"), 
+        self.assertEquals(urlutils.join(repos.base, "branches/brancha"),
                           branches[1].base)
-        self.assertEquals(urlutils.join(repos.base, "branches/branchab"), 
+        self.assertEquals(urlutils.join(repos.base, "branches/branchab"),
                           branches[0].base)
 
     def test_find_tags(self):
@@ -367,7 +349,7 @@ class TestSubversionMappingRepositoryWorks(SubversionTestCase):
         repos.set_layout(TrunkLayout(0))
         tags = repos.find_tags("")
         rm_provider = repos._revmeta_provider
-        self.assertEquals({"brancha": rm_provider.get_revision("tags/brancha", 1), 
+        self.assertEquals({"brancha": rm_provider.get_revision("tags/brancha", 1),
                            "branchab": rm_provider.get_revision("tags/branchab", 1)}, tags)
 
     def test_find_tags_unmodified(self):
@@ -412,12 +394,12 @@ class TestSubversionMappingRepositoryWorks(SubversionTestCase):
         rm_provider = repos._revmeta_provider
         self.assertEquals({"brancha": rm_provider.get_revision("tags/brancha", 3)}, tags)
 
-    def test_find_branchpaths_moved(self):
+    def test_find_branches_between_moved(self):
         repos_url = self.make_client("a", "dc")
         self.build_tree({
             'dc/tmp/branches/brancha': None,
             'dc/tmp/branches/branchab': None,
-            'dc/tmp/branches/brancha/data': "data", 
+            'dc/tmp/branches/brancha/data': "data",
             "dc/tmp/branches/branchab/data":"data"})
         self.client_add("dc/tmp")
         self.client_commit("dc", "My Message")
@@ -427,11 +409,11 @@ class TestSubversionMappingRepositoryWorks(SubversionTestCase):
         repos = Repository.open(repos_url)
         repos.set_layout(TrunkLayout(0))
 
-        self.assertEqual([("tags/branchab", 2, True), 
-                          ("tags/brancha", 2, True)], 
-                list(repos.find_branchpaths(TrunkLayout(0), from_revnum=2, to_revnum=0)))
+        self.assertEqual([("tags/branchab", 2, True),
+                          ("tags/brancha", 2, True)],
+                list(find_branches_between(repos._log, repos.transport, TrunkLayout(0), from_revnum=2, to_revnum=0)))
 
-    def test_find_branchpaths_start_revno(self):
+    def test_find_branches_between_start_revno(self):
         repos_url = self.make_repository("a")
 
         dc = self.get_commit_editor(repos_url)
@@ -448,9 +430,9 @@ class TestSubversionMappingRepositoryWorks(SubversionTestCase):
         repos.set_layout(TrunkLayout(0))
 
         self.assertEqual([("branches/branchb", 2, True)],
-                list(repos.find_branchpaths(TrunkLayout(0), from_revnum=2, to_revnum=2)))
+                list(find_branches_between(repos._log, repos.transport, TrunkLayout(0), from_revnum=2, to_revnum=2)))
 
-    def test_find_branchpaths_file_moved_from_nobranch(self):
+    def test_find_branches_between_file_moved_from_nobranch(self):
         repos_url = self.make_client("a", "dc")
         self.build_tree({
             'dc/tmp/trunk': None,
@@ -462,9 +444,10 @@ class TestSubversionMappingRepositoryWorks(SubversionTestCase):
         self.client_delete("dc/tmp/branches/somefile")
         self.client_commit("dc", "My Message 2")
 
-        Repository.open(repos_url).find_branchpaths(TrunkLayout(2), from_revnum=2, to_revnum=0)
+        repos = Repository.open(repos_url)
+        find_branches_between(repos._log, repos.transport, TrunkLayout(2), from_revnum=2, to_revnum=0)
 
-    def test_find_branchpaths_deleted_from_nobranch(self):
+    def test_find_branches_between_deleted_from_nobranch(self):
         repos_url = self.make_client("a", "dc")
         self.build_tree({
             'dc/tmp/trunk': None,
@@ -476,15 +459,16 @@ class TestSubversionMappingRepositoryWorks(SubversionTestCase):
         self.client_delete("dc/tmp/branches/somefile")
         self.client_commit("dc", "My Message 2")
 
-        Repository.open(repos_url).find_branchpaths(TrunkLayout(1), from_revnum=2, to_revnum=0)
+        repos = Repository.open(repos_url)
+        find_branches_between(repos._log, repos.transport, TrunkLayout(1), from_revnum=2, to_revnum=0)
 
-    def test_find_branchpaths_moved_nobranch(self):
+    def test_find_branches_between_moved_nobranch(self):
         repos_url = self.make_client("a", "dc")
         self.build_tree({
             'dc/tmp/nested/foobar': None,
             'dc/tmp/nested/branches/brancha': None,
             'dc/tmp/nested/branches/branchab': None,
-            'dc/tmp/nested/branches/brancha/data': "data", 
+            'dc/tmp/nested/branches/brancha/data': "data",
             "dc/tmp/nested/branches/branchab/data":"data"})
         self.client_add("dc/tmp")
         self.client_commit("dc", "My Message")
@@ -494,38 +478,38 @@ class TestSubversionMappingRepositoryWorks(SubversionTestCase):
         repos = Repository.open(repos_url)
         repos.set_layout(TrunkLayout(1))
 
-        self.assertEqual([("t2/branches/brancha", 2, True), 
-                          ("t2/branches/branchab", 2, True)], 
-                list(repos.find_branchpaths(TrunkLayout(1), to_revnum=2, from_revnum=2)))
+        self.assertEqual([("t2/branches/brancha", 2, True),
+                          ("t2/branches/branchab", 2, True)],
+                list(find_branches_between(repos._log, repos.transport, TrunkLayout(1), to_revnum=2, from_revnum=2)))
 
-    def test_find_branchpaths_root(self):
+    def test_find_branches_between_root(self):
         repos_url = self.make_repository("a")
 
         repos = Repository.open(repos_url)
         repos.set_layout(RootLayout())
 
-        self.assertEqual([("", 0, True)], 
-                list(repos.find_branchpaths(RootLayout(), to_revnum=0, from_revnum=0)))
+        self.assertEqual([("", 0, True)],
+                list(find_branches_between(repos._log, repos.transport, RootLayout(), to_revnum=0, from_revnum=0)))
 
-    def test_find_branchpaths_no_later(self):
+    def test_find_branches_between_no_later(self):
         repos_url = self.make_repository("a")
 
         repos = Repository.open(repos_url)
         repos.set_layout(RootLayout())
 
-        self.assertEqual([("", 0, True)], 
-                list(repos.find_branchpaths(RootLayout(), to_revnum=0, from_revnum=0)))
+        self.assertEqual([("", 0, True)],
+                list(find_branches_between(repos._log, repos.transport, RootLayout(), to_revnum=0, from_revnum=0)))
 
-    def test_find_branchpaths_trunk_empty(self):
+    def test_find_branches_between_trunk_empty(self):
         repos_url = self.make_repository("a")
 
         repos = Repository.open(repos_url)
         repos.set_layout(TrunkLayout(0))
 
-        self.assertEqual([], 
-                list(repos.find_branchpaths(TrunkLayout(0), to_revnum=0, from_revnum=0)))
+        self.assertEqual([],
+                list(find_branches_between(repos._log, repos.transport, TrunkLayout(0), to_revnum=0, from_revnum=0)))
 
-    def test_find_branchpaths_trunk_one(self):
+    def test_find_branches_between_trunk_one(self):
         repos_url = self.make_repository("a")
 
         repos = Repository.open(repos_url)
@@ -536,10 +520,10 @@ class TestSubversionMappingRepositoryWorks(SubversionTestCase):
         trunk.add_file("trunk/foo").modify("data")
         dc.close()
 
-        self.assertEqual([("trunk", 1, True)], 
-                list(repos.find_branchpaths(TrunkLayout(0), from_revnum=1, to_revnum=0)))
+        self.assertEqual([("trunk", 1, True)],
+                list(find_branches_between(repos._log, repos.transport, TrunkLayout(0), from_revnum=1, to_revnum=0)))
 
-    def test_find_branchpaths_removed(self):
+    def test_find_branches_between_removed(self):
         repos_url = self.make_repository("a")
 
         repos = Repository.open(repos_url)
@@ -554,10 +538,10 @@ class TestSubversionMappingRepositoryWorks(SubversionTestCase):
         dc.delete("trunk")
         dc.close()
 
-        self.assertEqual([("trunk", 1, True)], 
-                list(repos.find_branchpaths(TrunkLayout(0), from_revnum=2, to_revnum=0)))
-        self.assertEqual([("trunk", 1, True)], 
-                list(repos.find_branchpaths(TrunkLayout(0), from_revnum=1, to_revnum=0)))
+        self.assertEqual([("trunk", 1, True)],
+                list(find_branches_between(repos._log, repos.transport, TrunkLayout(0), from_revnum=2, to_revnum=0)))
+        self.assertEqual([("trunk", 1, True)],
+                list(find_branches_between(repos._log, repos.transport, TrunkLayout(0), from_revnum=1, to_revnum=0)))
 
     def test_has_revision(self):
         bzrdir = self.make_client_and_bzrdir('d', 'dc')
@@ -634,19 +618,19 @@ class TestSubversionMappingRepositoryWorks(SubversionTestCase):
         self.build_tree({'dc/trunk/foo': "data2"})
         repository = Repository.open(repos_url)
         repository.set_layout(TrunkLayout(0))
-        self.client_set_prop("dc/trunk", "svk:merge", 
+        self.client_set_prop("dc/trunk", "svk:merge",
             "%s:/branches/foo:1\n" % repository.uuid)
         self.client_commit("dc", "Second Message")
         mapping = repository.get_mapping()
         self.assertEqual((repository.generate_revision_id(1, "trunk", mapping),
-            repository.generate_revision_id(1, "branches/foo", mapping)), 
+            repository.generate_revision_id(1, "branches/foo", mapping)),
                 repository.get_revision(
                     repository.generate_revision_id(2, "trunk", mapping)).parent_ids)
-    
+
     def test_get_revision(self):
         repos_url = self.make_client('d', 'dc')
         repository = Repository.open(repos_url)
-        self.assertRaises(NoSuchRevision, repository.get_revision, 
+        self.assertRaises(NoSuchRevision, repository.get_revision,
                 "nonexisting")
         self.build_tree({'dc/foo': "data"})
         self.client_add("dc/foo")
@@ -660,7 +644,7 @@ class TestSubversionMappingRepositoryWorks(SubversionTestCase):
             repository.generate_revision_id(2, "", mapping))
         self.assertEqual((repository.generate_revision_id(1, "", mapping),),
                 rev.parent_ids)
-        self.assertEqual(rev.revision_id, 
+        self.assertEqual(rev.revision_id,
                 repository.generate_revision_id(2, "", mapping))
         self.assertEqual(author, rev.committer)
         self.assertIsInstance(rev.properties, dict)
@@ -671,117 +655,18 @@ class TestSubversionMappingRepositoryWorks(SubversionTestCase):
         mapping = repository.get_mapping()
         rev = repository.get_revision(
             repository.generate_revision_id(0, "", mapping))
-        self.assertEqual(repository.generate_revision_id(0, "", mapping), 
+        self.assertEqual(repository.generate_revision_id(0, "", mapping),
                          rev.revision_id)
         self.assertEqual("", rev.committer)
         self.assertEqual({}, rev.properties)
         self.assertEqual(0, rev.timezone)
 
-    def test_get_ancestry(self):
-        repos_url = self.make_repository('d')
-        repository = Repository.open(repos_url)
-        self.assertRaises(NoSuchRevision, repository.get_revision,
-                "nonexisting")
-
-        dc = self.get_commit_editor(repos_url)
-        dc.add_file("foo").modify("data")
-        dc.close()
-
-        dc = self.get_commit_editor(repos_url)
-        dc.open_file("foo").modify("data2")
-        dc.close()
-
-        dc = self.get_commit_editor(repos_url)
-        dc.open_file("foo").modify("data3")
-        dc.close()
-
-        repository = Repository.open(repos_url)
-        mapping = repository.get_mapping()
-        self.assertEqual([None, 
-            repository.generate_revision_id(0, "", mapping),
-            repository.generate_revision_id(1, "", mapping),
-            repository.generate_revision_id(2, "", mapping),
-            repository.generate_revision_id(3, "", mapping)],
-                repository.get_ancestry(
-                    repository.generate_revision_id(3, "", mapping)))
-        self.assertEqual([None, 
-            repository.generate_revision_id(0, "", mapping),
-            repository.generate_revision_id(1, "", mapping),
-            repository.generate_revision_id(2, "", mapping)],
-                repository.get_ancestry(
-                    repository.generate_revision_id(2, "", mapping)))
-        self.assertEqual([None,
-                    repository.generate_revision_id(0, "", mapping),
-                    repository.generate_revision_id(1, "", mapping)],
-                repository.get_ancestry(
-                    repository.generate_revision_id(1, "", mapping)))
-        self.assertEqual([None, repository.generate_revision_id(0, "", mapping)],
-                repository.get_ancestry(
-                    repository.generate_revision_id(0, "", mapping)))
-        self.assertEqual([None], repository.get_ancestry(NULL_REVISION))
-
-    def test_get_ancestry2(self):
-        repos_url = self.make_client('d', 'dc')
-        self.build_tree({'dc/foo': "data"})
-        self.client_add("dc/foo")
-        self.client_commit("dc", "My Message")
-        self.build_tree({'dc/foo': "data2"})
-        self.client_commit("dc", "Second Message")
-        repository = Repository.open(repos_url)
-        mapping = repository.get_mapping()
-        self.assertEqual([None, repository.generate_revision_id(0, "", mapping)],
-                repository.get_ancestry(
-                    repository.generate_revision_id(0, "", mapping)))
-        self.assertEqual([None, repository.generate_revision_id(0, "", mapping),
-            repository.generate_revision_id(1, "", mapping)],
-                repository.get_ancestry(
-                    repository.generate_revision_id(1, "", mapping)))
-        self.assertEqual([None, 
-            repository.generate_revision_id(0, "", mapping),
-            repository.generate_revision_id(1, "", mapping),
-            repository.generate_revision_id(2, "", mapping)], 
-                repository.get_ancestry(
-                    repository.generate_revision_id(2, "", mapping)))
-
     def test_get_inventory(self):
         repos_url = self.make_repository('d')
         repository = Repository.open(repos_url)
-        self.assertRaises(NoSuchRevision, repository.get_inventory, 
+        self.assertRaises(NotImplementedError, repository.get_inventory,
                 "nonexisting")
 
-        dc = self.get_commit_editor(repos_url)
-        dc.add_file("foo").modify("data")
-        dc.add_file("blah").modify("other data")
-        dc.close()
-
-        dc = self.get_commit_editor(repos_url)
-        dc.open_file("foo").modify("data2")
-        bar = dc.add_dir("bar")
-        bar.add_file("bar/foo").modify("data3")
-        dc.close()
-
-        dc = self.get_commit_editor(repos_url)
-        dc.open_file("foo").modify("data3")
-        dc.close()
-
-        repository = Repository.open(repos_url)
-        repository.set_layout(RootLayout())
-        mapping = repository.get_mapping()
-        inv = repository.get_inventory(
-                repository.generate_revision_id(1, "", mapping))
-        self.assertIsInstance(inv, Inventory)
-        self.assertIsInstance(inv.path2id("foo"), basestring)
-        inv = repository.get_inventory(
-            repository.generate_revision_id(2, "", mapping))
-        self.assertEqual(repository.generate_revision_id(2, "", mapping), 
-                         inv[inv.path2id("foo")].revision)
-        self.assertEqual(repository.generate_revision_id(1, "", mapping), 
-                         inv[inv.path2id("blah")].revision)
-        self.assertIsInstance(inv, Inventory)
-        self.assertIsInstance(inv.path2id("foo"), basestring)
-        self.assertIsInstance(inv.path2id("bar"), basestring)
-        self.assertIsInstance(inv.path2id("bar/foo"), basestring)
-
     def test_generate_revision_id(self):
         repos_url = self.make_client('d', 'dc')
         self.build_tree({'dc/bla/bloe': None})
@@ -790,14 +675,14 @@ class TestSubversionMappingRepositoryWorks(SubversionTestCase):
         repository = Repository.open(repos_url)
         mapping = repository.get_mapping()
         self.assertEqual(
-               mapping.revision_id_foreign_to_bzr((repository.uuid, "bla/bloe", 1)), 
+               mapping.revision_id_foreign_to_bzr((repository.uuid, "bla/bloe", 1)),
             repository.generate_revision_id(1, "bla/bloe", mapping))
 
     def test_generate_revision_id_zero(self):
         repos_url = self.make_client('d', 'dc')
         repository = Repository.open(repos_url)
         mapping = repository.get_mapping()
-        self.assertEqual(mapping.revision_id_foreign_to_bzr((repository.uuid, "", 0)), 
+        self.assertEqual(mapping.revision_id_foreign_to_bzr((repository.uuid, "", 0)),
                 repository.generate_revision_id(0, "", mapping))
 
     def test_lookup_revision_id(self):
@@ -806,7 +691,7 @@ class TestSubversionMappingRepositoryWorks(SubversionTestCase):
         self.client_add("dc/bloe")
         self.client_commit("dc", "foobar")
         repository = Repository.open(repos_url)
-        self.assertRaises(NoSuchRevision, repository.lookup_bzr_revision_id, 
+        self.assertRaises(NoSuchRevision, repository.lookup_bzr_revision_id,
             "nonexisting")
         mapping = repository.get_mapping()
         self.assertEqual(((repository.uuid, "bloe", 1), mapping),
@@ -817,10 +702,10 @@ class TestSubversionMappingRepositoryWorks(SubversionTestCase):
         repos_url = self.make_client('d', 'dc')
         repository = Repository.open(repos_url)
         mapping = repository.get_mapping()
-        self.assertRaises(NoSuchRevision, 
-            repository.lookup_bzr_revision_id, 
+        self.assertRaises(NoSuchRevision,
+            repository.lookup_bzr_revision_id,
                 mapping.revision_id_foreign_to_bzr(("invaliduuid", "", 0)))
-        
+
     def test_check(self):
         repos_url = self.make_client('d', 'dc')
         self.build_tree({'dc/foo': "data"})
@@ -829,7 +714,7 @@ class TestSubversionMappingRepositoryWorks(SubversionTestCase):
         repository = Repository.open(repos_url)
         mapping = repository.get_mapping()
         repository.check([
-            repository.generate_revision_id(0, "", mapping), 
+            repository.generate_revision_id(0, "", mapping),
             repository.generate_revision_id(1, "", mapping)])
 
     def test_copy_contents_into(self):
@@ -848,7 +733,7 @@ class TestSubversionMappingRepositoryWorks(SubversionTestCase):
         to_bzrdir = BzrDir.create("e")
         to_repos = to_bzrdir.create_repository()
 
-        repository.copy_content_into(to_repos, 
+        repository.copy_content_into(to_repos,
                 repository.generate_revision_id(2, "", mapping))
 
         self.assertTrue(repository.has_revision(
@@ -861,9 +746,9 @@ class TestSubversionMappingRepositoryWorks(SubversionTestCase):
         if len(expected) != len(got):
             self.assertEquals(expected, got, msg)
         for (branch_path, changes, revnum), g in zip(expected, got):
-            self.assertEquals(revnum, g.revnum)
-            self.assertEquals(branch_path, g.branch_path)
-            self.assertChangedPathsEquals(changes, g.get_paths())
+            self.assertEquals(revnum, g.metarev.revnum)
+            self.assertEquals(branch_path, g.metarev.branch_path)
+            self.assertChangedPathsEquals(changes, g.metarev.paths)
 
     def test_fetch_property_change_only_trunk(self):
         repos_url = self.make_repository('d')
@@ -924,7 +809,7 @@ class TestSubversionMappingRepositoryWorks(SubversionTestCase):
 
         oldrepos = Repository.open(repos_url)
         oldrepos.set_layout(TrunkLayout(0))
-        dir = BzrDir.create("f",format=format.get_rich_root_format())
+        dir = BzrDir.create("f")
         newrepos = dir.create_repository()
         oldrepos.copy_content_into(newrepos)
 
@@ -962,7 +847,8 @@ class TestSubversionMappingRepositoryWorks(SubversionTestCase):
         repos_url = self.make_repository('d')
         repos = Repository.open(repos_url)
         repos.set_layout(RootLayout())
-        self.assertEquals(NULL_REVISION, repos._revmeta_provider.get_revision("", 0).get_lhs_parent_revid(repos.get_mapping()))
+        revmeta0 = repos._revmeta_provider.get_revision("", 0)
+        self.assertEquals(NULL_REVISION, revmeta0.get_lhs_parent_revid(repos.get_mapping(), None))
 
     def testlhs_revision_parent_first(self):
         repos_url = self.make_client('d', 'dc')
@@ -972,12 +858,14 @@ class TestSubversionMappingRepositoryWorks(SubversionTestCase):
         self.client_add("dc/adir")
         self.client_commit("dc", "Initial commit")
         mapping = repos.get_mapping()
-        self.assertEquals(repos.generate_revision_id(0, "", mapping), \
-                repos._revmeta_provider.get_revision("", 1).get_lhs_parent_revid(mapping))
+        revmeta0 = repos._revmeta_provider.get_revision("", 0)
+        revmeta1 = repos._revmeta_provider.get_revision("", 1)
+        self.assertEquals(repos.generate_revision_id(0, "", mapping),
+            revmeta1.get_lhs_parent_revid(mapping, revmeta0))
 
     def testlhs_revision_parent_simple(self):
         repos_url = self.make_client('d', 'dc')
-        self.build_tree({'dc/trunk/adir/afile': "data", 
+        self.build_tree({'dc/trunk/adir/afile': "data",
                          'dc/trunk/adir/stationary': None,
                          'dc/branches/abranch': None})
         self.client_add("dc/trunk")
@@ -988,8 +876,10 @@ class TestSubversionMappingRepositoryWorks(SubversionTestCase):
         repos = Repository.open(repos_url)
         repos.set_layout(TrunkLayout(0))
         mapping = repos.get_mapping()
+        revmeta1 = repos._revmeta_provider.get_revision("trunk", 1)
+        revmeta2 = repos._revmeta_provider.get_revision("trunk", 2)
         self.assertEquals(repos.generate_revision_id(1, "trunk", mapping), \
-                repos._revmeta_provider.get_revision("trunk", 2).get_lhs_parent_revid(mapping))
+                revmeta2.get_lhs_parent_revid(mapping, revmeta1))
 
     def testlhs_revision_parent_copied(self):
         repos_url = self.make_repository('d')
@@ -1000,11 +890,11 @@ class TestSubversionMappingRepositoryWorks(SubversionTestCase):
         adir = trunk.add_dir("py/trunk/adir")
         adir.add_file("py/trunk/adir/afile").modify("data")
         adir.add_dir("py/trunk/adir/stationary")
-        dc.close()
+        dc.close() #1
 
         dc = self.get_commit_editor(repos_url)
         dc.add_dir("de", "py")
-        dc.close()
+        dc.close() #2
 
         dc = self.get_commit_editor(repos_url)
         de = dc.open_dir("de")
@@ -1015,8 +905,9 @@ class TestSubversionMappingRepositoryWorks(SubversionTestCase):
         repos = Repository.open(repos_url)
         repos.set_layout(TrunkLayout(1))
         mapping = repos.get_mapping()
+        revmeta3 = repos._revmeta_provider.get_revision("de/trunk", 3)
         self.assertEquals(repos.generate_revision_id(1, "py/trunk", mapping), \
-                repos._revmeta_provider.get_revision("de/trunk", 3).get_lhs_parent_revid(mapping))
+                revmeta3.get_lhs_parent_revid(mapping, revmeta3.get_lhs_parent_revmeta(mapping)))
 
     def test_mainline_revision_copied(self):
         repos_url = self.make_repository('d')
@@ -1037,8 +928,9 @@ class TestSubversionMappingRepositoryWorks(SubversionTestCase):
         repos = Repository.open(repos_url)
         repos.set_layout(TrunkLayout(1))
         mapping = repos.get_mapping()
+        revmeta2 = repos._revmeta_provider.get_revision("de/trunk", 2)
         self.assertEquals(repos.generate_revision_id(1, "py/trunk", mapping), \
-                repos._revmeta_provider.get_revision("de/trunk", 2).get_lhs_parent_revid(mapping))
+                revmeta2.get_lhs_parent_revid(mapping, revmeta2.get_lhs_parent_revmeta(mapping)))
 
     def test_mainline_revision_nested_deleted(self):
         repos_url = self.make_repository('d')
@@ -1064,29 +956,9 @@ class TestSubversionMappingRepositoryWorks(SubversionTestCase):
         repos = Repository.open(repos_url)
         repos.set_layout(TrunkLayout(1))
         mapping = repos.get_mapping()
+        revmeta3 = repos._revmeta_provider.get_revision("de/trunk", 3)
         self.assertEquals(repos.generate_revision_id(1, "py/trunk", mapping),
-            repos._revmeta_provider.get_revision("de/trunk", 3).get_lhs_parent_revid(mapping))
-
-    def test_item_keys_introduced_by(self):
-        repos_url = self.make_repository('d')
-
-        cb = self.get_commit_editor(repos_url)
-        cb.add_file("foo").modify()
-        cb.close()
-
-        cb = self.get_commit_editor(repos_url)
-        cb.open_file("foo").modify()
-        cb.close()
-
-        b = Branch.open(repos_url)
-        mapping = b.repository.get_mapping()
-        ch = list(b.repository.item_keys_introduced_by([b.last_revision()]))
-        revid = b.last_revision()
-        self.assertEquals([
-            ('file', mapping.generate_file_id((b.repository.uuid, "", 1), u"foo"), set([revid])),
-            ('inventory', None, [revid]),
-            ('signatures', None, set([])),
-            ('revisions', None, [revid])], ch)
+            revmeta3.get_lhs_parent_revid(mapping, revmeta3.get_lhs_parent_revmeta(mapping)))
 
     def test_fetch_file_from_non_branch(self):
         repos_url = self.make_repository('d')
@@ -1112,5 +984,5 @@ class TestSubversionMappingRepositoryWorks(SubversionTestCase):
         mapping = oldrepos.get_mapping()
         branch = Branch.open("%s/trunk" % repos_url)
         self.assertEquals(branch.mapping, mapping)
-        self.assertEqual([oldrepos.generate_revision_id(2, "trunk", mapping)], 
+        self.assertEqual([oldrepos.generate_revision_id(2, "trunk", mapping)],
                      branch.revision_history())
