@@ -19,7 +19,9 @@ import os
 import sys
 
 import bzrlib.gpg
+from bzrlib import version_info as bzrlib_version
 from bzrlib.repository import Repository
+from bzrlib.tests import KnownFailure
 from bzrlib.tests.blackbox import ExternalBase
 
 from bzrlib.plugins.svn.convert import load_dumpfile
@@ -30,9 +32,8 @@ from bzrlib.plugins.svn.tests import SubversionTestCase
 class TestBranch(SubversionTestCase, ExternalBase):
 
     def setUp(self):
-        SubversionTestCase.setUp(self)
-        SubversionTestCase.tearDown(self)
         ExternalBase.setUp(self)
+        self._init_client()
 
     def tearDown(self):
         ExternalBase.tearDown(self)
@@ -50,7 +51,7 @@ class TestBranch(SubversionTestCase, ExternalBase):
         repos_url = self.make_client('d', 'de')
         self.commit_something(repos_url)
         self.run_bzr("branch %s dc" % repos_url)
-        self.check_output("2\n", "revno de")
+        self.assertEquals("2\n", self.run_bzr("revno de")[0])
 
     def test_branch_onerev_stacked(self):
         repos_url = self.make_client('d', 'de')
@@ -69,6 +70,15 @@ class TestBranch(SubversionTestCase, ExternalBase):
         repos_url = self.make_repository('d')
         self.run_bzr('pack %s' % repos_url)
 
+    def test_rmbranch(self):
+        repos_url = self.make_repository('d')
+
+        dc = self.get_commit_editor(repos_url)
+        dc.add_dir("trunk")
+        dc.close()
+
+        self.run_bzr("rmbranch %s/trunk" % repos_url)
+
     def test_push_create_prefix(self):
         repos_url = self.make_repository('d')
 
@@ -96,7 +106,7 @@ class TestBranch(SubversionTestCase, ExternalBase):
         self.build_tree({"dc/foo": "blaaaa"})
         self.run_bzr("commit -m msg dc")
         self.run_bzr("push -d dc %s" % repos_url)
-        self.check_output("", "status dc")
+        self.assertEquals("", self.run_bzr("status dc")[0])
 
     def test_push_empty_existing(self):
         repos_url = self.make_repository('d')
@@ -174,7 +184,7 @@ class TestBranch(SubversionTestCase, ExternalBase):
         self.run_bzr("add dc/bar")
         self.run_bzr("commit -m msg dc")
         self.run_bzr("push --overwrite -d dc %s/trunk" % repos_url)
-        self.check_output("", "status dc")
+        self.assertEquals("", self.run_bzr("status dc")[0])
 
     def test_dpush_empty(self):
         repos_url = self.make_repository('dp')
@@ -193,7 +203,7 @@ class TestBranch(SubversionTestCase, ExternalBase):
         self.build_tree({"dc/foo": "blaaaa"})
         self.run_bzr("commit -m msg dc")
         self.run_bzr("dpush -d dc %s" % repos_url)
-        self.check_output("", "status dc")
+        self.assertEquals("", self.run_bzr("status dc")[0])
 
     def test_dpush_new(self):
         repos_url = self.make_repository('d')
@@ -207,8 +217,8 @@ class TestBranch(SubversionTestCase, ExternalBase):
         self.run_bzr("add dc/foofile")
         self.run_bzr("commit -m msg dc")
         self.run_bzr("dpush -d dc %s" % repos_url)
-        self.check_output("3\n", "revno dc")
-        self.check_output("", "status dc")
+        self.assertEquals("3\n", self.run_bzr("revno dc")[0])
+        self.assertEquals("", self.run_bzr("status dc")[0])
 
     def test_dpush_wt_diff(self):
         repos_url = self.make_repository('d')
@@ -223,10 +233,15 @@ class TestBranch(SubversionTestCase, ExternalBase):
         self.run_bzr("commit -m msg dc")
         self.build_tree({"dc/foofile": "blaaaal"})
         self.run_bzr("dpush -d dc %s" % repos_url)
-        self.check_output('modified:\n  foofile\n', "status dc")
+        self.assertEquals('modified:\n  foofile\n',
+                self.run_bzr("status dc")[0])
 
     def test_info_workingtree(self):
         repos_url = self.make_client('d', 'dc')
+        # Info is broken against bzr 2.3
+        if bzrlib_version < (2, 4, 0):
+            raise KnownFailure("bzr info against svn repositories is "
+                "known broken against 2.3")
         self.run_bzr('info -v dc')
 
     def test_dumpfile(self):
@@ -375,7 +390,12 @@ Node-copyfrom-path: x
 
 
 """)
-        self.check_output("", 'svn-import --layout=none %s dc' % filename)
+        self.record_directory_isolation()
+        try:
+            self.assertEquals("",
+                self.run_bzr('svn-import --layout=none %s dc' % filename)[0])
+        finally:
+            self.enable_directory_isolation()
         load_dumpfile(filename, "realrepo")
         svnrepo = Repository.open("realrepo")
         self.assertEquals(uuid, svnrepo.uuid)
@@ -386,15 +406,15 @@ Node-copyfrom-path: x
             mapping.revision_id_foreign_to_bzr((uuid, "", 5))))
         self.assertTrue(newrepos.has_revision(
             mapping.revision_id_foreign_to_bzr((uuid, "", 1))))
-        inv1 = newrepos.get_inventory(
+        tree1 = newrepos.revision_tree(
                 mapping.revision_id_foreign_to_bzr((uuid, "", 1)))
-        inv2 = newrepos.get_inventory(
+        tree2 = newrepos.revision_tree(
                 mapping.revision_id_foreign_to_bzr((uuid, "", 5)))
-        self.assertNotEqual(inv1.path2id("y"), inv2.path2id("y"))
+        self.assertNotEqual(tree1.path2id("y"), tree2.path2id("y"))
 
     def test_svn_import_bzr_branch(self):
         self.run_bzr('init foo')
-        self.run_bzr_error(['bzr: ERROR: Source repository is not a Subversion repository.\n'], 
+        self.run_bzr_error(['bzr: ERROR: Source repository is not a Subversion repository.\n'],
                            ['svn-import', 'foo', 'dc'])
 
     def test_svn_import_bzr_repo(self):
@@ -409,7 +429,7 @@ Node-copyfrom-path: x
         dc.add_file("foo").modify("test")
         dc.add_file("bla").modify("ha")
         dc.close()
-        self.check_output("a/bla\na/foo\n", "ls a")
+        self.assertEquals("a/bla\na/foo\n", self.run_bzr("ls a")[0])
 
     def test_info_remote(self):
         repos_url = self.make_repository("a")
@@ -417,8 +437,9 @@ Node-copyfrom-path: x
         dc.add_file("foo").modify("test")
         dc.add_file("bla").modify("ha")
         dc.close()
-        self.check_output(
-                "Repository branch (format: subversion)\nLocation:\n  shared repository: a\n  repository branch: a\n", 'info a')
+        self.assertEquals(
+                "Repository branch (format: subversion)\nLocation:\n  shared repository: a\n  repository branch: a\n",
+                self.run_bzr('info a')[0])
 
     def test_lightweight_checkout_lightweight_checkout(self):
         repos_url = self.make_client("a", "dc")
@@ -433,7 +454,7 @@ Node-copyfrom-path: x
         self.build_tree({'de/foo': 'bla'})
         self.run_bzr("add de/foo")
         self.run_bzr("commit -m test de")
-        self.check_output("2\n", "revno de")
+        self.assertEquals("2\n", self.run_bzr("revno de")[0])
 
     # this method imported from bzrlib.tests.test_msgeditor:
     def make_fake_editor(self, message='test message from fed\\n'):
@@ -475,12 +496,13 @@ if len(sys.argv) == 2:
     def test_set_branching_scheme_local(self):
         self.make_fake_editor()
         repos_url = self.make_repository("a")
-        self.check_output("", 'svn-branching-scheme --set %s' % repos_url)
+        self.assertEquals("", self.run_bzr('svn-branching-scheme --set %s' % repos_url)[0])
 
     def test_set_branching_scheme_global(self):
         self.make_fake_editor()
         repos_url = self.make_repository("a")
-        self.check_output("", 'svn-branching-scheme --repository-wide --set %s' % repos_url)
+        self.assertEquals("",
+            self.run_bzr('svn-branching-scheme --repository-wide --set %s' % repos_url)[0])
 
     def monkey_patch_gpg(self):
         """Monkey patch the gpg signing strategy to be a loopback.
@@ -563,6 +585,12 @@ if len(sys.argv) == 2:
         os.chdir('bzr-trunk')
         self.run_bzr('merge %s/branches/my-branch' % svn_url)
         self.run_bzr('nick my-branch')
+        f = open('.bzr/branch/branch.conf', 'w')
+        try:
+            f.write("append_revisions_only = False\n")
+        finally:
+            f.close()
+
         self.run_bzr('ci -m "Merge my-branch"')
 
         os.chdir(cwd)
@@ -571,9 +599,9 @@ if len(sys.argv) == 2:
 
     def test_svn_import_format(self):
         svn_url = self.make_repository('d')
-        
+
         self.run_bzr('svn-import --format 1.9-rich-root %s dc' % svn_url)
-        self.check_output("Repository branch (format: 1.14-rich-root or 1.9-rich-root)\nLocation:\n  shared repository: dc\n  repository branch: dc\n\nRelated branches:\n  parent branch: d\n", 'info dc')
+        self.assertEquals("Repository branch (format: 1.14-rich-root)\nLocation:\n  shared repository: dc\n  repository branch: dc\n\nRelated branches:\n  parent branch: d\n", self.run_bzr('info dc')[0])
 
     def test_svn_layout(self):
         svn_url = self.make_repository('d')
@@ -581,5 +609,59 @@ if len(sys.argv) == 2:
         dc = self.get_commit_editor(svn_url)
         dc.add_dir("trunk")
         dc.close()
-        
-        self.check_output('Repository root: %s\nLayout: root\nBranch path: \nPush merged revisions: False\n' % svn_url, 'svn-layout %s' % svn_url)
+
+        self.assertEquals(
+            'Repository root: %s\n'
+            'Layout: root\n'
+            'Branch path: \n'
+            'No tag support\n'
+            'No custom branch support\n'
+            'Push merged revisions: False\n' % svn_url,
+                self.run_bzr('svn-layout %s' % svn_url)[0])
+
+    def test_svn_branches(self):
+        svn_url = self.make_repository('d')
+
+        dc = self.get_commit_editor(svn_url)
+        dc.add_dir("trunk")
+        dc.close()
+
+        dc = self.get_commit_editor(svn_url)
+        branches = dc.add_dir("branches")
+        branches.add_dir("branches/somebranch", "trunk")
+        tags = dc.add_dir("tags")
+        tags.add_dir("tags/release-1.0", "trunk")
+        dc.close()
+
+        self.assertEquals(
+            'Branches:\n'
+            'branches/somebranch (somebranch)\n'
+            'trunk (trunk)\n'
+            'Tags:\n'
+            'tags/release-1.0 (release-1.0)\n',
+            self.run_bzr('svn-branches --layout trunk %s' % svn_url)[0])
+
+    def test_diff(self):
+        repos_url = self.make_client('d', 'dc')
+        self.build_tree_contents([("dc/file", "bar")])
+        self.client_add("dc/file")
+        self.run_bzr('diff dc', retcode=True)
+
+    def test_diff_svn(self):
+        repos_url = self.make_client('d', 'dc')
+        self.build_tree_contents([("dc/file", "bar")])
+        self.client_add("dc/file")
+        self.run_bzr('diff --format=svn dc', retcode=True)
+
+    def test_ls(self):
+        repos_url = self.make_client('d', 'dc')
+        self.build_tree_contents([("dc/file", "bar")])
+        self.client_add("dc/file")
+        self.assertEquals("dc/file\n",
+            self.run_bzr('ls dc', retcode=0)[0])
+
+    def test_log_v(self):
+        repos_url = self.make_client('d', 'dc')
+        self.commit_something(repos_url)
+        self.run_bzr('log -v d', retcode=0)
+        self.run_bzr('log -v dc', retcode=0)
