@@ -47,6 +47,7 @@ from bzrlib.plugins.svn.branch import (
 from bzrlib.plugins.svn.convert import load_dumpfile
 from bzrlib.plugins.svn.tests import SubversionTestCase
 
+
 class WorkingSubversionBranch(SubversionTestCase):
 
     def test_last_rev_rev_hist(self):
@@ -78,6 +79,27 @@ class WorkingSubversionBranch(SubversionTestCase):
         b = Branch.open(repos_url + "/trunk")
         self.assertEquals(["foo"], b.tags.get_tag_dict().keys())
 
+    def test_reverse_tags_dict(self):
+        repos_url = self.make_repository("a")
+
+        dc = self.get_commit_editor(repos_url)
+        dc.add_dir("trunk")
+        dc.close()
+
+        dc = self.get_commit_editor(repos_url)
+        tags = dc.add_dir("tags")
+        tags.add_dir("tags/foo", "trunk")
+        dc.close()
+
+        b = Branch.open(repos_url + "/trunk")
+        revid = b.repository.generate_revision_id(1, "trunk",
+            b.repository.get_mapping())
+
+        revtagdict = b.tags.get_reverse_tag_dict()
+        self.assertEquals([revid], revtagdict.keys())
+        self.assertEquals(["foo"], revtagdict[revid])
+        self.assertEquals([(revid, ["foo"])], revtagdict.items())
+
     def test_tags_other_project(self):
         repos_url = self.make_repository("a")
 
@@ -356,26 +378,6 @@ class WorkingSubversionBranch(SubversionTestCase):
         inter_branch.pull(stop_revision=revid1)
         self.assertEquals(revid1, new_branch.last_revision())
 
-    def test_interbranch_limited_pull(self):
-        svn_branch, (revid1, revid2) = self.make_tworev_branch()
-        new_branch = self.make_branch("b")
-        inter_branch = InterBranch.get(svn_branch, new_branch)
-        inter_branch.pull(limit=1)
-        self.assertEquals(revid1, new_branch.last_revision())
-        inter_branch.pull(limit=1)
-        self.assertEquals(revid2, new_branch.last_revision())
-
-    def test_interbranch_limited_overwrite(self):
-        svn_branch, (revid1, revid2) = self.make_tworev_branch()
-        new_branch = self.make_branch("b")
-        inter_branch = InterBranch.get(svn_branch, new_branch)
-        inter_branch.pull(limit=2)
-        self.assertEquals(revid2, new_branch.last_revision())
-        inter_branch.pull(limit=1, stop_revision=revid1)
-        self.assertEquals(revid2, new_branch.last_revision())
-        inter_branch.pull(limit=1, overwrite=True, stop_revision=revid1)
-        self.assertEquals(revid1, new_branch.last_revision())
-
     def test_get_branch_path_subdir(self):
         repos_url = self.make_repository("a")
 
@@ -409,6 +411,15 @@ class WorkingSubversionBranch(SubversionTestCase):
 
         self.assertEquals(["newtag"], newbranch.tags.get_tag_dict().keys())
 
+    def test_open_non_ascii_url(self):
+        repos_url = self.make_repository("a")
+
+        dc = self.get_commit_editor(repos_url)
+        dc.add_dir('\xd0\xb4\xd0\xbe\xd0\xbc')
+        dc.close()
+
+        branch = Branch.open(repos_url + "/%D0%B4%D0%BE%D0%BC")
+
     def test_open_nonexistant(self):
         repos_url = self.make_repository("a")
         self.assertRaises(NotBranchError, Branch.open, repos_url + "/trunk")
@@ -425,14 +436,12 @@ class WorkingSubversionBranch(SubversionTestCase):
     def test_lookup_revision_id_unknown(self):
         repos_url = self.make_repository("a")
         branch = Branch.open(repos_url)
-        self.assertRaises(NoSuchRevision,
-                lambda: branch.lookup_bzr_revision_id("bla"))
+        self.assertRaises(NoSuchRevision, lambda: branch.lookup_bzr_revision_id("bla"))
 
     def test_lookup_revision_id(self):
         repos_url = self.make_repository("a")
         branch = Branch.open(repos_url)
-        self.assertEquals(0,
-                branch.lookup_bzr_revision_id(branch.last_revision()))
+        self.assertEquals(0, branch.lookup_bzr_revision_id(branch.last_revision()))
 
     def test_set_parent(self):
         repos_url = self.make_repository('a')
@@ -443,8 +452,7 @@ class WorkingSubversionBranch(SubversionTestCase):
         repos_url = self.make_repository('a')
         bzrdir = BzrDir.open(repos_url)
         branch = bzrdir.open_branch()
-        self.assertEqual(branch.generate_revision_id(0),
-                         branch.last_revision())
+        self.assertEqual(branch.generate_revision_id(0), branch.last_revision())
 
         dc = self.get_commit_editor(repos_url)
         dc.add_file("foo").modify()
@@ -471,8 +479,14 @@ class WorkingSubversionBranch(SubversionTestCase):
 
     def test_set_revision_history_empty(self):
         repos_url = self.make_repository('a')
-        branch = Branch.open(repos_url)
-        self.assertRaises(NotImplementedError, branch.set_revision_history, [])
+        dc = self.get_commit_editor(repos_url)
+        trunk = dc.add_dir("trunk")
+        trunk.add_file('trunk/foo').modify()
+        dc.close()
+
+        branch = Branch.open(repos_url+"/trunk")
+        branch.set_revision_history([])
+        self.assertEquals(NULL_REVISION, branch.last_revision())
 
     def test_set_revision_history_ghost(self):
         repos_url = self.make_repository('a')
@@ -511,6 +525,7 @@ class WorkingSubversionBranch(SubversionTestCase):
         self.assertEquals(orig_history[:-1], branch.revision_history())
 
     def test_break_lock(self):
+        # duplicated by bzrlib.tests.per_branch.test_break_lock
         repos_url = self.make_repository('a')
         branch = Branch.open(repos_url)
         branch.break_lock()
@@ -526,6 +541,7 @@ class WorkingSubversionBranch(SubversionTestCase):
         self.assertFalse(branch.get_physical_lock_status())
 
     def test_set_push_location(self):
+        # duplicated by bt.per_branch.TestBranchPushLocations.test_set_push_location
         repos_url = self.make_repository('a')
         branch = Branch.open(repos_url)
         branch.set_push_location("http://bar/bloe")
@@ -536,12 +552,14 @@ class WorkingSubversionBranch(SubversionTestCase):
         self.assertEqual(None, branch.get_parent())
 
     def test_get_push_location(self):
+        # duplicated by bt.per_branch.TestBranchPushLocations.test_get_push_location_unset
         repos_url = self.make_repository('a')
         branch = Branch.open(repos_url)
         self.assertIs(None, branch.get_push_location())
 
     def test_revision_id_to_revno_none(self):
         """The None revid should map to revno 0."""
+        # duplicated by bt.per_branch.test_revision_id_to_revno
         repos_url = self.make_repository('a')
         branch = Branch.open(repos_url)
         self.assertEquals(0, branch.revision_id_to_revno(NULL_REVISION))
@@ -549,6 +567,7 @@ class WorkingSubversionBranch(SubversionTestCase):
     def test_revision_id_to_revno_nonexistant(self):
         """revision_id_to_revno() should raise NoSuchRevision if
         the specified revision did not exist in the branch history."""
+        # duplicated by bt.per_branch.test_revision_id_to_revno
         repos_url = self.make_repository('a')
         branch = Branch.open(repos_url)
         self.assertRaises(NoSuchRevision, branch.revision_id_to_revno, "bla")
@@ -826,7 +845,7 @@ foohosts""")
         newbranch.lock_read()
         tree = newbranch.repository.revision_tree(oldbranch.generate_revision_id(7))
 
-        host_fileid = tree.inventory.path2id("hosts")
+        host_fileid = tree.path2id("hosts")
 
         self.assertVersionsPresentEquals(newbranch.repository.texts,
                                         host_fileid, [
@@ -834,7 +853,6 @@ foohosts""")
             oldbranch.generate_revision_id(7)])
         newbranch.unlock()
 
-
     def test_fetch_odd(self):
         repos_url = self.make_repository('d')
 
@@ -884,7 +902,7 @@ foohosts""")
         transaction = newbranch.repository.get_transaction()
         newbranch.repository.lock_read()
         texts = newbranch.repository.texts
-        host_fileid = tree.inventory.path2id("hosts")
+        host_fileid = tree.path2id("hosts")
         mapping = oldbranch.repository.get_mapping()
         self.assertVersionsPresentEquals(texts, host_fileid, [
             mapping.revision_id_foreign_to_bzr((uuid, "trunk", 1)),
@@ -1040,8 +1058,6 @@ foohosts""")
                 olddir.open_branch().last_revision(),
                 newdir.open_branch().last_revision())
 
-
-
     def test_ghost_workingtree(self):
         # Looks like bazaar has trouble creating a working tree of a
         # revision that has ghost parents
@@ -1060,7 +1076,7 @@ foohosts""")
         newdir = olddir.sprout('dc')
         newdir.find_repository().get_revision(
                 newdir.open_branch().last_revision())
-        newdir.find_repository().get_inventory(
+        newdir.find_repository().revision_tree(
                 newdir.open_branch().last_revision())
 
 
@@ -1070,12 +1086,8 @@ class BranchFormatTests(TestCase):
         super(BranchFormatTests, self).setUp()
         self.format = SvnBranchFormat()
 
-    def test_initialize(self):
-        self.assertRaises(NotImplementedError, self.format.initialize, None)
-
     def test_get_format_string(self):
-        self.assertEqual("Subversion Smart Server",
-                         self.format.get_format_string())
+        self.assertRaises(NotImplementedError, self.format.get_format_string)
 
     def test_get_format_description(self):
         self.assertEqual("Subversion Smart Server",
@@ -1090,3 +1102,59 @@ class ForeignTestsBranchFactory(object):
     def make_branch(self, transport):
         subvertpy.repos.create(transport.local_abspath("."))
         return Branch.open(transport.base)
+
+
+class TestInterBranchFetch(SubversionTestCase):
+
+    def make_branch(self, relpath):
+        # The inherited make_branch is broken, thanks to the make_repository
+        # from subvertpy.
+        bzrdir = self.make_bzrdir(relpath)
+        bzrdir._find_or_create_repository(True)
+        return bzrdir.create_branch()
+
+    def make_tworev_branch(self):
+        self.repos_url = self.make_repository("a")
+
+        dc = self.get_commit_editor(self.repos_url)
+        trunk = dc.add_dir("trunk")
+        trunk.add_file('trunk/foo')
+        dc.close()
+
+        b = Branch.open(self.repos_url+"/trunk")
+        mapping = b.repository.get_mapping()
+        uuid = b.repository.uuid
+        revid1 = mapping.revision_id_foreign_to_bzr((uuid, 'trunk', 1))
+        return b, revid1
+
+    def test_fetch_stop_revision(self):
+        svn_branch, revid1 = self.make_tworev_branch()
+        new_branch = self.make_branch("b")
+        inter_branch = InterBranch.get(svn_branch, new_branch)
+        inter_branch.fetch(stop_revision=revid1)
+        self.assertEquals(set([revid1]), set(new_branch.repository.all_revision_ids()))
+
+    def test_fetch_no_stop_revision(self):
+        svn_branch, revid1 = self.make_tworev_branch()
+        new_branch = self.make_branch("b")
+        inter_branch = InterBranch.get(svn_branch, new_branch)
+        inter_branch.fetch()
+        self.assertEquals(set([revid1]), set(new_branch.repository.all_revision_ids()))
+
+    def test_fetch_tags(self):
+        svn_branch, revid1 = self.make_tworev_branch()
+
+        dc = self.get_commit_editor(self.repos_url)
+        tags = dc.add_dir("tags")
+        tags.add_dir("tags/foo")
+        dc.close()
+
+        mapping = svn_branch.repository.get_mapping()
+        uuid = svn_branch.repository.uuid
+        revid2 = mapping.revision_id_foreign_to_bzr((uuid, 'tags/foo', 2))
+
+        new_branch = self.make_branch("b")
+        inter_branch = InterBranch.get(svn_branch, new_branch)
+        inter_branch.fetch(fetch_tags=True)
+        self.assertEquals(set([revid1, revid2]),
+                set(new_branch.repository.all_revision_ids()))
