@@ -18,9 +18,12 @@
 
 """Commit and push tests."""
 
-from copy import copy
 import os
-from subvertpy import ra
+from subvertpy import (
+    ERR_FS_TXN_OUT_OF_DATE,
+    SubversionException,
+    ra,
+    )
 from subvertpy.properties import time_to_cstring
 
 from bzrlib.branch import (
@@ -29,26 +32,41 @@ from bzrlib.branch import (
     )
 from bzrlib.bzrdir import BzrDir
 from bzrlib.errors import (
-    DivergedBranches,
     BzrError,
+    DivergedBranches,
+    NoSuchId,
+    )
+from bzrlib.inventory import (
+    InventoryFile,
+    InventoryLink,
     )
 from bzrlib.repository import Repository
 from bzrlib.tests import (
     TestCase,
-    TestSkipped,
     )
+try:
+    from bzrlib.tests.features import (
+        UnicodeFilenameFeature
+        )
+except ImportError: # bzr < 2.5
+    from bzrlib.tests import (
+        UnicodeFilenameFeature
+        )
 from bzrlib.trace import mutter
 from bzrlib.workingtree import WorkingTree
 
-from bzrlib.plugins.svn import format
 from bzrlib.plugins.svn.commit import (
+    file_editor_send_prop_changes,
     set_svn_revprops,
     _revision_id_to_svk_feature,
     )
-from bzrlib.plugins.svn.errors import RevpropChangeFailed
+from bzrlib.plugins.svn.errors import (
+    RevpropChangeFailed,
+    )
 from bzrlib.plugins.svn.mapping import mapping_registry
 from bzrlib.plugins.svn.tests import SubversionTestCase
 
+
 class TestNativeCommit(SubversionTestCase):
 
     def test_simple_commit(self):
@@ -59,13 +77,13 @@ class TestNativeCommit(SubversionTestCase):
         revid = wt.commit(message="data")
         self.assertEqual(wt.branch.generate_revision_id(1), revid)
         self.client_update("dc")
-        self.assertEqual(wt.branch.generate_revision_id(1), 
+        self.assertEqual(wt.branch.generate_revision_id(1),
                 wt.branch.last_revision())
         wt = WorkingTree.open("dc")
-        new_inventory = wt.branch.repository.get_inventory(
+        new_tree = wt.branch.repository.revision_tree(
                             wt.branch.last_revision())
-        self.assertTrue(new_inventory.has_filename("foo"))
-        self.assertTrue(new_inventory.has_filename("foo/bla"))
+        self.assertTrue(new_tree.has_filename("foo"))
+        self.assertTrue(new_tree.has_filename("foo/bla"))
 
     def test_commit_message(self):
         self.make_client('d', 'dc')
@@ -96,22 +114,19 @@ class TestNativeCommit(SubversionTestCase):
         self.assertEqual(wt.branch.last_revision(), new_revision.revision_id)
 
     def test_commit_unicode_filename(self):
+        self.requireFeature(UnicodeFilenameFeature)
         self.make_client('d', 'dc')
-        try:
-            self.build_tree({u'dc/I²C': "data"})
-        except UnicodeError:
-            raise TestSkipped("This platform does not support unicode symlinks")
+        self.build_tree({u'dc/I²C': "data"})
         self.client_add(u"dc/I²C".encode("utf-8"))
         wt = WorkingTree.open("dc")
         wt.commit(message="data")
 
-
     def test_commit_local(self):
         self.make_client('d', 'dc')
         self.build_tree({'dc/foo/bla': "data"})
         self.client_add("dc/foo")
         wt = WorkingTree.open("dc")
-        self.assertRaises(BzrError, wt.commit, 
+        self.assertRaises(BzrError, wt.commit,
                 message="data", local=True)
 
     def test_commit_committer(self):
@@ -137,6 +152,41 @@ class TestNativeCommit(SubversionTestCase):
         self.assertEqual(wt.branch.last_revision(), new_revision.revision_id)
         self.assertEqual(u"føø", new_revision.message)
 
+    def test_commit_out_of_date(self):
+        repos_url = self.make_repository('d')
+        dc = self.get_commit_editor(repos_url)
+        trunk = dc.add_dir("trunk")
+        somedir = trunk.add_dir("trunk/somedir")
+        somefile = somedir.add_file("trunk/somedir/somefile").modify()
+        dc.close()
+
+        dc = self.get_commit_editor(repos_url)
+
+        br = Branch.open(repos_url+"/trunk")
+        br.get_config().set_user_option("append_revisions_only", "False")
+        builder = br.get_commit_builder([br.last_revision()])
+        builder.finish_inventory()
+
+        trunk = dc.open_dir("trunk")
+        somedir = trunk.open_dir("trunk/somedir")
+        somefile = somedir.open_file("trunk/somedir/somefile").modify()
+        dc.close()
+
+        e = self.assertRaises(SubversionException, builder.commit, "foo")
+
+        self.assertEquals(ERR_FS_TXN_OUT_OF_DATE, e.args[1])
+
+    def test_lossy_commit_builder(self):
+        repos_url = self.make_repository('d')
+        branch = Branch.open(repos_url)
+        cb = branch.repository.get_commit_builder(branch,
+            [branch.last_revision()], branch.get_config(),
+            revision_id="foorevid", lossy=True)
+        revid = cb.commit("msg")
+        self.assertEquals(
+            branch.repository.generate_revision_id(1, "", branch.mapping),
+            revid)
+
     def test_commit_update(self):
         self.make_client('d', 'dc')
         self.build_tree({'dc/foo/bla': "data"})
@@ -166,8 +216,9 @@ class TestNativeCommit(SubversionTestCase):
         self.assertEquals('/foo', paths["/bar"][1])
         self.assertEquals(1, paths["/bar"][2])
         svnrepo = Repository.open(repos_url)
-        self.assertEquals({u"bar": oldid}, 
-                svnrepo._revmeta_provider.get_revision("", 2).get_fileid_overrides(svnrepo.get_mapping()))
+        revmeta = svnrepo._revmeta_provider.get_revision("", 2)
+        self.assertEquals({u"bar": oldid},
+                revmeta.get_fileid_overrides(svnrepo.get_mapping()))
 
     def test_commit_rename_file_from_directory(self):
         repos_url = self.make_client('d', 'dc')
@@ -208,19 +259,14 @@ class TestNativeCommit(SubversionTestCase):
 
         branch = Branch.open(repos_url+"/trunk")
         foobranch = Branch.open(repos_url+"/tags/foo")
-        builder = branch.get_commit_builder([branch.last_revision(), foobranch.last_revision()], 
-                revision_id="my-revision-id")
-        tree = branch.repository.revision_tree(branch.last_revision())
-        new_tree = copy(tree)
-        ie = new_tree.inventory.root
-        ie.revision = None
-        builder.record_entry_contents(ie, [tree.inventory], '', new_tree, 
-                                      None)
+        builder = branch.get_commit_builder(
+            [branch.last_revision(), foobranch.last_revision()],
+            revision_id="my-revision-id")
         builder.finish_inventory()
         builder.commit("foo")
 
         self.assertEqual("/tags/foo:2-3\n",
-            self.client_get_prop("%s/trunk" % repos_url, 
+            self.client_get_prop("%s/trunk" % repos_url,
                 "svn:mergeinfo", 4))
 
         try:
@@ -244,7 +290,7 @@ class TestNativeCommit(SubversionTestCase):
         self.client_update('sc')
         os.mkdir('sc/de/trunk')
         self.client_add('sc/de/trunk')
-        mv(('sc/de/foo', 'sc/de/trunk'), ('sc/de/bar', 'sc/de/trunk')) #2
+        mv(('sc/de/foo', 'sc/de/trunk/foo'), ('sc/de/bar', 'sc/de/trunk/bar')) #2
         mv(('sc/de', 'sc/pyd'))  #3
         self.client_delete('sc/pyd/trunk/foo')
         self.client_commit('sc', '.') #4
@@ -265,7 +311,7 @@ class TestNativeCommit(SubversionTestCase):
 
         self.client_update('pyd')
         self.assertEqual("data", open('pyd/bar').read())
-        
+
 
 class TestPush(SubversionTestCase):
 
@@ -282,15 +328,16 @@ class TestPush(SubversionTestCase):
         self.newdir = self.olddir.sprout("dc")
 
     def test_empty(self):
-        self.assertEqual(0, int(self.olddir.open_branch().pull(
-                                self.newdir.open_branch())))
+        result = self.olddir.open_branch().pull(self.newdir.open_branch())
+        self.assertEquals(2, result.old_revno)
+        self.assertEquals(2, result.new_revno)
 
     def test_empty_result(self):
         result = self.olddir.open_branch().pull(self.newdir.open_branch())
         self.assertIsInstance(result, PullResult)
         self.assertEqual(result.old_revno, self.olddir.open_branch().revno())
         self.assertEqual(result.master_branch, None)
-        self.assertEqual(result.source_branch.bzrdir.root_transport.base, 
+        self.assertEqual(result.source_branch.bzrdir.root_transport.base,
                          self.newdir.root_transport.base)
 
     def test_child(self):
@@ -298,8 +345,9 @@ class TestPush(SubversionTestCase):
         self.client_add("sc/foo/bar")
         self.client_commit("sc", "second message")
 
-        self.assertEqual(0, int(self.olddir.open_branch().pull(
-                                self.newdir.open_branch())))
+        result = self.olddir.open_branch().pull(self.newdir.open_branch())
+        self.assertEquals(3, result.old_revno)
+        self.assertEquals(3, result.new_revno)
 
     def test_diverged(self):
         self.build_tree({'sc/foo/bar': "data"})
@@ -313,33 +361,47 @@ class TestPush(SubversionTestCase):
         wt.add('file')
         wt.commit(message="Commit from Bzr")
 
-        self.assertRaises(DivergedBranches, 
+        self.assertRaises(DivergedBranches,
                           olddir.open_branch().pull,
                           self.newdir.open_branch())
 
     def test_unicode_filename(self):
-        try:
-            self.build_tree({u'dc/I²C': 'other data'})
-        except UnicodeError:
-            raise TestSkipped("This platform does not support unicode paths")
+        self.requireFeature(UnicodeFilenameFeature)
+        self.build_tree({u'dc/I²C': 'other data'})
         wt = self.newdir.open_workingtree()
         wt.add(u'I²C')
         wt.commit(message="Commit from Bzr")
-        self.assertEqual(1, int(self.olddir.open_branch().pull(
-                                self.newdir.open_branch())))
+        result = self.olddir.open_branch().pull(self.newdir.open_branch())
+        self.assertEqual(2, result.old_revno)
+        self.assertEquals(3, result.new_revno)
+
+    def test_add_file_to_unicode_folder(self):
+        self.requireFeature(UnicodeFilenameFeature)
+        self.build_tree({u'dc/I²C/': None})
+        wt = self.newdir.open_workingtree()
+        wt.add(u'I²C')
+        wt.commit(message="Commit from Bzr")
+        self.build_tree({u'dc/I²C/file': 'data'})
+        wt.add(u'I²C/file')
+        wt.commit(message="Commit from Bzr2")
+        result = self.olddir.open_branch().pull(self.newdir.open_branch())
+        self.assertEquals(2, result.old_revno)
+        self.assertEquals(4, result.new_revno)
+        owt = self.olddir.open_workingtree()
+        owt.update()
+        self.assertTrue(owt.has_filename(u'I²C/file'))
 
     def test_rename_from_unicode_filename(self):
-        try:
-            self.build_tree({u'dc/I²C': 'other data'})
-        except UnicodeError:
-            raise TestSkipped("This platform does not support symlink paths")
+        self.requireFeature(UnicodeFilenameFeature)
+        self.build_tree({u'dc/I²C': 'other data'})
         wt = self.newdir.open_workingtree()
         wt.add(u'I²C')
         wt.commit(message="Commit from Bzr")
         wt.rename_one(u'I²C', u'I²C2')
         wt.commit(message="Commit from Bzr2")
-        self.assertEqual(2, int(self.olddir.open_branch().pull(
-                                self.newdir.open_branch())))
+        result = self.olddir.open_branch().pull(self.newdir.open_branch())
+        self.assertEquals(2, result.old_revno)
+        self.assertEquals(4, result.new_revno)
 
     def test_change(self):
         self.build_tree({'dc/foo/bla': 'other data'})
@@ -350,15 +412,15 @@ class TestPush(SubversionTestCase):
 
         repos = self.olddir._find_repository()
         mapping = repos.get_mapping()
-        inv = repos.get_inventory(repos.generate_revision_id(2, "", mapping))
+        tree = repos.revision_tree(repos.generate_revision_id(2, "", mapping))
         self.assertEqual(repos.generate_revision_id(2, "", mapping),
-                         inv[inv.path2id('foo/bla')].revision)
+                         tree.get_file_revision(tree.path2id('foo/bla')))
         self.assertEqual(wt.branch.last_revision(),
           repos.generate_revision_id(2, "", mapping))
         self.assertEqual(wt.branch.last_revision(),
                         self.olddir.open_branch().last_revision())
-        self.assertEqual("other data", 
-            repos.revision_tree(repos.generate_revision_id(2, "", mapping)).get_file_text( inv.path2id("foo/bla")))
+        self.assertEqual("other data",
+            repos.revision_tree(repos.generate_revision_id(2, "", mapping)).get_file_text(tree.path2id("foo/bla")))
 
     def test_simple(self):
         self.build_tree({'dc/file': 'data'})
@@ -370,9 +432,9 @@ class TestPush(SubversionTestCase):
 
         repos = self.olddir._find_repository()
         mapping = repos.get_mapping()
-        inv = repos.get_inventory(repos.generate_revision_id(2, "", mapping))
-        self.assertTrue(inv.has_filename('file'))
-        self.assertEqual(wt.branch.last_revision(), 
+        tree = repos.revision_tree(repos.generate_revision_id(2, "", mapping))
+        self.assertTrue(tree.has_filename('file'))
+        self.assertEqual(wt.branch.last_revision(),
                 repos.generate_revision_id(2, "", mapping))
         self.assertEqual(repos.generate_revision_id(2, "", mapping),
                         self.olddir.open_branch().last_revision())
@@ -387,9 +449,9 @@ class TestPush(SubversionTestCase):
 
         repos = self.olddir._find_repository()
         mapping = repos.get_mapping()
-        inv = repos.get_inventory(repos.generate_revision_id(2, "", mapping))
-        self.assertTrue(inv.has_filename('file'))
-        self.assertEquals(wt.branch.last_revision(), 
+        tree = repos.revision_tree(repos.generate_revision_id(2, "", mapping))
+        self.assertTrue(tree.has_filename('file'))
+        self.assertEquals(wt.branch.last_revision(),
                          repos.generate_revision_id(2, "", mapping))
 
         self.assertEqual(repos.generate_revision_id(2, "", mapping),
@@ -526,15 +588,13 @@ class JoinedCommitTests(SubversionTestCase):
         self.build_tree({"dc/lala": "data"})
         wt.add(["lala"])
         wt.commit(message="init")
-        joinedwt = BzrDir.create_standalone_workingtree("dc/newdir", 
-            format=format.get_rich_root_format())
+        joinedwt = BzrDir.create_standalone_workingtree("dc/newdir")
         joinedwt.pull(Branch.open(repos_url+"/branches/newbranch"))
         wt.subsume(joinedwt)
         wt.commit(message="doe")
 
         self.olddir.open_branch().pull(self.newdir.open_branch())
         paths = self.client_log(repos_url, 4, 0)[4][0]
-        mutter('paths %r' % paths)
         self.assertEquals(('A', "/branches/newbranch", 2), paths["/trunk/newdir"])
 
 
@@ -606,9 +666,9 @@ class HeavyWeightCheckoutTests(SubversionTestCase):
         master_branch = Branch.open(repos_url)
         rm_provider = master_branch.repository._revmeta_provider
         mapping = master_branch.repository.get_mapping()
-        self.assertEquals({u"file": oldid}, 
+        self.assertEquals({u"file": oldid},
                 rm_provider.get_revision("", 1).get_fileid_overrides(mapping))
-        self.assertEquals({u"file2": oldid}, 
+        self.assertEquals({u"file2": oldid},
                 rm_provider.get_revision("", 2).get_fileid_overrides(mapping))
         tree1 = master_branch.repository.revision_tree(revid1)
         tree2 = master_branch.repository.revision_tree(revid2)
@@ -635,10 +695,49 @@ class HeavyWeightCheckoutTests(SubversionTestCase):
         master_branch = Branch.open(repos_url)
         rm_provider = master_branch.repository._revmeta_provider
         mapping = master_branch.repository.get_mapping()
-        self.assertEquals({"dir": dirid, 
+        self.assertEquals({"dir": dirid,
                           "dir/file": fileid},
                           rm_provider.get_revision("", 1).get_fileid_overrides(mapping))
 
+    def test_push_merged_revision(self):
+        repos_url = self.make_repository("d")
+
+        dc = self.get_commit_editor(repos_url)
+        dc.add_dir("trunk")
+        dc.add_dir("branches")
+        dc.add_dir("tags")
+        dc.close() # 1
+
+        master_branch = Branch.open(repos_url+"/trunk")
+        trunk = master_branch.bzrdir.sprout("trunk")
+        trunk.open_branch().bind(master_branch)
+
+        config = master_branch.repository.get_config()
+        config.set_user_option("push_merged_revisions", "True")
+
+        feature = trunk.sprout("feature")
+        wt = feature.open_workingtree()
+        self.build_tree_contents([("feature/test.txt", "test")])
+        wt.add("test.txt")
+        revid2 = wt.commit("added test.txt") # 2
+
+        wt = trunk.open_workingtree()
+        wt.merge_from_branch(feature.open_branch(), to_revision=revid2)
+        wt.commit("merged feature") # 3
+
+        log = self.client_log(repos_url, 3, 0)
+        self.assertEquals({
+            '/branches': ('A', None, -1),
+            '/tags': ('A', None, -1),
+            '/trunk': ('A', None, -1)}, log[1][0])
+        self.assertEquals({
+            '/branches/feature': ('A', '/trunk', 1),
+            '/branches/feature/test.txt': ('A', None, -1)}, log[2][0])
+        self.assertEquals({
+            '/trunk': ('M', None, -1),
+            '/trunk/test.txt': ('A', '/branches/feature/test.txt', 2)},
+            log[3][0])
+
 
 class RevpropTests(SubversionTestCase):
 
@@ -650,12 +749,12 @@ class RevpropTests(SubversionTestCase):
         dc.close()
 
         repository = Repository.open(repos_url)
-        set_svn_revprops(repository, 1, {"svn:author": "Somebody", 
+        set_svn_revprops(repository, 1, {"svn:author": "Somebody",
                                         "svn:date": time_to_cstring(1000000*473385600)})
 
         self.assertEquals(1, repository.get_latest_revnum())
 
-        self.assertEquals(("Somebody", "1985-01-01T00:00:00.000000Z", "My commit"), 
+        self.assertEquals(("Somebody", "1985-01-01T00:00:00.000000Z", "My commit"),
                           self.client_log(repos_url, 1, 1)[1][1:])
 
     def test_change_revprops_disallowed(self):
@@ -666,15 +765,78 @@ class RevpropTests(SubversionTestCase):
         dc.close()
 
         repository = Repository.open(repos_url)
-        self.assertRaises(RevpropChangeFailed, 
+        self.assertRaises(RevpropChangeFailed,
                 lambda: set_svn_revprops(repository, 1, {"svn:author": "Somebody", "svn:date": time_to_cstring(1000000*473385600)}))
 
 
 class SvkTestCase(TestCase):
 
     def test_revid_svk_map(self):
-        self.assertEqual("auuid:/:6", 
+        self.assertEqual("auuid:/:6",
               _revision_id_to_svk_feature("svn-v3-undefined:auuid::6",
                   mapping_registry.parse_revision_id))
 
 
+class SendPropChangesTests(TestCase):
+
+    class MockTree(object):
+
+        def __init__(self, file_id, kind, name, executable):
+            self._file_id = file_id
+            self._name = name
+            self._executable = executable
+            self._kind = kind
+
+        def has_id(self, file_id):
+            return (self._file_id == file_id)
+
+        def is_executable(self, file_id):
+            if self._file_id == file_id:
+                return self._executable
+            raise NoSuchId
+
+        def kind(self, file_id):
+            if self._file_id == file_id:
+                return self._kind
+            raise NoSuchId
+
+    class RecordingFileEditor(object):
+
+        def __init__(self):
+            self._props = {}
+
+        def change_prop(self, name, value):
+            self._props[name] = value
+
+    def test_become_symlink(self):
+        ie2 = InventoryLink("myfileid", "name", "theroot")
+        editor = SendPropChangesTests.RecordingFileEditor()
+        file_editor_send_prop_changes(
+            self.MockTree("myfileid", "file", "name", True), "myfileid",
+            ie2, editor)
+        self.assertEquals(editor._props,
+            {"svn:special": "*", "svn:executable": None})
+
+    def test_become_executable(self):
+        tree = self.MockTree("myfileid", "file", "name", False)
+        ie2 = InventoryFile("myfileid", "name", "theroot")
+        ie2.executable = True
+        editor = SendPropChangesTests.RecordingFileEditor()
+        file_editor_send_prop_changes(tree, "myfileid",ie2, editor)
+        self.assertEquals(editor._props, {"svn:executable": "*"})
+
+    def test_unbecome_executable(self):
+        tree = self.MockTree("myfileid", "file", "name", True)
+        ie2 = InventoryFile("myfileid", "name", "theroot")
+        ie2.executable = False
+        editor = SendPropChangesTests.RecordingFileEditor()
+        file_editor_send_prop_changes(tree, "myfileid", ie2, editor)
+        self.assertEquals(editor._props, {"svn:executable": None})
+
+    def test_nothing_changes(self):
+        tree = self.MockTree("myfileid", "file", "name", False)
+        ie2 = InventoryFile("myfileid", "name", "theroot")
+        editor = SendPropChangesTests.RecordingFileEditor()
+        file_editor_send_prop_changes(tree, "myfileid",ie2, editor)
+        self.assertEquals(editor._props, {})
+
