@@ -18,7 +18,6 @@
 
 """Subversion fetch tests."""
 
-
 import os
 import shutil
 import sys
@@ -32,7 +31,6 @@ from bzrlib.bzrdir import (
     BzrDir,
     )
 from bzrlib.osutils import (
-    has_symlinks,
     sha_string,
     )
 from bzrlib.repository import (
@@ -42,15 +40,22 @@ from bzrlib.revision import (
     NULL_REVISION,
     )
 from bzrlib.tests import (
-    KnownFailure,
+    TestCase,
     TestSkipped,
     )
+try:
+    from bzrlib.tests.features import (
+        SymlinkFeature,
+        )
+except ImportError: # bzr < 2.5
+    from bzrlib.tests import (
+        SymlinkFeature,
+        )
 from bzrlib.trace import (
     mutter,
     )
 
 from bzrlib.plugins.svn import (
-    format,
     remote,
     )
 from bzrlib.plugins.svn.convert import (
@@ -60,7 +65,10 @@ from bzrlib.plugins.svn.errors import (
     InvalidFileName,
     )
 from bzrlib.plugins.svn.fetch import (
+   FetchRevisionFinder,
    InterFromSvnRepository,
+   check_filename,
+   chunks_start_with_link,
    )
 from bzrlib.plugins.svn.layout.standard import (
     RootLayout,
@@ -239,6 +247,38 @@ class TestFetchWorks(FetchTestCase):
         self.copy_content(oldrepos, newrepos)
         self.assertEquals("SIGNATURE", newrepos.get_signature_text(oldrepos.generate_revision_id(1, "trunk", oldrepos.get_mapping())))
 
+    def test_fetch_invalid_svk_rev(self):
+        repos_url = self.make_repository('d')
+
+        uuid = SvnRaTransport(repos_url).get_uuid()
+
+        dc = self.get_commit_editor(repos_url)
+        branches = dc.add_dir("branches")
+        dc.close()
+
+        dc = self.get_commit_editor(repos_url)
+        trunk = dc.add_dir("trunk")
+        trunk.add_file("trunk/bar").modify("data")
+        dc.close()
+
+        dc = self.get_commit_editor(repos_url)
+        trunk = dc.open_dir("trunk")
+        trunk.change_prop("svk:merge", "%s:/branches/trunk:1\n" % uuid)
+        dc.close()
+
+        oldrepos = Repository.open(repos_url)
+        oldrepos.set_layout(TrunkLayout(0))
+        dir = BzrDir.create("f")
+        newrepos = dir.create_repository()
+        rf = FetchRevisionFinder(oldrepos, newrepos)
+        rm = oldrepos._revmeta_provider.get_revision("trunk", 3)
+        mapping = oldrepos.get_mapping()
+        self.assertEquals((uuid, "trunk", 3), rm.metarev.get_foreign_revid())
+        self.assertEquals(
+            (mapping.revision_id_foreign_to_bzr((uuid, "branches/trunk", 1)), ),
+            rm.get_rhs_parents(mapping, rm.get_lhs_parent_revmeta(mapping)))
+        self.assertEquals([], list(rf.find_rhs_parents([(rm, mapping)])))
+
     def test_fetch_file_branch(self):
         repos_url = self.make_repository('d')
         dc = self.get_commit_editor(repos_url)
@@ -344,9 +384,9 @@ class TestFetchWorks(FetchTestCase):
         self.copy_content(oldrepos, newrepos)
         mapping = oldrepos.get_mapping()
         tree = newrepos.revision_tree(oldrepos.generate_revision_id(1, "", mapping))
-        self.assertEquals(3, len(tree.inventory))
+        self.assertEquals(3, len(tree.all_file_ids()))
         tree = newrepos.revision_tree(oldrepos.generate_revision_id(2, "", mapping))
-        self.assertEquals(1, len(tree.inventory))
+        self.assertEquals(1, len(tree.all_file_ids()))
 
     def test_fetch_local(self):
         repos_url = self.make_repository('d')
@@ -411,7 +451,7 @@ class TestFetchWorks(FetchTestCase):
         last_rev = oldrepos.generate_revision_id(3, "branch2",
             oldrepos.get_mapping())
         self.fetch(oldrepos, repo, last_rev)
-        self.assertEquals(repo.get_inventory(last_rev).root.file_id,
+        self.assertEquals(repo.revision_tree(last_rev).get_root_id(),
             oldrepos.get_mapping().generate_file_id(
                 (oldrepos.uuid, "branch1", 1), u""))
 
@@ -438,11 +478,11 @@ class TestFetchWorks(FetchTestCase):
             oldrepos.generate_revision_id(1, "", mapping)))
         self.assertTrue(newrepos.has_revision(
             oldrepos.generate_revision_id(2, "", mapping)))
-        inv1 = newrepos.get_inventory(
+        tree1 = newrepos.revision_tree(
                 oldrepos.generate_revision_id(1, "", mapping))
-        inv2 = newrepos.get_inventory(
+        tree2 = newrepos.revision_tree(
                 oldrepos.generate_revision_id(2, "", mapping))
-        self.assertNotEqual(inv1.path2id("bla"), inv2.path2id("bla"))
+        self.assertNotEqual(tree1.path2id("bla"), tree2.path2id("bla"))
 
     def test_fetch_copy_subdir(self):
         repos_url = self.make_repository('d')
@@ -558,9 +598,9 @@ Node-action: delete
         mapping = oldrepos.get_mapping()
         self.assertTrue(newrepos.has_revision(
             oldrepos.generate_revision_id(1, "", mapping)))
-        inv1 = newrepos.get_inventory(
+        tree1 = newrepos.revision_tree(
                 oldrepos.generate_revision_id(1, "", mapping))
-        self.assertTrue(inv1.has_filename(u"x\xe1"))
+        self.assertTrue(tree1.has_filename(u"x\xe1"))
 
     def test_fetch_replace_with_subreplace(self):
         filename = os.path.join(self.test_dir, "dumpfile")
@@ -702,9 +742,9 @@ Node-copyfrom-path: u
             oldrepos.generate_revision_id(1, "", mapping)))
         self.assertTrue(newrepos.has_revision(
             oldrepos.generate_revision_id(3, "", mapping)))
-        inv1 = newrepos.get_inventory(
+        tree1 = newrepos.revision_tree(
                 oldrepos.generate_revision_id(1, "", mapping))
-        inv2 = newrepos.get_inventory(
+        tree2 = newrepos.revision_tree(
                 oldrepos.generate_revision_id(3, "", mapping))
 
     def test_fetch_replace_self(self):
@@ -849,11 +889,11 @@ Node-copyfrom-path: bla
             oldrepos.generate_revision_id(1, "", mapping)))
         self.assertTrue(newrepos.has_revision(
             oldrepos.generate_revision_id(3, "", mapping)))
-        inv1 = newrepos.get_inventory(
+        tree1 = newrepos.revision_tree(
                 oldrepos.generate_revision_id(1, "", mapping))
-        inv2 = newrepos.get_inventory(
+        tree2 = newrepos.revision_tree(
                 oldrepos.generate_revision_id(3, "", mapping))
-        self.assertEqual(inv1.path2id("bla"), inv2.path2id("bla"))
+        self.assertEqual(tree1.path2id("bla"), tree2.path2id("bla"))
 
     def test_fetch_replace_backup(self):
         filename = os.path.join(self.test_dir, "dumpfile")
@@ -1030,11 +1070,11 @@ Node-copyfrom-path: bla
             oldrepos.generate_revision_id(1, "", mapping)))
         self.assertTrue(newrepos.has_revision(
             oldrepos.generate_revision_id(3, "", mapping)))
-        inv1 = newrepos.get_inventory(
+        tree1 = newrepos.revision_tree(
                 oldrepos.generate_revision_id(1, "", mapping))
-        inv2 = newrepos.get_inventory(
+        tree2 = newrepos.revision_tree(
                 oldrepos.generate_revision_id(3, "", mapping))
-        self.assertEqual(inv1.path2id("bla"), inv2.path2id("bla"))
+        self.assertEqual(tree1.path2id("bla"), tree2.path2id("bla"))
 
     def test_fetch_replace_unrelated(self):
         filename = os.path.join(self.test_dir, "dumpfile")
@@ -1172,11 +1212,11 @@ Node-copyfrom-path: x
             oldrepos.generate_revision_id(1, "", mapping)))
         self.assertTrue(newrepos.has_revision(
             oldrepos.generate_revision_id(4, "", mapping)))
-        inv1 = newrepos.get_inventory(
+        tree1 = newrepos.revision_tree(
                 oldrepos.generate_revision_id(1, "", mapping))
-        inv2 = newrepos.get_inventory(
+        tree2 = newrepos.revision_tree(
                 oldrepos.generate_revision_id(4, "", mapping))
-        self.assertNotEqual(inv1.path2id("x"), inv2.path2id("x"))
+        self.assertNotEqual(tree1.path2id("x"), tree2.path2id("x"))
 
     def test_fetch_replace_related(self):
         filename = os.path.join(self.test_dir, "dumpfile")
@@ -1336,11 +1376,11 @@ Node-copyfrom-path: x
             oldrepos.generate_revision_id(1, "", mapping)))
         self.assertTrue(newrepos.has_revision(
             oldrepos.generate_revision_id(5, "", mapping)))
-        inv1 = newrepos.get_inventory(
+        tree1 = newrepos.revision_tree(
                 oldrepos.generate_revision_id(1, "", mapping))
-        inv2 = newrepos.get_inventory(
+        tree2 = newrepos.revision_tree(
                 oldrepos.generate_revision_id(5, "", mapping))
-        self.assertNotEqual(inv1.path2id("y"), inv2.path2id("y"))
+        self.assertNotEqual(tree1.path2id("y"), tree2.path2id("y"))
 
     def test_fetch_dir_replace_self(self):
         repos_url = self.make_repository('d')
@@ -1555,7 +1595,7 @@ Node-copyfrom-path: x
         dc.close()
 
         repos = remote.SvnRemoteAccess(SvnRaTransport(repos_url),
-                format.SvnRemoteFormat()).find_repository()
+                remote.SvnRemoteFormat()).find_repository()
         repos.set_layout(TrunkLayout(0))
 
         mapping = repos.get_mapping()
@@ -1565,7 +1605,7 @@ Node-copyfrom-path: x
 
         self.assertEqual(
             mapping.generate_file_id((repos.uuid, "trunk", 1), u""),
-            tree.inventory.root.file_id)
+            tree.get_root_id())
 
     def test_fetch_odd(self):
         repos_url = self.make_repository('d')
@@ -1599,7 +1639,7 @@ Node-copyfrom-path: x
         foobranch.open_file("branches/foobranch/hosts").modify('foohosts')
         dc.close()
 
-        repos = remote.SvnRemoteAccess(SvnRaTransport(repos_url), format.SvnRemoteFormat()).find_repository()
+        repos = remote.SvnRemoteAccess(SvnRaTransport(repos_url), remote.SvnRemoteFormat()).find_repository()
         repos.set_layout(TrunkLayout(0))
 
         mapping = repos.get_mapping()
@@ -1625,11 +1665,11 @@ Node-copyfrom-path: x
         self.copy_content(oldrepos, newrepos1)
         self.copy_content(oldrepos, newrepos2)
         mapping = oldrepos.get_mapping()
-        inv1 = newrepos1.get_inventory(
+        tree1 = newrepos1.revision_tree(
                 oldrepos.generate_revision_id(1, "", mapping))
-        inv2 = newrepos2.get_inventory(
+        tree2 = newrepos2.revision_tree(
                 oldrepos.generate_revision_id(1, "", mapping))
-        self.assertEqual(inv1, inv2)
+        self.assertFalse(tree1.changes_from(tree2).has_changed())
 
     def test_fetch_executable(self):
         repos_url = self.make_repository('d')
@@ -1651,10 +1691,10 @@ Node-copyfrom-path: x
         mapping = oldrepos.get_mapping()
         self.assertTrue(newrepos.has_revision(
             oldrepos.generate_revision_id(1, "", mapping)))
-        inv1 = newrepos.get_inventory(
+        tree1 = newrepos.revision_tree(
                 oldrepos.generate_revision_id(1, "", mapping))
-        self.assertTrue(inv1[inv1.path2id("bla")].executable)
-        self.assertTrue(inv1[inv1.path2id("blie")].executable)
+        self.assertTrue(tree1.is_executable(tree1.path2id("bla")))
+        self.assertTrue(tree1.is_executable(tree1.path2id("blie")))
 
     def test_fetch_executable_persists(self):
         repos_url = self.make_repository('d')
@@ -1675,16 +1715,15 @@ Node-copyfrom-path: x
         newrepos = dir.create_repository()
         self.copy_content(oldrepos, newrepos)
         mapping = oldrepos.get_mapping()
-        inv1 = newrepos.get_inventory(
+        tree1 = newrepos.revision_tree(
                 oldrepos.generate_revision_id(1, "", mapping))
-        self.assertTrue(inv1[inv1.path2id("bla")].executable)
-        inv2 = newrepos.get_inventory(
+        self.assertTrue(tree1.is_executable(tree1.path2id("bla")))
+        tree2 = newrepos.revision_tree(
                 oldrepos.generate_revision_id(2, "", mapping))
-        self.assertTrue(inv2[inv2.path2id("bla")].executable)
+        self.assertTrue(tree2.is_executable(tree2.path2id("bla")))
 
     def test_fetch_symlink(self):
-        if not has_symlinks():
-            return
+        self.requireFeature(SymlinkFeature)
         repos_url = self.make_repository('d')
 
         dc = self.get_commit_editor(repos_url)
@@ -1702,13 +1741,13 @@ Node-copyfrom-path: x
         mapping = oldrepos.get_mapping()
         self.assertTrue(newrepos.has_revision(
             oldrepos.generate_revision_id(1, "", mapping)))
-        inv1 = newrepos.get_inventory(
+        tree1 = newrepos.revision_tree(
                 oldrepos.generate_revision_id(1, "", mapping))
-        ie = inv1[inv1.path2id("mylink")]
-        self.assertEqual('symlink', ie.kind)
-        self.assertEqual('bla', ie.symlink_target)
-        self.assertEquals(None, ie.text_sha1)
-        key = (ie.file_id, ie.revision)
+        file_id = tree1.path2id("mylink")
+        self.assertEqual('symlink', tree1.kind(file_id))
+        self.assertEqual('bla', tree1.get_symlink_target(file_id))
+        self.assertEquals(None, tree1.get_file_sha1(file_id))
+        key = (file_id, tree1.get_file_revision(file_id))
         newrepos.lock_read()
         try:
             self.assertEquals(sha_string(""),
@@ -1717,9 +1756,7 @@ Node-copyfrom-path: x
             newrepos.unlock()
 
     def test_fetch_symlink_with_newlines(self):
-        if not has_symlinks():
-            return
-        raise KnownFailure("Bazaar doesn't support newlines in symlink targets (#219832)")
+        self.requireFeature(SymlinkFeature)
         repos_url = self.make_repository('d')
 
         dc = self.get_commit_editor(repos_url)
@@ -1732,14 +1769,17 @@ Node-copyfrom-path: x
         oldrepos = Repository.open(repos_url)
         dir = BzrDir.create("f")
         newrepos = dir.create_repository()
-        self.copy_content(oldrepos, newrepos)
+        self.expectFailure(
+            "Bazaar doesn't support newlines in symlink targets (#219832)",
+            self.copy_content, oldrepos, newrepos)
         mapping = oldrepos.get_mapping()
         self.assertTrue(newrepos.has_revision(
             oldrepos.generate_revision_id(1, "", mapping)))
-        inv1 = newrepos.get_inventory(
+        tree1 = newrepos.revision_tree(
                 oldrepos.generate_revision_id(1, "", mapping))
-        self.assertEqual('symlink', inv1[inv1.path2id("mylink")].kind)
-        self.assertEqual('bla\nbar\nbla', inv1[inv1.path2id("mylink")].symlink_target)
+        self.assertEqual('symlink', tree1.kind(tree1.path2id("mylink")))
+        self.assertEqual('bla\nbar\nbla',
+            tree1.get_symlink_target(tree1.path2id("mylink")))
 
     def test_fetch_special_non_symlink(self):
         repos_url = self.make_repository('d')
@@ -1758,9 +1798,9 @@ Node-copyfrom-path: x
         mapping = oldrepos.get_mapping()
         self.assertTrue(newrepos.has_revision(
             oldrepos.generate_revision_id(1, "", mapping)))
-        inv1 = newrepos.get_inventory(
+        tree1 = newrepos.revision_tree(
                 oldrepos.generate_revision_id(1, "", mapping))
-        self.assertEqual('file', inv1[inv1.path2id("mylink")].kind)
+        self.assertEqual('file', tree1.kind(tree1.path2id("mylink")))
 
     def test_fetch_special_unbecomes_symlink(self):
         repos_url = self.make_repository('d')
@@ -1784,9 +1824,23 @@ Node-copyfrom-path: x
         mapping = oldrepos.get_mapping()
         self.assertTrue(newrepos.has_revision(
             oldrepos.generate_revision_id(2, "", mapping)))
-        inv1 = newrepos.get_inventory(
+        tree1 = newrepos.revision_tree(
                 oldrepos.generate_revision_id(2, "", mapping))
-        self.assertEqual('file', inv1[inv1.path2id("mylink")].kind)
+        self.assertEqual('file', tree1.kind(tree1.path2id("mylink")))
+
+        # Now test the same thing again as two separate copy_content calls, to
+        # check it also works if the symlink's "text" is not in the fetch
+        # text_cache.
+        dir2 = BzrDir.create("f2")
+        newrepos2 = dir2.create_repository()
+        self.copy_content(oldrepos, newrepos2,
+              oldrepos.generate_revision_id(1, "", mapping))
+        self.copy_content(oldrepos, newrepos2)
+        self.assertTrue(newrepos2.has_revision(
+            oldrepos.generate_revision_id(2, "", mapping)))
+        tree2 = newrepos2.revision_tree(
+                oldrepos.generate_revision_id(2, "", mapping))
+        self.assertEqual('file', tree2.kind(tree2.path2id("mylink")))
 
     def test_fetch_special_becomes_symlink(self):
         repos_url = self.make_repository('d')
@@ -1810,10 +1864,13 @@ Node-copyfrom-path: x
         mapping = oldrepos.get_mapping()
         self.assertTrue(newrepos.has_revision(
             oldrepos.generate_revision_id(2, "", mapping)))
-        inv1 = newrepos.get_inventory(
+        tree1 = newrepos.revision_tree(
                 oldrepos.generate_revision_id(2, "", mapping))
-        raise KnownFailure("not allowing svn:special invalid files to be restored to symlinks yet")
-        self.assertEqual('symlink', inv1[inv1.path2id("mylink")].kind)
+        self.expectFailure(
+            "not allowing svn:special invalid files to be restored to symlinks "
+            "yet",
+            self.assertEqual,
+            'symlink', tree1.kind(tree1.path2id("mylink")))
 
     def test_fetch_symlink_kind_change(self):
         repos_url = self.make_repository('d')
@@ -1834,13 +1891,13 @@ Node-copyfrom-path: x
         mapping = oldrepos.get_mapping()
         self.assertTrue(newrepos.has_revision(
             oldrepos.generate_revision_id(1, "", mapping)))
-        inv1 = newrepos.get_inventory(
+        tree1 = newrepos.revision_tree(
                 oldrepos.generate_revision_id(1, "", mapping))
-        inv2 = newrepos.get_inventory(
+        tree2 = newrepos.revision_tree(
                 oldrepos.generate_revision_id(2, "", mapping))
-        self.assertEqual('file', inv1[inv1.path2id("mylink")].kind)
-        self.assertEqual('symlink', inv2[inv2.path2id("mylink")].kind)
-        self.assertEqual('bla', inv2[inv2.path2id("mylink")].symlink_target)
+        self.assertEqual('file', tree1.kind(tree1.path2id("mylink")))
+        self.assertEqual('symlink', tree2.kind(tree2.path2id("mylink")))
+        self.assertEqual('bla', tree2.get_symlink_target(tree2.path2id("mylink")))
 
     def test_fetch_executable_separate(self):
         repos_url = self.make_repository('d')
@@ -1861,14 +1918,14 @@ Node-copyfrom-path: x
         mapping = oldrepos.get_mapping()
         self.assertTrue(newrepos.has_revision(
             oldrepos.generate_revision_id(1, "", mapping)))
-        inv1 = newrepos.get_inventory(
+        tree1 = newrepos.revision_tree(
                 oldrepos.generate_revision_id(1, "", mapping))
-        self.assertFalse(inv1[inv1.path2id("bla")].executable)
-        inv2 = newrepos.get_inventory(
+        self.assertFalse(tree1.is_executable(tree1.path2id("bla")))
+        tree2 = newrepos.revision_tree(
                 oldrepos.generate_revision_id(2, "", mapping))
-        self.assertTrue(inv2[inv2.path2id("bla")].executable)
+        self.assertTrue(tree2.is_executable(tree2.path2id("bla")))
         self.assertEqual(oldrepos.generate_revision_id(2, "", mapping),
-                         inv2[inv2.path2id("bla")].revision)
+                         tree2.get_file_revision(tree2.path2id("bla")))
 
     def test_fetch_hidden(self):
         repos_url = self.make_repository('d')
@@ -2095,7 +2152,7 @@ Node-copyfrom-path: x
         self.assertChangedPathsEquals({
             'trunk/adir': (u'R', 'trunk/adir', 2, NODE_DIR),
             'trunk/adir/foofile': (u'A', None, -1, NODE_FILE)},
-              oldrepos._revmeta_provider.get_revision("trunk", 3).get_paths())
+              oldrepos._revmeta_provider.get_revision("trunk", 3).metarev.paths)
         oldrepos.set_layout(TrunkLayout(0))
         dir = BzrDir.create("f")
         newrepos = dir.create_repository()
@@ -2132,7 +2189,7 @@ Node-copyfrom-path: x
         self.assertChangedPathsEquals({
             'trunk/adir': (u'R', 'trunk/adir', 1, NODE_DIR),
             'trunk/adir/foofile': (u'A', None, -1, NODE_FILE)},
-              oldrepos._revmeta_provider.get_revision("trunk", 3).get_paths())
+              oldrepos._revmeta_provider.get_revision("trunk", 3).metarev.paths)
         oldrepos.set_layout(TrunkLayout(0))
         dir = BzrDir.create("f")
         newrepos = dir.create_repository()
@@ -2179,12 +2236,62 @@ Node-copyfrom-path: x
         lastrev = oldrepos.generate_revision_id(4, "branches/abranch", mapping)
         self.copy_content(oldrepos, newrepos, lastrev)
 
-        inventory = newrepos.get_inventory(lastrev)
+        tree = newrepos.revision_tree(lastrev)
         self.assertEqual(prevrev,
-                         inventory[inventory.path2id("bdir/afile")].revision)
+                         tree.get_file_revision(tree.path2id("bdir/afile")))
 
-        inventory = newrepos.get_inventory(prevrev)
+        tree = newrepos.revision_tree(prevrev)
         self.assertEqual(copyrev,
-                         inventory[inventory.path2id("bdir/stationary")].revision)
+                         tree.get_file_revision(tree.path2id("bdir/stationary")))
+
+    def test_fetch_different_parent_path(self):
+        repos_url = self.make_repository('d')
+
+        dc = self.get_commit_editor(repos_url)
+        myproj = dc.add_dir("myproj")
+        trunk = myproj.add_dir("myproj/trunk")
+        trunk.add_file("myproj/trunk/afile").modify()
+        dc.close()
+
+        dc = self.get_commit_editor(repos_url)
+        dc.add_dir("oldlayout")
+        dc.close()
+
+        dc = self.get_commit_editor(repos_url)
+        oldlayout = dc.open_dir("oldlayout")
+        oldlayout.add_dir("oldlayout/myproj", "myproj")
+        dc.delete("myproj")
+        dc.close()
+
+        dc = self.get_commit_editor(repos_url)
+        trunk = dc.add_dir("trunk")
+        trunk.add_dir("trunk/myproj", "oldlayout/myproj/trunk")
+        oldlayout = dc.open_dir("oldlayout")
+        myproj = oldlayout.open_dir("oldlayout/myproj")
+        myproj.delete("oldlayout/myproj/trunk")
+        dc.close()
+
+        oldbranch = BzrDir.open(repos_url+"/trunk/myproj")
+        oldbranch.sprout("dc")
 
 
+class ChunksStartWithLinkTests(TestCase):
+
+    def test_simple(self):
+        vf = ["not", "a link"]
+        self.assertFalse(chunks_start_with_link(vf))
+        vf = ["link a link"]
+        self.assertTrue(chunks_start_with_link(vf))
+        vf = ["link", " a", " link"]
+        self.assertTrue(chunks_start_with_link(vf))
+        vf = ["li", "nk", " a", " link"]
+        self.assertTrue(chunks_start_with_link(vf))
+
+
+class TestCheckFilename(TestCase):
+
+    def test_valid(self):
+        check_filename(u"foo/bar")
+
+    def test_backspace(self):
+        self.assertRaises(InvalidFileName, check_filename, u"foo\\bar")
