@@ -59,6 +59,24 @@ class TestLogWalker(SubversionTestCase):
 
         self.assertEqual(2, len(list(walker.iter_changes(None, 1))))
 
+    def test_get_branch_log_copy_from_root(self):
+        repos_url = self.make_repository("a")
+
+        cb = self.get_commit_editor(repos_url)
+        cb.add_file("bla").modify()
+        cb.close()
+
+        cb = self.get_commit_editor(repos_url)
+        cb.add_dir("trunk", "")
+        cb.close()
+
+        walker = self.get_log_walker(transport=SvnRaTransport(repos_url))
+
+        self.assertLogEquals( [({'trunk': ('A', '', 1, NODE_DIR)}, 2),
+             ({'bla': ('A', None, -1, NODE_FILE)}, 1),
+             ({'': ('A', None, -1, NODE_DIR)}, 0)],
+            (l[:2] for l in walker.iter_changes(["trunk"], 2, 0)))
+
     def test_get_branch_follow_branch(self):
         repos_url = self.make_repository("a")
         cb = self.get_commit_editor(repos_url)
@@ -569,9 +587,51 @@ class TestCachingLogWalker(TestLogWalker):
 
         logwalker.cache_dir = os.path.join(self.test_dir, "cache-dir")
 
+    def test_fetch_revisions(self):
+        repos_url = self.make_repository('d')
+
+        cb = self.get_commit_editor(repos_url)
+        cb.close()
+
+        cb = self.get_commit_editor(repos_url)
+        cb.close()
+
+        walker = self.get_log_walker(transport=SvnRaTransport(repos_url))
+        self.assertEquals(0, walker.saved_maxrevnum)
+        self.assertEquals(None, walker.saved_minrevnum)
+
+        updater = logwalker.CachingLogWalkerUpdater(walker, True)
+        updater({"trunk": ("A", None, -1)}, 2, {"svn:author": "author"})
+
+        self.assertEquals(2, walker.saved_maxrevnum)
+        self.assertEquals(2, walker.saved_minrevnum)
+
+        walker._fetch_revisions(2)
+
+        self.assertEquals(2, walker.saved_maxrevnum)
+        self.assertEquals(0, walker.saved_minrevnum)
+
+    def test_fetch_revisions_fill_in(self):
+        repos_url = self.make_repository('d')
+
+        cb = self.get_commit_editor(repos_url)
+        cb.close()
+
+        cb = self.get_commit_editor(repos_url)
+        cb.close()
+
+        walker = self.get_log_walker(transport=SvnRaTransport(repos_url))
+        self.assertEquals(0, walker.saved_maxrevnum)
+        self.assertEquals(None, walker.saved_minrevnum)
+
+        walker._fetch_revisions(2)
+
+        self.assertEquals(2, walker.saved_maxrevnum)
+        self.assertEquals(0, walker.saved_minrevnum)
+
     def get_log_walker(self, transport):
-        from bzrlib.plugins.svn.cache.sqlitecache import LogCache
-        return logwalker.CachingLogWalker(super(TestCachingLogWalker, self).get_log_walker(transport), LogCache())
+        from bzrlib.plugins.svn.cache.sqlitecache import SqliteLogCache
+        return logwalker.CachingLogWalker(super(TestCachingLogWalker, self).get_log_walker(transport), SqliteLogCache())
 
 
 
