@@ -16,33 +16,55 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+"""Tests for push from a Bazaar to a Subversion repository."""
+
 import os
-from subvertpy import NODE_DIR, NODE_FILE, ra
+from subvertpy import (
+    NODE_DIR,
+    NODE_FILE,
+    ra,
+    )
 
 from bzrlib import osutils
 from bzrlib.branch import Branch
 from bzrlib.bzrdir import BzrDir
 from bzrlib.errors import (
     AlreadyBranchError,
+    AppendRevisionsOnlyViolation,
     BzrError,
     DivergedBranches,
-    AppendRevisionsOnlyViolation,
     )
 from bzrlib.inventory import InventoryDirectory
 from bzrlib.merge import (
     Merger,
     Merge3Merger,
     )
-from bzrlib.repository import Repository
-from bzrlib.revision import Revision
+from bzrlib.repository import (
+    InterRepository,
+    Repository,
+    )
+from bzrlib.revision import (
+    NULL_REVISION,
+    Revision,
+    )
 from bzrlib.trace import mutter
-from bzrlib.tests import TestCase
+from bzrlib.tests import (
+    TestCase,
+    )
+try:
+    from bzrlib.tests.features import (
+        SymlinkFeature,
+        )
+except ImportError: # bzr < 2.5
+    from bzrlib.tests import (
+        SymlinkFeature,
+        )
 
 from bzrlib.plugins.svn import (
     transport,
     )
 from bzrlib.plugins.svn.branch import (
-    InterOtherSvnBranch,
+    InterToSvnBranch,
     )
 from bzrlib.plugins.svn.errors import (
     MissingPrefix,
@@ -50,8 +72,10 @@ from bzrlib.plugins.svn.errors import (
 from bzrlib.plugins.svn.layout.standard import (
     RootLayout,
     TrunkLayout,
+    TrunkLayout0,
     )
 from bzrlib.plugins.svn.push import (
+    create_branch_container,
     determine_branch_path,
     )
 from bzrlib.plugins.svn.tests import (
@@ -59,7 +83,6 @@ from bzrlib.plugins.svn.tests import (
     )
 
 
-
 class TestDPush(SubversionTestCase):
 
     def setUp(self):
@@ -84,7 +107,7 @@ class TestDPush(SubversionTestCase):
         return self.get_commit_editor(self.repos_url, message)
 
     def dpush(self, source, target):
-        return InterOtherSvnBranch(source, target)._update_revisions_lossy()
+        return InterToSvnBranch(source, target)._update_revisions_lossy()
 
     def test_change_single(self):
         self.build_tree({'dc/foo/bla': 'other data'})
@@ -160,7 +183,7 @@ class TestDPush(SubversionTestCase):
         repos = self.svndir.find_repository()
         revmeta = repos._revmeta_provider.get_revision("", 3)
         self.assertChangedPathsEquals(
-                {"foo/bliel": ('A', None, -1, NODE_FILE)}, revmeta.get_paths())
+                {'': ('M', None, -1, 2), "foo/bliel": ('A', None, -1, NODE_FILE)}, revmeta.metarev.paths)
 
     def test_diverged(self):
         dc = self.commit_editor()
@@ -250,15 +273,13 @@ class TestPush(SubversionTestCase):
         repos = self.svndir.find_repository()
         mapping = repos.get_mapping()
         self.assertEquals(newid, svnbranch.last_revision())
-        inv = repos.get_inventory(repos.generate_revision_id(2, "", mapping))
-        self.assertEqual(newid, inv[inv.path2id('foo/bla')].revision)
+        tree = repos.revision_tree(repos.generate_revision_id(2, "", mapping))
+        self.assertEqual(newid, tree.get_file_revision(tree.path2id('foo/bla')))
         self.assertEqual(wt.branch.last_revision(),
           repos.generate_revision_id(2, "", mapping))
         self.assertEqual(repos.generate_revision_id(2, "", mapping),
                         self.svndir.open_branch().last_revision())
-        self.assertEqual("other data",
-            repos.revision_tree(repos.generate_revision_id(2, "",
-                                mapping)).get_file_text(inv.path2id("foo/bla")))
+        self.assertEqual("other data", tree.get_file_text(tree.path2id("foo/bla")))
 
     def test_simple(self):
         self.build_tree({'dc/file': 'data'})
@@ -270,8 +291,8 @@ class TestPush(SubversionTestCase):
 
         repos = self.svndir.find_repository()
         mapping = repos.get_mapping()
-        inv = repos.get_inventory(repos.generate_revision_id(2, "", mapping))
-        self.assertTrue(inv.has_filename('file'))
+        tree = repos.revision_tree(repos.generate_revision_id(2, "", mapping))
+        self.assertTrue(tree.has_filename('file'))
         self.assertEquals(wt.branch.last_revision(),
                 repos.generate_revision_id(2, "", mapping))
         self.assertEqual(repos.generate_revision_id(2, "", mapping),
@@ -299,16 +320,15 @@ class TestPush(SubversionTestCase):
 
         repos = self.svndir.find_repository()
         mapping = repos.get_mapping()
-        inv = repos.get_inventory(repos.generate_revision_id(2, "", mapping))
-        self.assertTrue(inv.has_filename('file'))
+        tree = repos.revision_tree(repos.generate_revision_id(2, "", mapping))
+        self.assertTrue(tree.has_filename('file'))
         self.assertEquals(wt.branch.last_revision(),
                 repos.generate_revision_id(2, "", mapping))
         self.assertEqual(repos.generate_revision_id(2, "", mapping),
                         self.svndir.open_branch().last_revision())
 
     def test_symlink(self):
-        if not osutils.has_symlinks():
-            return
+        self.requireFeature(SymlinkFeature)
         os.symlink("bla", "dc/south")
         self.assertTrue(os.path.islink("dc/south"))
         wt = self.bzrdir.open_workingtree()
@@ -320,10 +340,10 @@ class TestPush(SubversionTestCase):
 
         repos = self.svndir.find_repository()
         mapping = repos.get_mapping()
-        inv = repos.get_inventory(repos.generate_revision_id(2, "", mapping))
-        self.assertTrue(inv.has_filename('south'))
-        self.assertEquals('symlink', inv[inv.path2id('south')].kind)
-        self.assertEquals('bla', inv[inv.path2id('south')].symlink_target)
+        tree = repos.revision_tree(repos.generate_revision_id(2, "", mapping))
+        self.assertTrue(tree.has_filename('south'))
+        self.assertEquals('symlink', tree.kind(tree.path2id('south')))
+        self.assertEquals('bla', tree.get_symlink_target(tree.path2id('south')))
 
     def test_pull_after_push(self):
         self.build_tree({'dc/file': 'data'})
@@ -335,8 +355,8 @@ class TestPush(SubversionTestCase):
 
         repos = self.svndir.find_repository()
         mapping = repos.get_mapping()
-        inv = repos.get_inventory(repos.generate_revision_id(2, "", mapping))
-        self.assertTrue(inv.has_filename('file'))
+        tree = repos.revision_tree(repos.generate_revision_id(2, "", mapping))
+        self.assertTrue(tree.has_filename('file'))
         self.assertEquals(wt.branch.last_revision(),
                          repos.generate_revision_id(2, "", mapping))
         self.assertEqual(repos.generate_revision_id(2, "", mapping),
@@ -376,13 +396,13 @@ class TestPush(SubversionTestCase):
         b = Branch.open("b")
 
         def check_tree_revids(rtree):
-            self.assertEqual(rtree.inventory[rtree.path2id("file")].revision,
+            self.assertEqual(rtree.get_file_revision(rtree.path2id("file")),
                              revid)
-            self.assertEqual(rtree.inventory[rtree.path2id("foo")].revision,
+            self.assertEqual(rtree.get_file_revision(rtree.path2id("foo")),
                              b.revision_history()[1])
-            self.assertEqual(rtree.inventory[rtree.path2id("foo/bla")].revision,
+            self.assertEqual(rtree.get_file_revision(rtree.path2id("foo/bla")),
                              revid)
-            self.assertEqual(rtree.inventory.root.revision, b.revision_history()[0])
+            self.assertEqual(rtree.get_revision_id(), b.last_revision())
 
         check_tree_revids(wt.branch.repository.revision_tree(b.last_revision()))
 
@@ -451,22 +471,19 @@ class TestPush(SubversionTestCase):
         self.assertEqual(repos.generate_revision_id(3, "", mapping),
                         self.svndir.open_branch().last_revision())
 
-        inv = repos.get_inventory(repos.generate_revision_id(2, "", mapping))
-        self.assertTrue(inv.has_filename('file'))
-        self.assertFalse(inv.has_filename('adir'))
+        tree = repos.revision_tree(repos.generate_revision_id(2, "", mapping))
+        self.assertTrue(tree.has_filename('file'))
+        self.assertFalse(tree.has_filename('adir'))
 
-        inv = repos.get_inventory(repos.generate_revision_id(3, "", mapping))
-        self.assertTrue(inv.has_filename('file'))
-        self.assertTrue(inv.has_filename('adir'))
+        tree = repos.revision_tree(repos.generate_revision_id(3, "", mapping))
+        self.assertTrue(tree.has_filename('file'))
+        self.assertTrue(tree.has_filename('adir'))
 
         self.assertEqual(self.svndir.open_branch().revision_history(),
                          self.bzrdir.open_branch().revision_history())
 
         self.assertEqual(wt.branch.last_revision(),
                 repos.generate_revision_id(3, "", mapping))
-        self.assertEqual(
-                wt.branch.repository.get_ancestry(wt.branch.last_revision()),
-                repos.get_ancestry(wt.branch.last_revision()))
 
     def test_multiple_diverged(self):
         oc_url = self.make_repository("o")
@@ -516,7 +533,9 @@ class TestPush(SubversionTestCase):
         b = Branch.open("%s/trunk" % self.repos_url)
         wt.branch.push(b)
         mutter('log %r' % self.client_log("%s/trunk" % self.repos_url, 0, 5)[5][0])
-        self.assertEquals({'/trunk/foo': ('M', None, -1)},
+        self.assertEquals({
+            '/trunk': ('M', None, -1),
+            '/trunk/foo': ('M', None, -1)},
             self.client_log("%s/trunk" % self.repos_url, 0, 5)[5][0])
 
     def test_comics(self):
@@ -540,8 +559,9 @@ class TestPush(SubversionTestCase):
 
         self.svndir.open_branch().pull(wt.branch)
 
-        paths = self.svndir.find_repository()._revmeta_provider.get_revision("", 3).get_paths()
+        paths = self.svndir.find_repository()._revmeta_provider.get_revision("", 3).metarev.paths
         self.assertChangedPathsEquals({
+            'trunk': ('M', None, -1, NODE_DIR),
             'trunk/comics': (u'R', None, -1, NODE_DIR),
             'trunk/comics/bin': (u'A', 'trunk/comics/bin', 2, NODE_DIR),
             'trunk/comics/core': (u'A', 'trunk/comics', 2, NODE_DIR),
@@ -572,8 +592,8 @@ class PushNewBranchTests(SubversionTestCase):
 
         bzrwt.lock_read()
         try:
-            self.assertEquals(bzrwt.inventory.root.file_id,
-                              newtree.inventory.root.file_id)
+            self.assertEquals(bzrwt.get_root_id(),
+                              newtree.get_root_id())
         finally:
             bzrwt.unlock()
 
@@ -585,8 +605,7 @@ class PushNewBranchTests(SubversionTestCase):
         newtree = newbranch.repository.revision_tree(revid)
         bzrwt.lock_read()
         try:
-            self.assertEquals(bzrwt.inventory.root.file_id,
-                              newtree.inventory.root.file_id)
+            self.assertEquals(bzrwt.get_root_id(), newtree.get_root_id())
         finally:
             bzrwt.unlock()
         self.assertEquals(revid, newbranch.last_revision())
@@ -710,13 +729,16 @@ class PushNewBranchTests(SubversionTestCase):
 
     def test_push_merge_unchanged_file(self):
         def check_tree(t):
-            self.assertEquals(base_revid, t.inventory[t.path2id("bar.txt")].revision)
-            self.assertEquals(other_revid, t.inventory[t.path2id("bar2.txt")].revision)
+            self.assertEquals(base_revid,
+                t.get_file_revision(t.path2id("bar.txt")))
+            self.assertEquals(other_revid,
+                t.get_file_revision(t.path2id("bar2.txt")))
         repos_url = self.make_repository("test")
 
         dc = self.get_commit_editor(repos_url)
         trunk = dc.add_dir("trunk")
         trunk.add_file("trunk/foo.txt").modify("add file")
+        dc.add_dir("branches")
         dc.close()
 
         os.mkdir('bzrco1')
@@ -735,6 +757,9 @@ class PushNewBranchTests(SubversionTestCase):
         self.build_tree({'bzrco2/bar2.txt': 'bar'})
         wt2.add("bar2.txt")
         other_revid = wt2.commit("add yet another file", rev_id="side1")
+        self.assertEquals("side1", other_revid)
+        side1_dir = BzrDir.open(repos_url+"/branches/side1")
+        side1_dir.push_branch(wt2.branch)
 
         wt1.lock_write()
         try:
@@ -748,16 +773,15 @@ class PushNewBranchTests(SubversionTestCase):
         wt1.branch.push(Branch.open(repos_url+"/trunk"))
         r = Repository.open(repos_url)
         revmeta = r._revmeta_provider.get_revision("trunk", 3)
-        self.assertEquals({"bar2.txt": "side1"}, revmeta.get_text_revisions(r.get_mapping()))
 
         os.mkdir("cpy")
         cpy = BzrDir.create("cpy")
         cpyrepos = cpy.create_repository()
-        r.copy_content_into(cpyrepos)
-        check_tree(cpyrepos.revision_tree(mergingrevid))
 
         t = r.revision_tree(mergingrevid)
         check_tree(t)
+        r.copy_content_into(cpyrepos)
+        check_tree(cpyrepos.revision_tree(mergingrevid))
 
     def test_missing_prefix_error(self):
         repos_url = self.make_repository("a")
@@ -956,13 +980,12 @@ class PushNewBranchTests(SubversionTestCase):
         self.assertEquals(revid2, newbranch.last_revision())
         self.assertEquals([revid1, revid2], newbranch.revision_history())
         tree = newbranch.repository.revision_tree(revid2)
-        mutter("inventory: %r" % tree.inventory.entries())
         delta = tree.changes_from(bzrwt)
         self.assertFalse(delta.has_changed())
-        self.assertTrue(tree.inventory.has_filename("registry"))
-        self.assertTrue(tree.inventory.has_filename("registry.moved"))
-        self.assertTrue(tree.inventory.has_filename("registry/generic.c"))
-        self.assertFalse(tree.inventory.has_filename("registry.moved/generic.c"))
+        self.assertTrue(tree.has_filename("registry"))
+        self.assertTrue(tree.has_filename("registry.moved"))
+        self.assertTrue(tree.has_filename("registry/generic.c"))
+        self.assertFalse(tree.has_filename("registry.moved/generic.c"))
         os.mkdir("n")
         BzrDir.open(repos_url+"/trunk").sprout("n")
 
@@ -1021,7 +1044,7 @@ class PushNewBranchTests(SubversionTestCase):
         newbranch = newdir.import_branch(bzrwt.branch)
         self.assertChangedPathsEquals({"trunk": ("R", None, -1, NODE_DIR),
                            "trunk/foo": ("A", "trunk/foo", 1, NODE_FILE)},
-            newbranch.repository._revmeta_provider.get_revision("trunk", 2).get_paths())
+            newbranch.repository._revmeta_provider.get_revision("trunk", 2).metarev.paths)
         tree1 = newbranch.repository.revision_tree(revid1)
         tree2 = newbranch.repository.revision_tree(revid2)
         self.assertEquals(tree1.path2id(""), old_ie.file_id)
@@ -1082,9 +1105,6 @@ class PushNewBranchTests(SubversionTestCase):
         check(copybranch)
 
     def test_push_non_lhs_parent(self):
-        from bzrlib.debug import debug_flags
-        debug_flags.add("commit")
-        debug_flags.add("fetch")
         repos_url = self.make_repository("a")
         bzrwt = BzrDir.create_standalone_workingtree("c")
         self.build_tree({'c/registry/generic.c': "Tour"})
@@ -1099,7 +1119,6 @@ class PushNewBranchTests(SubversionTestCase):
         newbranch = newdir.import_branch(bzrwt.branch)
 
         # Should create dc/trunk
-
         dc = self.get_commit_editor(repos_url)
         branches = dc.add_dir("branches")
         branches.add_dir('branches/foo', 'trunk')
@@ -1161,7 +1180,7 @@ class PushNewBranchTests(SubversionTestCase):
         trunk = dc.add_dir("trunk")
         subdir = trunk.add_dir("trunk/mysubdir")
         subdir.add_file("trunk/mysubdir/myfile").modify("blabla")
-        dc.close()
+        dc.close() #1
 
         os.mkdir("dc")
         svndir = BzrDir.open(repos_url+"/trunk")
@@ -1170,18 +1189,19 @@ class PushNewBranchTests(SubversionTestCase):
         wt.lock_write()
         try:
             wt.apply_inventory_delta([
-                ("", None, wt.inventory.root.file_id, None),
-                ("mysubdir", "", wt.inventory.path2id("mysubdir"),
-                    InventoryDirectory(wt.inventory.path2id("mysubdir"), "",
+                ("", None, wt.get_root_id(), None),
+                ("mysubdir", "", wt.path2id("mysubdir"),
+                    InventoryDirectory(wt.path2id("mysubdir"), "",
                         None))])
             os.rename("dc/mysubdir/myfile", "dc/myfile")
             osutils.rmtree("dc/mysubdir")
         finally:
             wt.unlock()
-        wt.commit("Change branch root")
+        wt.commit("Change branch root") #2
         svnbranch = svndir.open_branch()
         svnbranch.pull(wt.branch)
         self.assertEquals(svnbranch.last_revision(), wt.branch.last_revision())
+        self.assertEquals(2, svndir.root_transport.get_latest_revnum())
         self.assertEquals(["myfile"], svndir.root_transport.list_dir("."))
         paths = svnbranch.repository._revmeta_provider._log.get_revision_paths(2)
         self.assertChangedPathEquals(('R', "trunk/mysubdir", 1, NODE_DIR),
@@ -1211,6 +1231,7 @@ class PushNewBranchTests(SubversionTestCase):
 
 
 class TestPushTwice(SubversionTestCase):
+
     def test_push_twice(self):
         # bug 208566
         repos_url = self.make_repository('d')
@@ -1261,3 +1282,271 @@ class DetermineBranchPathTests(TestCase):
         rev.properties['branch-nick'] = 'bla'
         self.assertEquals("someproj/branches/bla",
             determine_branch_path(rev, TrunkLayout(1), "someproj"))
+
+    def test_nick_slashes(self):
+        rev = Revision("fooid")
+        rev.properties['branch-nick'] = 'bla/bloe'
+        self.assertEquals("someproj/branches/bla_bloe",
+            determine_branch_path(rev, TrunkLayout(1), "someproj"))
+
+
+class InterToSvnRepositoryTestCase(SubversionTestCase):
+
+    def setUp(self):
+        super(InterToSvnRepositoryTestCase, self).setUp()
+        self.from_bzrdir = BzrDir.create('bzrrepo')
+        self.from_repo = self.from_bzrdir.create_repository(shared=True)
+        self.svn_repo_url = self.make_repository('svnrepo')
+        self.to_repo = Repository.open(self.svn_repo_url)
+        self.interrepo = InterRepository.get(self.from_repo, self.to_repo)
+
+
+class ForeignRevisionInfoTests(InterToSvnRepositoryTestCase):
+
+    def test__target_has_revision(self):
+        self.assertTrue(self.interrepo._target_has_revision(NULL_REVISION))
+        self.assertFalse(self.interrepo._target_has_revision("foo"))
+        self.interrepo._add_path_info("trunk", "myrevid", ("myuuid", "trunk", 4))
+        self.assertTrue(self.interrepo._target_has_revision("myrevid"))
+
+    def test_foreign_revision_info_null(self):
+        self.assertEquals((None, None),
+            self.interrepo._get_foreign_revision_info(NULL_REVISION, "apath"))
+
+    def test_foreign_revision_info_no_path(self):
+        self.interrepo._add_path_info("trunk", "myrevid", ("myuuid", "trunk", 4))
+        self.assertEquals(("myuuid", "trunk", 4),
+            self.interrepo._get_foreign_revision_info("myrevid"))
+
+    def test_foreign_revision_info_available(self):
+        # Prefer path if it is available
+        self.interrepo._add_path_info("trunk", "myrevid", ("myuuid", "trunk", 4))
+        self.interrepo._add_path_info("otherpath", "myrevid", ("myuuid", "trunk", 4))
+        self.assertEquals(("myuuid", "trunk", 4),
+            self.interrepo._get_foreign_revision_info("myrevid", "trunk"))
+
+    def test_foreign_revision_info_first(self):
+        # Prefer path if it is available
+        self.interrepo._add_path_info("anotherpath", "myrevid", ("myuuid", "trunk1", 4))
+        self.interrepo._add_path_info("otherpath", "myrevid", ("myuuid", "trunk2", 4))
+        foreign_info = self.interrepo._get_foreign_revision_info("myrevid", "trunk")
+        self.assertEquals("myuuid", foreign_info[0])
+        self.assertEquals(4, foreign_info[2])
+
+    def test_lookup(self):
+        revids_check = []
+        # If the foreign revision info is not cached it should be retrieved
+        def lookup(revid):
+            revids_check.append(revid)
+            return ("someuuid", "somebranch", 42)
+        self.to_repo.lookup_bzr_revision_id = lookup
+        self.assertEquals(("someuuid", "somebranch", 42),
+            self.interrepo._get_foreign_revision_info("auuid", "trunk"))
+        self.assertEquals(["auuid"], revids_check)
+
+
+class PushRevisionTests(InterToSvnRepositoryTestCase):
+
+    def setUp(self):
+        super(PushRevisionTests, self).setUp()
+        branch = BzrDir.create_branch_convenience('bzrrepo/tree1')
+        tree = branch.bzrdir.open_workingtree()
+        self.build_tree_contents([('bzrrepo/tree1/a', 'data')])
+        tree.add(['a'])
+        self.revid1 = tree.commit('msg')
+
+    def test_push_first_revision_with_metadata(self):
+        self.interrepo.push_single_revision("trunk",
+            self.interrepo._get_branch_config("trunk"),
+            self.from_repo.get_revision(self.revid1),
+            push_metadata=True, base_foreign_info=(None, None), root_action=("create",))
+        paths = self.client_log(self.svn_repo_url, 1, 0)[1][0]
+        self.assertEquals(paths,
+            {'/trunk': ('A', None, -1), '/trunk/a': ('A', None, -1)})
+        # FIXME: Check revision properties
+
+    def test_push_first_revision_without_metadata(self):
+        self.interrepo.push_single_revision("trunk",
+            self.interrepo._get_branch_config("trunk"),
+            self.from_repo.get_revision(self.revid1),
+            push_metadata=False, base_foreign_info=(None, None), root_action=("create",))
+        paths = self.client_log(self.svn_repo_url, 1, 0)[1][0]
+        self.assertEquals(paths,
+            {'/trunk': ('A', None, -1), '/trunk/a': ('A', None, -1)})
+        # FIXME: Check revision properties
+
+    def test_push_first_revision_overwrite(self):
+        ce = self.get_commit_editor(self.svn_repo_url, "msg")
+        ce.add_dir("trunk")
+        ce.close()
+
+        paths = self.client_log(self.svn_repo_url, 1, 0)[1][0]
+        self.assertEquals(paths, {'/trunk': ('A', None, -1)})
+
+        config = self.interrepo._get_branch_config("trunk")
+        rev1 = self.from_repo.get_revision(self.revid1)
+
+        # With append revisions only disabled but overwrite it should work
+        self.interrepo.push_single_revision("trunk", config, rev1,
+            push_metadata=False, base_foreign_info=(None, None),
+            root_action=("replace", 2))
+
+        paths = self.client_log(self.svn_repo_url, 2, 0)[2][0]
+        self.assertEquals(paths,
+            {'/trunk': ('R', None, -1), '/trunk/a': ('A', None, -1)})
+
+    def test_push_first_revision_append_revisions_only(self):
+        ce = self.get_commit_editor(self.svn_repo_url, "msg")
+        ce.add_dir("trunk")
+        ce.close()
+
+        paths = self.client_log(self.svn_repo_url, 1, 0)[1][0]
+        self.assertEquals(paths, {'/trunk': ('A', None, -1)})
+
+        config = self.interrepo._get_branch_config("trunk")
+        rev1 = self.from_repo.get_revision(self.revid1)
+
+        self.interrepo.push_single_revision("trunk", config, rev1,
+            push_metadata=False, base_foreign_info=(None, None), root_action=("replace", 1))
+
+        paths = self.client_log(self.svn_repo_url, 2, 0)[2][0]
+        self.assertEquals(paths,
+            {'/trunk': ('R', None, -1), '/trunk/a': ('A', None, -1)})
+
+
+class PushRevisionInclusiveTests(InterToSvnRepositoryTestCase):
+
+    # revid1    revid2
+    #   \       /
+    #  revid_merge
+
+    def setUp(self):
+        super(PushRevisionInclusiveTests, self).setUp()
+        branch = BzrDir.create_branch_convenience('bzrrepo/tree1')
+        self.tree1 = branch.bzrdir.open_workingtree()
+        self.build_tree_contents([('bzrrepo/tree1/a', 'data')])
+        self.tree1.add(['a'])
+        self.revid1 = self.tree1.commit('msg1')
+
+        branch = BzrDir.create_branch_convenience('bzrrepo/tree2')
+        self.tree2 = branch.bzrdir.open_workingtree()
+        self.build_tree_contents([('bzrrepo/tree2/b', 'data')])
+        self.tree2.add(['b'])
+        self.revid2 = self.tree2.commit('msg2')
+
+        self.tree1.merge_from_branch(self.tree2.branch, from_revision=NULL_REVISION,
+                to_revision=self.revid2)
+        self.revid_merge = self.tree1.commit("merge")
+
+    def test_push_no_merged(self):
+        config = self.interrepo._get_branch_config("trunk")
+        self.addCleanup(self.from_repo.unlock)
+        self.from_repo.lock_read()
+        rev1 = self.from_repo.get_revision(self.revid1)
+        self.interrepo.push_single_revision("trunk",
+            config, rev1, push_metadata=True, root_action=("create", ),
+            base_foreign_info=(None, None))
+        rev_merged = self.from_repo.get_revision(self.revid_merge)
+        foreign_rev_info = self.interrepo._get_foreign_revision_info(
+            rev_merged.parent_ids[0])
+        self.interrepo.push_revision_inclusive("trunk", config,
+            rev_merged, push_merged=False,
+            layout=TrunkLayout0(), project="",
+            push_metadata=True, root_action=("open",),
+            base_foreign_info=foreign_rev_info)
+        log = self.client_log(self.svn_repo_url, 2, 0)
+        self.assertEquals(log[1][0],
+            {'/trunk': ('A', None, -1), '/trunk/a': ('A', None, -1)})
+        self.assertEquals(log[2][0],
+            {'/trunk': ('M', None, -1), '/trunk/b': ('A', None, -1)})
+
+    def test_push_merged(self):
+        config = self.interrepo._get_branch_config("trunk")
+        self.addCleanup(self.from_repo.unlock)
+        self.from_repo.lock_read()
+        rev1 = self.from_repo.get_revision(self.revid1)
+        self.interrepo.push_single_revision("trunk",
+            config, rev1, push_metadata=True, root_action=("create", ),
+            base_foreign_info=(None, None))
+        rev_merged = self.from_repo.get_revision(self.revid_merge)
+        self.interrepo.push_revision_inclusive("trunk", config,
+            rev_merged, push_merged=True,
+            layout=TrunkLayout0(), project="",
+            push_metadata=True, root_action=("open", ),
+            base_foreign_info=self.interrepo._get_foreign_revision_info(
+                rev_merged.parent_ids[0]))
+        log = self.client_log(self.svn_repo_url, 4, 0)
+        self.assertEquals(log[1][0],
+            {'/trunk': ('A', None, -1), '/trunk/a': ('A', None, -1)})
+        self.assertEquals(log[2][0], {'/branches': ('A', None, -1)})
+        self.assertEquals(log[2][3], "Add branches directory.")
+        self.assertEquals(log[3][3], 'msg2')
+        self.assertEquals(log[3][0],
+            {'/branches/tree2': ('A', None, -1), '/branches/tree2/b': ('A', None, -1)})
+        self.assertEquals(log[4][0],
+            {'/trunk': ('M', None, -1),
+             '/trunk/b': ('A', '/branches/tree2/b', 3)})
+        self.assertEquals(log[4][3], 'merge')
+
+    def test_push_merged_again(self):
+        # Push a branch with a merge, then later push another branch with a
+        # merge.
+        config = self.interrepo._get_branch_config("trunk")
+        self.addCleanup(self.from_repo.unlock)
+        self.from_repo.lock_read()
+        rev1 = self.from_repo.get_revision(self.revid1)
+        self.interrepo.push_single_revision("trunk",
+            config, rev1, push_metadata=True, root_action=("create", ),
+            base_foreign_info=(None, None))
+        rev_merged = self.from_repo.get_revision(self.revid_merge)
+        self.interrepo.push_revision_inclusive("trunk", config,
+            rev_merged, push_merged=True,
+            layout=TrunkLayout0(), project="",
+            push_metadata=True, root_action=("open", ),
+            base_foreign_info=self.interrepo._get_foreign_revision_info(rev_merged.parent_ids[0]))
+
+        revid3 = self.tree2.commit('msg2 again')
+        self.tree1.merge_from_branch(self.tree2.branch, to_revision=revid3)
+        revid_merge = self.tree1.commit("merge again")
+        self.from_repo.unlock()
+        self.from_repo.lock_read()
+        rev_merged = self.from_repo.get_revision(revid_merge)
+        self.interrepo.push_revision_inclusive("trunk", config,
+            rev_merged, push_merged=True,
+            layout=TrunkLayout0(), project="",
+            push_metadata=True, root_action=("open",),
+            base_foreign_info=self.interrepo._get_foreign_revision_info(rev_merged.parent_ids[0]))
+
+        log = self.client_log(self.svn_repo_url, 6, 4)
+        self.assertEquals(log[5][3], 'msg2 again')
+        self.assertEquals(log[5][0], {'/branches/tree2': ('M', None, -1)})
+        self.assertEquals(log[6][0], {'/trunk': ('M', None, -1)})
+        self.assertEquals(log[6][3], 'merge again')
+
+
+class CreateBranchContainerTests(SubversionTestCase):
+
+    def setUp(self):
+        super(CreateBranchContainerTests, self).setUp()
+        self.repos_url = self.make_repository("d")
+        self.repo = Repository.open(self.repos_url)
+
+    def test_create_single(self):
+        create_branch_container(self.repo.transport, "branches/abranch", "")
+        log = self.client_log(self.repos_url, 1, 0)
+        self.assertEquals(log[1][0], {'/branches': ('A', None, -1)})
+        self.assertEquals(log[1][3], "Add branches directory.")
+
+    def test_create_double(self):
+        create_branch_container(self.repo.transport, "project/branches/abranch", "")
+        paths = self.client_log(self.repos_url, 1, 0)[1][0]
+        self.assertEquals(paths,
+            {'/project': ('A', None, -1), '/project/branches': ('A', None, -1)})
+
+    def test_part_exists(self):
+        dc = self.get_commit_editor(self.repos_url)
+        dc.add_dir("project")
+        dc.close()
+        create_branch_container(self.repo.transport, "project/branches/abranch", "project")
+        paths = self.client_log(self.repos_url, 2, 0)[2][0]
+        self.assertEquals(paths, {'/project/branches': ('A', None, -1)})
