@@ -32,6 +32,7 @@ from bzrlib.errors import (
 
 from bzrlib.plugins.svn.tests import SubversionTestCase
 from bzrlib.plugins.svn.transport import (
+    Connection,
     SvnRaTransport,
     bzr_to_svn_url,
     url_join_unescaped_path,
@@ -50,6 +51,17 @@ class SvnRaTest(SubversionTestCase):
         t = SvnRaTransport(repos_url)
         self.assertIsInstance(t, SvnRaTransport)
         self.assertEqual(t.base, repos_url)
+        self.assertEqual(t.is_readonly(), False)
+
+    def test_segments(self):
+        repos_url = self.make_repository('a')
+        t = SvnRaTransport("%s,branch=name" % repos_url)
+        self.assertIsInstance(t, SvnRaTransport)
+        self.assertEqual(t.svn_url, repos_url)
+        try:
+            self.assertEqual(t.get_segment_parameters(), {"branch": "name"})
+        except AttributeError: # bzr < 2.5
+            return
 
     def test_create_direct(self):
         repos_url = self.make_repository('a')
@@ -57,6 +69,13 @@ class SvnRaTest(SubversionTestCase):
         self.assertIsInstance(t, SvnRaTransport)
         self.assertEqual(t.base, repos_url)
 
+    def test_create_readonly(self):
+        repos_url = self.make_repository('a')
+        t = SvnRaTransport("readonly+"+repos_url)
+        self.assertIsInstance(t, SvnRaTransport)
+        self.assertEqual(t.base, "readonly+"+repos_url)
+        self.assertEqual(t.is_readonly(), True)
+
     def test_lock_read(self):
         repos_url = self.make_repository('a')
         t = SvnRaTransport(repos_url)
@@ -134,6 +153,11 @@ class SvnRaTest(SubversionTestCase):
         tt = t.clone()
         self.assertEqual(tt.base, t.base)
 
+    def test_mkdir_readonly(self):
+        repos_url = self.make_repository('a')
+        t = SvnRaTransport("readonly+"+repos_url)
+        self.assertRaises(TransportNotPossible, t.mkdir, "bla")
+
     def test_mkdir(self):
         repos_url = self.make_repository('a')
         t = SvnRaTransport(repos_url)
@@ -209,29 +233,39 @@ class SvnRaTest(SubversionTestCase):
 class UrlConversionTest(TestCase):
 
     def test_bzr_to_svn_url(self):
-        self.assertEqual("svn://host/location",
+        self.assertEqual(("svn://host/location", False),
                          bzr_to_svn_url("svn://host/location"))
-        self.assertEqual("svn+ssh://host/location",
+        self.assertEqual(("svn+ssh://host/location", False),
                          bzr_to_svn_url("svn+ssh://host/location"))
-        self.assertEqual("http://host/location",
+        self.assertEqual(("http://host/location", False),
                          bzr_to_svn_url("http://host/location"))
-        self.assertEqual("http://host/location",
+        self.assertEqual(("http://host/location", False),
                          bzr_to_svn_url("svn+http://host/location"))
-        self.assertEqual("http://host/gtk+/location",
+        self.assertEqual(("http://host/location", True),
+                         bzr_to_svn_url("readonly+http://host/location"))
+        self.assertEqual(("http://host/gtk+/location", False),
                          bzr_to_svn_url("svn+http://host/gtk%2B/location"))
 
     def test_url_unescape_uri(self):
         self.assertEquals("http://svn.gnome.org/svn/gtk+/trunk",
                 _url_unescape_uri("http://svn.gnome.org/svn/gtk%2B/trunk"))
+        self.assertEquals("http://svn.gnome.org/svn/gtk%20/trunk",
+                _url_unescape_uri("http://svn.gnome.org/svn/gtk%20/trunk"))
 
     def test_url_escape_uri(self):
         self.assertEquals("http://svn.gnome.org/svn/gtk+/trunk",
                 _url_escape_uri("http://svn.gnome.org/svn/gtk+/trunk"))
+        self.assertEquals("http://svn.gnome.org/svn/gtk%20/trunk",
+                _url_escape_uri("http://svn.gnome.org/svn/gtk /trunk"))
 
     def test_url_join_unescaped_path(self):
         self.assertEquals("http://svn.gnome.org/svn/gtk+/trunk",
                 url_join_unescaped_path("http://svn.gnome.org/svn/", "gtk+/trunk"))
 
+    def test_url_join_unescaped_comment(self):
+        self.assertEquals("http://svn.gnome.org/svn/gtk+/trunk%23",
+                url_join_unescaped_path("http://svn.gnome.org/svn/", "gtk+/trunk#"))
+
 
 class UrlSplitSvnPlus(TestCase):
 
@@ -242,3 +276,29 @@ class UrlSplitSvnPlus(TestCase):
                 fragment=''),
             urlparse.urlsplit('svn+http://foo.bar@host.com/svn/trunk'))
 
+
+class ReadonlyConnectionTests(SubversionTestCase):
+
+    def setUp(self):
+        super(ReadonlyConnectionTests, self).setUp()
+        self.repos_url = self.make_repository("d")
+
+    def test_get_latest_revnum(self):
+        self.recordRemoteAccessCalls()
+        conn = Connection(self.repos_url, readonly=True)
+        self.assertEquals(0, conn.get_latest_revnum())
+        self.assertRemoteAccessCalls([('get-latest-revnum', ())])
+
+    def test_change_rev_prop(self):
+        conn = Connection(self.repos_url, readonly=True)
+        self.recordRemoteAccessCalls()
+        self.assertRaises(TransportNotPossible, conn.change_rev_prop,
+             3, "foo", "bar")
+        self.assertRemoteAccessCalls([])
+
+    def test_get_commit_editor(self):
+        conn = Connection(self.repos_url, readonly=True)
+        self.recordRemoteAccessCalls()
+        self.assertRaises(TransportNotPossible, conn.get_commit_editor,
+                { "svn:log": "msg" } )
+        self.assertRemoteAccessCalls([])
