@@ -22,13 +22,22 @@ from bzrlib.inventory import (
     Inventory,
     TreeReference,
     )
-from bzrlib.osutils import has_symlinks
 from bzrlib.repository import Repository
 from bzrlib.revision import (
     NULL_REVISION,
     CURRENT_REVISION,
     )
-from bzrlib.tests import TestSkipped
+from bzrlib.tests import (
+    TestSkipped,
+    )
+try:
+    from bzrlib.tests.features import (
+        SymlinkFeature,
+        )
+except ImportError: # bzr < 2.5
+    from bzrlib.tests import (
+        SymlinkFeature,
+        )
 from bzrlib.workingtree import WorkingTree
 
 from bzrlib.plugins.svn.layout.standard import RootLayout
@@ -53,7 +62,7 @@ class TestBasisTree(SubversionTestCase):
         self.client_update("dc")
 
         tree = SvnBasisTree(WorkingTree.open("dc"))
-        self.assertTrue(tree.inventory[tree.inventory.path2id("file")].executable)
+        self.assertTrue(tree.is_executable(tree.path2id("file")))
 
     def test_executable_changed(self):
         repos_url = self.make_client("d", "dc")
@@ -65,7 +74,7 @@ class TestBasisTree(SubversionTestCase):
         self.client_update("dc")
         self.client_set_prop("dc/file", "svn:executable", "*")
         tree = SvnBasisTree(WorkingTree.open("dc"))
-        self.assertFalse(tree.inventory[tree.inventory.path2id("file")].executable)
+        self.assertFalse(tree.is_executable(tree.path2id("file")))
 
     def test_symlink(self):
         repos_url = self.make_client("d", "dc")
@@ -78,10 +87,10 @@ class TestBasisTree(SubversionTestCase):
 
         self.client_update("dc")
         tree = SvnBasisTree(WorkingTree.open("dc"))
-        self.assertEqual('symlink', 
-                         tree.inventory[tree.inventory.path2id("file")].kind)
+        self.assertEqual('symlink',
+                         tree.kind(tree.path2id("file")))
         self.assertEqual("target",
-                         tree.inventory[tree.inventory.path2id("file")].symlink_target)
+                         tree.get_symlink_target(tree.path2id("file")))
 
     def test_symlink_with_newlines_in_target(self):
         repos_url = self.make_client("d", "dc")
@@ -94,21 +103,21 @@ class TestBasisTree(SubversionTestCase):
 
         self.client_update("dc")
         tree = SvnBasisTree(WorkingTree.open("dc"))
-        self.assertEqual('symlink', 
-                         tree.inventory[tree.inventory.path2id("file")].kind)
+        self.assertEqual('symlink',
+                         tree.kind(tree.path2id("file")))
         self.assertEqual("target\nbar\nbla",
-                         tree.inventory[tree.inventory.path2id("file")].symlink_target)
+                         tree.get_symlink_target(tree.path2id("file")))
 
     def test_symlink_not_special(self):
         repos_url = self.make_client("d", "dc")
 
         dc = self.get_commit_editor(repos_url)
-        file = dc.add_file("file")
-        file.modify("fsdfdslhfdsk h")
-        file.change_prop("svn:special", "*")
+        file1 = dc.add_file("file")
+        file1.modify("fsdfdslhfdsk h")
+        file1.change_prop("svn:special", "*")
         file2 = dc.add_file("file2")
-        file.modify("a")
-        file.change_prop("svn:special", "*")
+        file2.modify("a")
+        file2.change_prop("svn:special", "*")
         dc.close()
 
         try:
@@ -118,8 +127,7 @@ class TestBasisTree(SubversionTestCase):
                 raise TestSkipped("Unable to run test with svn 1.4")
             raise
         tree = SvnBasisTree(WorkingTree.open("dc"))
-        self.assertEqual('file', 
-                         tree.inventory[tree.inventory.path2id("file")].kind)
+        self.assertEqual('file', tree.kind(tree.path2id("file")))
 
     def test_symlink_next(self):
         repos_url = self.make_client("d", "dc")
@@ -138,14 +146,11 @@ class TestBasisTree(SubversionTestCase):
         self.client_update("dc")
 
         tree = SvnBasisTree(WorkingTree.open("dc"))
-        self.assertEqual('symlink', 
-                         tree.inventory[tree.inventory.path2id("file")].kind)
+        self.assertEqual('symlink', tree.kind(tree.path2id("file")))
         self.assertEqual("target",
-                         tree.inventory[tree.inventory.path2id("file")].symlink_target)
+                         tree.get_symlink_target(tree.path2id("file")))
 
     def test_annotate_iter(self):
-        if subvertpy.__version__ < (0, 6, 5):
-            raise TestSkipped("Unable to run without subvertpy >= 0.6.4")
         repos_url = self.make_client("d", "dc")
 
         dc = self.get_commit_editor(repos_url)
@@ -188,8 +193,7 @@ class TestBasisTree(SubversionTestCase):
         self.assertEquals("bloe", props["bzrbla"])
 
     def test_executable_link(self):
-        if not has_symlinks():
-            return
+        self.requireFeature(SymlinkFeature)
         repos_url = self.make_client("d", "dc")
 
         dc = self.get_commit_editor(repos_url)
@@ -208,8 +212,8 @@ class TestBasisTree(SubversionTestCase):
 
         wt = WorkingTree.open("dc")
         tree = SvnBasisTree(wt)
-        self.assertFalse(tree.inventory[tree.inventory.path2id("file")].executable)
-        self.assertFalse(wt.inventory[wt.inventory.path2id("file")].executable)
+        self.assertFalse(tree.is_executable(tree.path2id("file")))
+        self.assertFalse(wt.is_executable(wt.path2id("file")))
 
 
 class TestInventoryExternals(SubversionTestCase):
@@ -220,14 +224,14 @@ class TestInventoryExternals(SubversionTestCase):
         repos = Repository.open(repos_url)
         mapping = repos.get_mapping()
         inv = Inventory(root_id='blabloe')
-        inventory_add_external(inv, 'blabloe', 'blie/bla', 
-                mapping.revision_id_foreign_to_bzr((repos.uuid, "", 1)), 
+        inventory_add_external(inv, 'blabloe', 'blie/bla',
+                mapping.revision_id_foreign_to_bzr((repos.uuid, "", 1)),
                 None, repos_url)
         self.assertEqual(TreeReference(
             mapping.generate_file_id((repos.uuid, "", 0), u""),
-             'bla', inv.path2id('blie'), 
+             'bla', inv.path2id('blie'),
              reference_revision=CURRENT_REVISION,
-             revision=mapping.revision_id_foreign_to_bzr((repos.uuid, "", 1))), 
+             revision=mapping.revision_id_foreign_to_bzr((repos.uuid, "", 1))),
              inv[inv.path2id('blie/bla')])
 
     def test_add_simple_norev(self):
@@ -235,15 +239,15 @@ class TestInventoryExternals(SubversionTestCase):
         repos = Repository.open(repos_url)
         mapping = repos.get_mapping()
         inv = Inventory(root_id='blabloe')
-        inventory_add_external(inv, 'blabloe', 'bla', 
-            mapping.revision_id_foreign_to_bzr((repos.uuid, "", 1)), None, 
+        inventory_add_external(inv, 'blabloe', 'bla',
+            mapping.revision_id_foreign_to_bzr((repos.uuid, "", 1)), None,
             repos_url)
 
         self.assertEqual(TreeReference(
             mapping.generate_file_id((repos.uuid, "", 0), u""),
-             'bla', 'blabloe', 
+             'bla', 'blabloe',
              reference_revision=CURRENT_REVISION,
-             revision=mapping.revision_id_foreign_to_bzr((repos.uuid, "", 1))), 
+             revision=mapping.revision_id_foreign_to_bzr((repos.uuid, "", 1))),
              inv[inv.path2id('bla')])
 
     def test_add_simple_rev(self):
@@ -251,15 +255,15 @@ class TestInventoryExternals(SubversionTestCase):
         repos = Repository.open(repos_url)
         inv = Inventory(root_id='blabloe')
         mapping = repos.get_mapping()
-        inventory_add_external(inv, 'blabloe', 'bla', 
+        inventory_add_external(inv, 'blabloe', 'bla',
             mapping.revision_id_foreign_to_bzr((repos.uuid, "", 1)), 0, repos_url)
         expected_ie = TreeReference(mapping.generate_file_id((repos.uuid, "", 0), u""),
-            'bla', 'blabloe', 
+            'bla', 'blabloe',
             revision=mapping.revision_id_foreign_to_bzr((repos.uuid, "", 1)),
             reference_revision=NULL_REVISION)
         ie = inv[inv.path2id('bla')]
         self.assertEqual(NULL_REVISION, ie.reference_revision)
-        self.assertEqual(mapping.revision_id_foreign_to_bzr((repos.uuid, "", 1)), 
+        self.assertEqual(mapping.revision_id_foreign_to_bzr((repos.uuid, "", 1)),
                          ie.revision)
         self.assertEqual(expected_ie, inv[inv.path2id('bla')])
 
@@ -275,18 +279,13 @@ class TestSvnRevisionTree(SubversionTestCase):
         self.repos = Repository.open(repos_url)
         self.repos.set_layout(RootLayout())
         mapping = self.repos.get_mapping()
-        self.inventory = self.repos.get_inventory(
-                self.repos.generate_revision_id(1, "", mapping))
         self.tree = self.repos.revision_tree(
                 self.repos.generate_revision_id(1, "", mapping))
 
-    def test_inventory(self):
-        self.assertIsInstance(self.tree.inventory, Inventory)
-        self.assertEqual(self.inventory, self.tree.inventory)
-
     def test_get_parent_ids(self):
         mapping = self.repos.get_mapping()
-        self.assertEqual((self.repos.generate_revision_id(0, "", mapping),), self.tree.get_parent_ids())
+        self.assertEqual((self.repos.generate_revision_id(0, "", mapping),),
+                self.tree.get_parent_ids())
 
     def test_get_parent_ids_zero(self):
         mapping = self.repos.get_mapping()
@@ -300,36 +299,34 @@ class TestSvnRevisionTree(SubversionTestCase):
                          self.tree.get_revision_id())
 
     def test_get_file_lines(self):
-        self.assertEqual(["data"], 
-                self.tree.get_file_lines(self.inventory.path2id("foo/bla")))
+        self.assertEqual(["data"],
+                self.tree.get_file_lines(self.tree.path2id("foo/bla")))
 
     def test_executable(self):
         self.client_set_prop("dc/foo/bla", "svn:executable", "*")
         self.client_commit("dc", "My Message")
 
         mapping = self.repos.get_mapping()
-        
-        inventory = self.repos.get_inventory(
+
+        tree = self.repos.revision_tree(
                 self.repos.generate_revision_id(2, "", mapping))
 
-        self.assertTrue(inventory[inventory.path2id("foo/bla")].executable)
+        self.assertTrue(tree.is_executable(tree.path2id("foo/bla")))
 
     def test_symlink(self):
-        if not has_symlinks():
-            return
+        self.requireFeature(SymlinkFeature)
         os.symlink('foo/bla', 'dc/bar')
         self.client_add('dc/bar')
         self.client_commit("dc", "My Message")
 
         mapping = self.repos.get_mapping()
-        
-        inventory = self.repos.get_inventory(
+
+        tree = self.repos.revision_tree(
                 self.repos.generate_revision_id(2, "", mapping))
 
-        self.assertEqual('symlink', inventory[inventory.path2id("bar")].kind)
-        self.assertEqual('foo/bla', 
-                inventory[inventory.path2id("bar")].symlink_target)
+        self.assertEqual('symlink', tree.kind(tree.path2id("bar")))
+        self.assertEqual('foo/bla',
+                tree.get_symlink_target(tree.path2id("bar")))
 
     def test_not_executable(self):
-        self.assertFalse(self.inventory[
-            self.inventory.path2id("foo/bla")].executable)
+        self.assertFalse(self.tree.is_executable(self.tree.path2id("foo/bla")))
