@@ -19,7 +19,14 @@
 
 import os
 
-from bzrlib import osutils
+from subvertpy.wc import (
+    check_wc,
+    )
+
+from bzrlib import (
+    osutils,
+    version_info as bzrlib_version,
+    )
 from bzrlib.branch import Branch
 from bzrlib.bzrdir import BzrDir
 from bzrlib.errors import (
@@ -29,14 +36,23 @@ from bzrlib.errors import (
     )
 from bzrlib.inventory import Inventory
 from bzrlib.osutils import (
-    has_symlinks,
     supports_executable,
     )
 from bzrlib.repository import Repository
 from bzrlib.tests import (
+    KnownFailure,
     TestCase,
-    TestSkipped,
     )
+try:
+    from bzrlib.tests.features import (
+        SymlinkFeature,
+        UnicodeFilenameFeature,
+        )
+except ImportError: # bzr < 2.5
+    from bzrlib.tests import (
+        SymlinkFeature,
+        UnicodeFilenameFeature,
+        )
 from bzrlib.trace import mutter
 from bzrlib.workingtree import WorkingTree
 
@@ -45,10 +61,29 @@ from bzrlib.plugins.svn.mapping3.base import config_set_scheme
 from bzrlib.plugins.svn.mapping3.scheme import TrunkBranchingScheme
 from bzrlib.plugins.svn.transport import svn_config
 from bzrlib.plugins.svn.tests import SubversionTestCase
-from bzrlib.plugins.svn.workingtree import generate_ignore_list
+from bzrlib.plugins.svn.workingtree import (
+    CorruptWorkingTree,
+    generate_ignore_list,
+    )
 
 class TestWorkingTree(SubversionTestCase):
 
+    def assertCleanTree(self, wt):
+        delta = wt.changes_from(wt.basis_tree())
+        self.assertFalse(delta.has_changed(), delta)
+
+    def test_invalid_entries(self):
+        self.make_client('a', 'dc')
+        entries_path = os.path.join(self.test_dir, 'dc/.svn/entries')
+        os.chmod(entries_path, 0755)
+        num = check_wc("dc")
+        f = open(entries_path, "w+")
+        try:
+            f.write("%d\n" % num)
+        finally:
+            f.close()
+        self.assertRaises(CorruptWorkingTree, WorkingTree.open, "dc")
+
     def test_add_duplicate(self):
         self.make_client('a', 'dc')
         self.build_tree({"dc/bl": "data"})
@@ -67,22 +102,18 @@ class TestWorkingTree(SubversionTestCase):
         tree = WorkingTree.open("dc")
         tree.add(["bl"])
 
-        inv = tree.read_working_inventory()
-        self.assertIsInstance(inv, Inventory)
-        self.assertTrue(inv.has_filename("bl"))
-        self.assertFalse(inv.has_filename("aa"))
+        self.assertEquals(
+            set(["aa"]), tree.filter_unversioned_files(["bl", "aa"]))
 
     def test_special_char(self):
+        self.requireFeature(UnicodeFilenameFeature)
         self.make_client('a', 'dc')
-        try:
-            self.build_tree({u"dc/I²C": "data"})
-        except UnicodeError:
-            raise TestSkipped("This platform does not support unicode paths")
+        self.build_tree({u"dc/I²C": "data"})
         self.client_add("dc/I²C")
         tree = WorkingTree.open("dc")
-        inv = tree.read_working_inventory()
-        self.assertIsInstance(inv, Inventory)
-        self.assertTrue(inv.has_filename(u"I²C"))
+        self.assertEquals(
+            set(),
+            tree.filter_unversioned_files([u"I²C"]))
 
     def test_not_branch_path(self):
         repos_url = self.make_client('a', 'dc')
@@ -104,23 +135,16 @@ class TestWorkingTree(SubversionTestCase):
         self.build_tree({"dc/bl": "data"})
         tree = WorkingTree.open("dc")
         tree.smart_add(["dc/bl"])
-
-        inv = tree.read_working_inventory()
-        self.assertIsInstance(inv, Inventory)
-        self.assertTrue(inv.has_filename("bl"))
-        self.assertFalse(inv.has_filename("aa"))
+        self.assertEquals(
+            set(["aa"]), tree.filter_unversioned_files(["bl", "aa"]))
 
     def test_smart_add_unicode(self):
-        try:
-            self.make_client('a', u'dć'.encode(osutils._fs_enc))
-        except UnicodeDecodeError:
-            raise TestSkipped("This platform does not support unicode paths")
+        self.requireFeature(UnicodeFilenameFeature)
+        self.make_client('a', u'dć'.encode(osutils._fs_enc))
         self.build_tree({u"dć/bé".encode(osutils._fs_enc): "data"})
         tree = WorkingTree.open(u"dć")
         tree.smart_add([u"dć/bé"])
-        inv = tree.read_working_inventory()
-        self.assertIsInstance(inv, Inventory)
-        self.assertTrue(inv.has_filename(u"bé"))
+        self.assertEquals(set(), tree.filter_unversioned_files([u"bé"]))
 
     def test_is_control_filename(self):
         self.make_client('a', 'dc')
@@ -135,11 +159,8 @@ class TestWorkingTree(SubversionTestCase):
         tree = WorkingTree.open("dc")
         tree.smart_add(["dc/bl"])
 
-        inv = tree.read_working_inventory()
-        self.assertIsInstance(inv, Inventory)
-        self.assertTrue(inv.has_filename("bl"))
-        self.assertTrue(inv.has_filename("bl/foo"))
-        self.assertFalse(inv.has_filename("aa"))
+        self.assertEquals(set(["aa"]),
+            tree.filter_unversioned_files(["bl", "bl/foo", "aa"]))
 
     def test_smart_add_recurse_more(self):
         self.make_client('a', 'dc')
@@ -147,12 +168,8 @@ class TestWorkingTree(SubversionTestCase):
         tree = WorkingTree.open("dc")
         tree.smart_add(["dc/bl"])
 
-        inv = tree.read_working_inventory()
-        self.assertIsInstance(inv, Inventory)
-        self.assertTrue(inv.has_filename("bl"))
-        self.assertTrue(inv.has_filename("bl/foo"))
-        self.assertTrue(inv.has_filename("bl/foo/da"))
-        self.assertFalse(inv.has_filename("aa"))
+        self.assertEquals(set(["aa"]),
+            tree.filter_unversioned_files(["bl", "bl/foo", "bl/foo/da", "aa"]))
 
     def test_smart_add_more(self):
         self.make_client('a', 'dc')
@@ -160,13 +177,8 @@ class TestWorkingTree(SubversionTestCase):
         tree = WorkingTree.open("dc")
         tree.smart_add(["dc/bl", "dc/ha"])
 
-        inv = tree.read_working_inventory()
-        self.assertIsInstance(inv, Inventory)
-        self.assertTrue(inv.has_filename("bl"))
-        self.assertTrue(inv.has_filename("bl/foo"))
-        self.assertTrue(inv.has_filename("bl/foo/da"))
-        self.assertTrue(inv.has_filename("ha"))
-        self.assertFalse(inv.has_filename("aa"))
+        self.assertEquals(set(["aa"]),
+            tree.filter_unversioned_files(["bl", "bl/foo", "bl/foo/da", "ha", "aa"]))
 
     def test_smart_add_ignored(self):
         self.make_client('a', 'dc')
@@ -175,13 +187,8 @@ class TestWorkingTree(SubversionTestCase):
         tree = WorkingTree.open("dc")
         tree.smart_add(["dc/bl"])
 
-        inv = tree.read_working_inventory()
-        self.assertIsInstance(inv, Inventory)
-        self.assertTrue(inv.has_filename("bl"))
-        self.assertTrue(inv.has_filename("bl/foo"))
-        self.assertTrue(inv.has_filename("bl/foo/da"))
-        self.assertFalse(inv.has_filename("ha"))
-        self.assertFalse(inv.has_filename("aa"))
+        self.assertEquals(set(["ha", "aa"]),
+            tree.filter_unversioned_files(["bl", "bl/foo", "bl/foo/da", "ha", "aa"]))
 
     def test_smart_add_toplevel(self):
         self.make_client('a', 'dc')
@@ -189,13 +196,8 @@ class TestWorkingTree(SubversionTestCase):
         tree = WorkingTree.open("dc")
         tree.smart_add(["dc"])
 
-        inv = tree.read_working_inventory()
-        self.assertIsInstance(inv, Inventory)
-        self.assertTrue(inv.has_filename("bl"))
-        self.assertTrue(inv.has_filename("bl/foo"))
-        self.assertTrue(inv.has_filename("bl/foo/da"))
-        self.assertTrue(inv.has_filename("ha"))
-        self.assertFalse(inv.has_filename("aa"))
+        self.assertEquals(set(["aa"]),
+            tree.filter_unversioned_files(["bl", "bl/foo", "bl/foo/da", "ha", "aa"]))
 
     def test_add_nolist(self):
         self.make_client('a', 'dc')
@@ -203,10 +205,8 @@ class TestWorkingTree(SubversionTestCase):
         tree = WorkingTree.open("dc")
         tree.add("bl")
 
-        inv = tree.read_working_inventory()
-        self.assertIsInstance(inv, Inventory)
-        self.assertTrue(inv.has_filename("bl"))
-        self.assertFalse(inv.has_filename("aa"))
+        self.assertEquals(
+            set(["aa"]), tree.filter_unversioned_files(["bl", "aa"]))
 
     def test_add_nolist_withid(self):
         self.make_client('a', 'dc')
@@ -214,11 +214,10 @@ class TestWorkingTree(SubversionTestCase):
         tree = WorkingTree.open("dc")
         tree.add("bl", "bloe")
 
-        inv = tree.read_working_inventory()
-        self.assertIsInstance(inv, Inventory)
-        self.assertTrue(inv.has_filename("bl"))
-        self.assertFalse(inv.has_filename("aa"))
-        self.assertEqual("bloe", tree.inventory.path2id("bl"))
+        self.assertEqual(
+            set(["aa"]),
+            tree.filter_unversioned_files(["bl", "aa"]))
+        self.assertEqual("bloe", tree.path2id("bl"))
 
     def test_add_not_recursive(self):
         self.make_client('a', 'dc')
@@ -227,8 +226,8 @@ class TestWorkingTree(SubversionTestCase):
         tree.add(["bl"])
 
         tree = WorkingTree.open("dc")
-        self.assertTrue(tree.inventory.has_filename("bl"))
-        self.assertFalse(tree.inventory.has_filename("bl/file"))
+        self.assertEquals(set(["bl/file"]),
+            tree.filter_unversioned_files(["bl/file", "bl"]))
 
     def test_add_nested(self):
         self.make_client('a', 'dc')
@@ -237,8 +236,8 @@ class TestWorkingTree(SubversionTestCase):
         tree.add(["bl", "bl/file"])
 
         tree = WorkingTree.open("dc")
-        self.assertTrue(tree.inventory.has_filename("bl"))
-        self.assertTrue(tree.inventory.has_filename("bl/file"))
+        self.assertEquals(set(),
+            tree.filter_unversioned_files(["bl", "bl/file"]))
 
     def test_lock_write(self):
         self.make_client('a', 'dc')
@@ -255,7 +254,8 @@ class TestWorkingTree(SubversionTestCase):
     def test_get_ignore_list_empty(self):
         self.make_client('a', 'dc')
         tree = WorkingTree.open("dc")
-        self.assertEqual(set([".svn"] + svn_config.get_default_ignores()), tree.get_ignore_list())
+        self.assertEqual(set([".svn"] + svn_config.get_default_ignores()),
+            tree.get_ignore_list())
 
     def test_get_ignore_list_onelevel(self):
         self.make_client('a', 'dc')
@@ -278,8 +278,8 @@ class TestWorkingTree(SubversionTestCase):
         tree = WorkingTree.open("dc")
         tree.add(["bl"])
 
-        inv = WorkingTree.open("dc").read_working_inventory()
-        self.assertTrue(inv.has_filename("bl"))
+        tree = WorkingTree.open("dc")
+        self.assertEquals(set(), tree.filter_unversioned_files(["bl"]))
 
     def test_remove(self):
         self.make_client('a', 'dc')
@@ -287,8 +287,7 @@ class TestWorkingTree(SubversionTestCase):
         tree = WorkingTree.open("dc")
         tree.add(["bl"])
         tree.remove(["bl"])
-        inv = tree.read_working_inventory()
-        self.assertFalse(inv.has_filename("bl"))
+        self.assertEquals(set(["bl"]), tree.filter_unversioned_files(["bl"]))
 
     def test_remove_dup(self):
         self.make_client('a', 'dc')
@@ -296,8 +295,7 @@ class TestWorkingTree(SubversionTestCase):
         tree = WorkingTree.open("dc")
         tree.add(["bl"])
         os.remove("dc/bl")
-        inv = tree.read_working_inventory()
-        self.assertFalse(inv.has_filename("bl"))
+        self.assertEquals(set([]), tree.filter_unversioned_files(["bl"]))
 
     def test_is_control_file(self):
         self.make_client('a', 'dc')
@@ -313,8 +311,10 @@ class TestWorkingTree(SubversionTestCase):
         self.client_update("dc")
         tree = WorkingTree.open("dc")
         os.remove("dc/bl")
+        if bzrlib_version < (2, 4, 0):
+            raise KnownFailure("revert is known broken with bzr 2.3")
         tree.revert(["bl"])
-        self.assertFalse(tree.changes_from(tree.basis_tree()).has_changed())
+        self.assertCleanTree(tree)
         self.assertEqual("data", open('dc/bl').read())
 
     def test_rename_one(self):
@@ -325,20 +325,24 @@ class TestWorkingTree(SubversionTestCase):
         tree = WorkingTree.open("dc")
         tree.rename_one("bl", "bloe")
 
-        basis_inv = tree.basis_tree().inventory
-        inv = tree.read_working_inventory()
-        self.assertFalse(inv.has_filename("bl"))
-        self.assertTrue(inv.has_filename("bloe"))
-        self.assertEqual(basis_inv.path2id("bl"),
-                         inv.path2id("bloe"))
-        self.assertIs(None, inv.path2id("bl"))
-        self.assertIs(None, basis_inv.path2id("bloe"))
+        basis_tree = tree.basis_tree()
+        self.assertEquals(set(["bl"]), tree.filter_unversioned_files(["bl", "bloe"]))
+        self.assertEqual(basis_tree.path2id("bl"), tree.path2id("bloe"))
+        self.assertIs(None, tree.path2id("bl"))
+        self.assertIs(None, basis_tree.path2id("bloe"))
+
+    def test_set_root_id(self):
+        self.make_client('a', 'dc')
+        wt = WorkingTree.open("dc")
+        wt.set_root_id("somefileid")
+        self.assertEquals("somefileid", wt.path2id(""))
+        self.assertEquals("somefileid", wt.get_root_id())
 
     def test_empty_basis_tree(self):
         self.make_client('a', 'dc')
         wt = WorkingTree.open("dc")
         self.assertEqual(wt.branch.generate_revision_id(0),
-                         wt.basis_tree().inventory.revision_id)
+                         wt.basis_tree().get_revision_id())
         inv = Inventory()
         root_id = wt.branch.repository.get_mapping().generate_file_id((wt.branch.repository.uuid, "", 0), u"")
         inv.revision_id = wt.branch.generate_revision_id(0)
@@ -366,17 +370,10 @@ class TestWorkingTree(SubversionTestCase):
         self.client_commit("dc", "Bla")
         tree = WorkingTree.open("dc")
         tree.move(["bl", "a"], "dir")
-
-        basis_inv = tree.basis_tree().inventory
-        inv = tree.read_working_inventory()
-        self.assertFalse(inv.has_filename("bl"))
-        self.assertFalse(inv.has_filename("a"))
-        self.assertTrue(inv.has_filename("dir/bl"))
-        self.assertTrue(inv.has_filename("dir/a"))
-        mutter('basis: %r' % basis_inv.entries())
-        mutter('working: %r' % inv.entries())
-        self.assertFalse(inv.has_filename("bl"))
-        self.assertFalse(basis_inv.has_filename("dir/bl"))
+        self.assertEquals(
+            set(["bl", "a"]),
+            tree.filter_unversioned_files(["bl", "a", "dir/bl", "dir/a"]))
+        self.assertFalse(tree.basis_tree().has_filename("dir/bl"))
 
     def test_get_parent_ids_no_merges(self):
         repos_url = self.make_client('a', 'dc')
@@ -423,23 +420,6 @@ class TestWorkingTree(SubversionTestCase):
         self.assertEquals(tree.branch, result.target_branch)
         self.assertEquals(br, result.source_branch)
 
-    def test_working_inventory(self):
-        self.make_client('a', 'dc')
-        self.build_tree({"dc/bl": "data", "dc/foo/bar": "bla", "dc/foo/bla": "aa"})
-        self.client_add("dc/bl")
-        self.client_add("dc/foo")
-        self.client_commit("dc", "bla")
-        self.build_tree({"dc/test": "data"})
-        self.client_add("dc/test")
-        tree = WorkingTree.open("dc")
-        inv = tree.read_working_inventory()
-        self.assertEqual(inv.path2id(""), inv.root.file_id)
-        self.assertTrue(inv.path2id("foo") != "")
-        self.assertTrue(inv.has_filename("bl"))
-        self.assertTrue(inv.has_filename("foo"))
-        self.assertTrue(inv.has_filename("foo/bar"))
-        self.assertTrue(inv.has_filename("test"))
-
     def test_ignore_list(self):
         self.make_client('a', 'dc')
         self.build_tree({"dc/bl": None})
@@ -486,7 +466,8 @@ class TestWorkingTree(SubversionTestCase):
         self.build_tree({"dc/bl": None})
 
         tree = WorkingTree.open("dc")
-        self.assertFalse(tree.inventory.has_filename("bl"))
+        self.assertEquals(set(["bl"]),
+            tree.filter_unversioned_files(["bl"]))
 
     def test_fileid_missing(self):
         self.make_client("repos", "svn-wc")
@@ -529,19 +510,16 @@ class TestWorkingTree(SubversionTestCase):
         self.client_add("dc/bla")
         self.client_set_prop("dc/bla", "svn:executable", "*")
         tree = WorkingTree.open("dc")
-        inv = tree.read_working_inventory()
-        self.assertTrue(inv[inv.path2id("bla")].executable)
+        self.assertTrue(tree.is_executable(tree.path2id("bla")))
 
     def test_symlink(self):
-        if not has_symlinks():
-            return
+        self.requireFeature(SymlinkFeature)
         self.make_client('a', 'dc')
         os.symlink("target", "dc/bla")
         self.client_add("dc/bla")
         tree = WorkingTree.open("dc")
-        inv = tree.read_working_inventory()
-        self.assertEqual('symlink', inv[inv.path2id("bla")].kind)
-        self.assertEqual("target", inv[inv.path2id("bla")].symlink_target)
+        self.assertEqual('symlink', tree.kind(tree.path2id("bla")))
+        self.assertEqual("target", tree.get_symlink_target(tree.path2id("bla")))
 
     def test_get_parent_ids(self):
         repos_url = self.make_client('a', 'dc')
@@ -611,25 +589,26 @@ class TestWorkingTree(SubversionTestCase):
         self.client_add("dc/bl")
         tree = WorkingTree.open("dc")
         orig_tree = tree.basis_tree()
+        self.assertTrue(tree.changes_from(tree.basis_tree()).has_changed())
         tree.commit(message="data")
+        self.assertCleanTree(tree)
         self.assertEqual(
                 tree.branch.generate_revision_id(1),
                 tree.basis_tree().get_revision_id())
-        delta = tree.basis_tree().changes_from(tree.branch.repository.revision_tree(tree.branch.generate_revision_id(0)))
-        self.assertTrue(delta.has_changed())
+        rev0tree = tree.branch.repository.revision_tree(tree.branch.generate_revision_id(0))
+        delta = tree.basis_tree().changes_from(rev0tree)
+        self.assertTrue(delta.has_changed(), repr(delta))
         tree = WorkingTree.open("dc")
-        delta = tree.basis_tree().changes_from(tree)
         self.assertEqual(
              tree.branch.generate_revision_id(1),
              tree.basis_tree().get_revision_id())
-        self.assertFalse(delta.has_changed())
+        self.assertCleanTree(tree)
 
     def test_status(self):
         self.make_client('a', 'dc')
         tree = WorkingTree.open("dc")
         self.assertTrue(os.path.exists(os.path.join("dc", ".svn")))
         self.assertFalse(os.path.exists(os.path.join("dc", ".bzr")))
-        tree.read_working_inventory()
 
     def test_update(self):
         repos_url = self.make_client('a', 'dc')
@@ -642,6 +621,33 @@ class TestWorkingTree(SubversionTestCase):
         self.assertTrue(os.path.exists(os.path.join("de", ".svn")))
         self.assertTrue(os.path.exists(os.path.join("de", "bla")))
 
+    def test_update_revision(self):
+        repos_url = self.make_client('a', 'dc')
+        self.make_checkout(repos_url, "de")
+        self.build_tree({'dc/bla': "data"})
+        self.client_add("dc/bla")
+        self.client_commit("dc", "msg")
+        self.build_tree({'dc/bla': "data2"})
+        self.client_commit("dc", "msg2")
+        tree = WorkingTree.open("de")
+        tree.update(revision=tree.branch.generate_revision_id(1))
+        self.assertTrue(os.path.exists(os.path.join("de", ".svn")))
+        self.assertFileEqual("data", "de/bla")
+
+    def test_update_old_tip(self):
+        repos_url = self.make_client('a', 'dc')
+        self.make_checkout(repos_url, "de")
+        self.build_tree({'dc/bla': "data"})
+        self.client_add("dc/bla")
+        self.client_commit("dc", "msg")
+        self.build_tree({'dc/bla': "data2"})
+        self.client_commit("dc", "msg2")
+        tree = WorkingTree.open("de")
+        tree.update(revision=tree.branch.generate_revision_id(1),
+                    old_tip=tree.branch.generate_revision_id(0))
+        self.assertTrue(os.path.exists(os.path.join("de", ".svn")))
+        self.assertFileEqual("data", "de/bla")
+
     def test_status_bzrdir(self):
         self.make_client('a', 'dc')
         bzrdir = BzrDir.open("dc")
@@ -654,9 +660,9 @@ class TestWorkingTree(SubversionTestCase):
         self.build_tree({'dc/file': 'data'})
         tree = WorkingTree.open("dc")
         tree.add(["file"])
-        oldid = tree.inventory.path2id("file")
+        oldid = tree.path2id("file")
         tree = WorkingTree.open("dc")
-        newid = tree.inventory.path2id("file")
+        newid = tree.path2id("file")
         self.assertEqual(oldid, newid)
 
     def test_file_id_kept(self):
@@ -664,9 +670,9 @@ class TestWorkingTree(SubversionTestCase):
         self.build_tree({'dc/file': 'data'})
         tree = WorkingTree.open("dc")
         tree.add(["file"], ["fooid"])
-        self.assertEqual("fooid", tree.inventory.path2id("file"))
+        self.assertEqual("fooid", tree.path2id("file"))
         tree = WorkingTree.open("dc")
-        self.assertEqual("fooid", tree.inventory.path2id("file"))
+        self.assertEqual("fooid", tree.path2id("file"))
 
     def test_file_rename_id(self):
         self.make_client('a', 'dc')
@@ -677,18 +683,18 @@ class TestWorkingTree(SubversionTestCase):
         tree.rename_one("file", "file2")
         delta = tree.branch.repository.get_revision_delta(tree.last_revision())
         self.assertEquals([("file", "fooid", "file")], delta.added)
-        self.assertEqual(None, tree.inventory.path2id("file"))
-        self.assertEqual("fooid", tree.inventory.path2id("file2"))
+        self.assertEqual(None, tree.path2id("file"))
+        self.assertEqual("fooid", tree.path2id("file2"))
         tree = WorkingTree.open("dc")
-        self.assertEqual("fooid", tree.inventory.path2id("file2"))
+        self.assertEqual("fooid", tree.path2id("file2"))
 
     def test_file_id_kept_2(self):
         self.make_client('a', 'dc')
         self.build_tree({'dc/file': 'data', 'dc/other': 'blaid'})
         tree = WorkingTree.open("dc")
         tree.add(["file", "other"], ["fooid", "blaid"])
-        self.assertEqual("fooid", tree.inventory.path2id("file"))
-        self.assertEqual("blaid", tree.inventory.path2id("other"))
+        self.assertEqual("fooid", tree.path2id("file"))
+        self.assertEqual("blaid", tree.path2id("other"))
 
     def test_file_remove_id(self):
         self.make_client('a', 'dc')
@@ -697,9 +703,9 @@ class TestWorkingTree(SubversionTestCase):
         tree.add(["file"], ["fooid"])
         tree.commit("msg")
         tree.remove(["file"])
-        self.assertEqual(None, tree.inventory.path2id("file"))
+        self.assertEqual(None, tree.path2id("file"))
         tree = WorkingTree.open("dc")
-        self.assertEqual(None, tree.inventory.path2id("file"))
+        self.assertEqual(None, tree.path2id("file"))
 
     def test_file_move_id(self):
         self.make_client('a', 'dc')
@@ -707,12 +713,13 @@ class TestWorkingTree(SubversionTestCase):
         tree = WorkingTree.open("dc")
         tree.add(["file", "dir"], ["fooid", "blaid"])
         tree.commit("msg")
+        self.assertEqual("fooid", tree.path2id("file"))
         tree.move(["file"], "dir")
-        self.assertEqual(None, tree.inventory.path2id("file"))
-        self.assertEqual("fooid", tree.inventory.path2id("dir/file"))
+        self.assertEqual(None, tree.path2id("file"))
+        self.assertEqual("fooid", tree.path2id("dir/file"))
         tree = WorkingTree.open("dc")
-        self.assertEqual(None, tree.inventory.path2id("file"))
-        self.assertEqual("fooid", tree.inventory.path2id("dir/file"))
+        self.assertEqual(None, tree.path2id("file"))
+        self.assertEqual("fooid", tree.path2id("dir/file"))
 
     def test_escaped_char_filename(self):
         self.make_client('a', 'dc')
@@ -720,7 +727,7 @@ class TestWorkingTree(SubversionTestCase):
         tree = WorkingTree.open("dc")
         tree.add(["file with spaces"], ["fooid"])
         tree.commit("msg")
-        self.assertEqual("fooid", tree.inventory.path2id("file with spaces"))
+        self.assertEqual("fooid", tree.path2id("file with spaces"))
 
     def test_get_branch_nick(self):
         self.make_client('a', 'dc')
@@ -745,13 +752,10 @@ class TestWorkingTree(SubversionTestCase):
         self.assertRaises(OutOfDateTree, lambda: tree.commit("bar"))
 
     def test_unicode_symlink(self):
-        if not has_symlinks():
-            return
+        self.requireFeature(SymlinkFeature)
+        self.requireFeature(UnicodeFilenameFeature)
         repos_url = self.make_client('a', 'dc')
-        try:
-            self.build_tree({u"dc/\U00020001".encode(osutils._fs_enc): ""})
-        except UnicodeError:
-            raise TestSkipped("This platform does not support unicode code paths")
+        self.build_tree({u"dc/\U00020001".encode(osutils._fs_enc): ""})
         os.symlink(u"\U00020001", "dc/a")
         self.build_tree({"dc/b": ""})
         os.symlink("b", u"dc/\U00020002")
@@ -760,7 +764,8 @@ class TestWorkingTree(SubversionTestCase):
         self.client_add(u"dc/\U00020001".encode("utf-8"))
         self.client_add(u"dc/\U00020002".encode("utf-8"))
         self.client_commit("dc", "Added files and links")
-        WorkingTree.open("dc").read_working_inventory()
+        wt = WorkingTree.open("dc")
+        list(wt.iter_entries_by_dir())
 
     def test_get_dir_properties(self):
         repos_url = self.make_client('a', 'dc')
@@ -787,6 +792,34 @@ class TestWorkingTree(SubversionTestCase):
         wt = WorkingTree.open('dc')
         wt.apply_inventory_delta([("bla", None, wt.path2id("bla"), None)])
 
+    def test_keywords_commit(self):
+        repos_url = self.make_client('a', 'dc')
+        # md5sum: a31b9ad28573e11e3e653a0c038c49b5
+        self.build_tree({'dc/file.txt': 'This is a file with a $Id$'})
+        self.client_add('dc/file.txt')
+        self.client_set_prop('dc/file.txt', 'svn:keywords', 'Id')
+        self.client_commit('dc', "Initial commit")
+        wt = WorkingTree.open('dc')
+        self.assertCleanTree(wt)
+        # md5sum: 2936900e291f6c12d1c2ffc7b83f4da1
+        self.build_tree({'dc/file.txt': 'This is a file with a $Id$\n'
+                                        'New line added\n'})
+        wt.commit("Commit via bzr")
+        self.assertCleanTree(wt)
+        self.build_tree({'dc/file.txt': 'This is a file with a $Id$\n'
+                                        'New line added\n'
+                                        'Another change\n'})
+        self.client_commit('dc', "Commit via svn")
+
+    def test_revision_tree(self):
+        repos_url = self.make_client('a', 'dc')
+        # md5sum: a31b9ad28573e11e3e653a0c038c49b5
+        self.build_tree({'dc/file.txt': 'Foo'})
+        self.client_add('dc/file.txt')
+        wt = WorkingTree.open('dc')
+        revid = wt.commit("Initial commit")
+        self.assertEquals(revid, wt.revision_tree(revid).get_revision_id())
+
 
 class IgnoreListTests(TestCase):
 
@@ -804,5 +837,3 @@ class IgnoreListTests(TestCase):
     def test_multiple(self):
         self.assertEquals(["./twin*", "./twin/peaks"],
                 generate_ignore_list({"twin": "peaks", "": "twin*"}))
-
-
