@@ -16,11 +16,15 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 """Simple transport for accessing Subversion smart servers."""
 
-import itertools
 import stat
 import subvertpy
-from subvertpy import ERR_FS_NOT_DIRECTORY
+from subvertpy import (
+    AUTH_PARAM_DEFAULT_USERNAME,
+    AUTH_PARAM_DEFAULT_PASSWORD,
+    ERR_FS_NOT_DIRECTORY,
+    )
 from subvertpy.client import get_config
+from subvertpy.ra import RemoteAccess
 import sys
 import urlparse
 urlparse.uses_netloc.extend(['svn+http', 'svn+https'])
@@ -34,7 +38,6 @@ from bzrlib import (
     )
 from bzrlib.errors import (
     BzrError,
-    DependencyNotPresent,
     FileExists,
     InvalidURL,
     NoSuchFile,
@@ -52,22 +55,23 @@ from bzrlib.transport import (
 
 import bzrlib.plugins.svn
 from bzrlib.plugins.svn.auth import (
-    AUTH_PARAM_DEFAULT_USERNAME,
-    AUTH_PARAM_DEFAULT_PASSWORD,
     create_auth_baton,
     )
+from bzrlib.plugins.svn.changes import (
+    common_prefix,
+    )
 from bzrlib.plugins.svn.errors import (
     convert_svn_error,
+    DavRequestFailed,
     NoSvnRepositoryPresent,
     convert_error,
     )
 
-ERR_MALFORMED_FILE = getattr(subvertpy, "ERR_MALFORMED_FILE", 200002)
 
 try:
     svn_config = get_config()
 except subvertpy.SubversionException, (msg, num):
-    if num == ERR_MALFORMED_FILE:
+    if num == subvertpy.ERR_MALFORMED_FILE:
         raise BzrError(msg)
     raise
 
@@ -83,53 +87,13 @@ def get_test_permutations():
     return []
 
 
-def dav_options(transport, url):
-    # FIXME: Integrate this into HttpTransport.options().
-    from bzrlib.transport.http._urllib import HttpTransport_urllib, Request
-    if isinstance(transport, HttpTransport_urllib):
-        req = Request('OPTIONS', url, accepted_errors=[200, 403, 404, 405])
-        req.follow_redirections = True
-        resp = transport._perform(req)
-        if resp.code == 404:
-            raise NoSuchFile(transport._path)
-        if resp.code in (403, 405):
-            raise subvertpy.SubversionException("version control not supported for remote URL", subvertpy.ERR_RA_DAV_NOT_VCC)
-        return resp.headers.getheaders('DAV')
-    else:
-        try:
-            from bzrlib.transport.http._pycurl import PyCurlTransport
-        except DependencyNotPresent:
-            pass
-        else:
-            import pycurl
-            from cStringIO import StringIO
-            if isinstance(transport, PyCurlTransport):
-                conn = transport._get_curl()
-                conn.setopt(pycurl.URL, url)
-                transport._set_curl_options(conn)
-                conn.setopt(pycurl.CUSTOMREQUEST, 'OPTIONS')
-                conn.setopt(pycurl.NOBODY, 1)
-                header = StringIO()
-                data = StringIO()
-                conn.setopt(pycurl.HEADERFUNCTION, header.write)
-                conn.setopt(pycurl.WRITEFUNCTION, data.write)
-                transport._curl_perform(conn, header)
-                code = conn.getinfo(pycurl.HTTP_CODE)
-                if code == 404:
-                    raise NoSuchFile(transport._path)
-                if code in (403, 405):
-                    raise subvertpy.SubversionException("version control not supported for remote URL", subvertpy.ERR_RA_DAV_NOT_VCC)
-                headers = transport._parse_headers(header)
-                return headers.getheaders('DAV')
-    raise NotImplementedError
-
-
 _warned_codeplex = False
 def warn_codeplex(host):
     global _warned_codeplex
     if not _warned_codeplex:
         warning("Please note %s is hosted on Codeplex which runs a broken "
-                "Subversion server." % host)
+                "Subversion server. Please consider using the bzr-tfs plugin, "
+                "which provides support for CodePlex' native Team Foundation Server." % host)
         _warned_codeplex = True
 
 
@@ -142,26 +106,11 @@ def get_svn_ra_transport(bzr_transport):
     if ra_transport is not None:
         return ra_transport
 
-    svnbase = bzr_to_svn_url(bzr_transport.external_url())
-    # If this is a HTTP transport, use the existing connection to check
-    # that the remote end supports version control.
-    from bzrlib.transport.http import HttpTransportBase
-    if isinstance(bzr_transport, HttpTransportBase):
-        url = bzr_transport._unsplit_url(bzr_transport._unqualified_scheme,
-            None, None, bzr_transport._host, bzr_transport._port,
-            bzr_transport._path)
-        try:
-            dav_entries = dav_options(bzr_transport, url)
-        except NotImplementedError:
-            pass
-        else:
-            dav_entries = list(itertools.chain(*[entry.split(",") for entry in dav_entries]))
-            if not "version-control" in dav_entries:
-                raise subvertpy.SubversionException("version control not supported for remote URL", subvertpy.ERR_RA_DAV_NOT_VCC)
     # Save _svn_ra transport here so we don't have to connect again next time
     # we try to use bzr svn on this transport
     shared_connection = getattr(bzr_transport, "_shared_connection", None)
-    if shared_connection is not None and shared_connection.credentials is not None:
+    if (shared_connection is not None and
+        shared_connection.credentials is not None):
         if isinstance(shared_connection.credentials, dict):
             creds = shared_connection.credentials
         elif isinstance(shared_connection.credentials[0], dict):
@@ -170,7 +119,8 @@ def get_svn_ra_transport(bzr_transport):
             creds = None
     else:
         creds = None
-    ra_transport = SvnRaTransport(svnbase, credentials=creds)
+    ra_transport = SvnRaTransport(bzr_transport.external_url(),
+        credentials=creds)
     bzr_transport._svn_ra = ra_transport
     return ra_transport
 
@@ -179,7 +129,7 @@ def _url_unescape_uri(url):
     (scheme, netloc, path, query, fragment) = urlparse.urlsplit(url)
     if scheme in ("http", "https"):
         # Without this, URLs with + in them break
-        path = urllib.unquote(path)
+        path = urllib.quote(urllib.unquote(path), safe="/+%")
     while "//" in path:
         path = path.replace("//", "/")
     return urlparse.urlunsplit((scheme, netloc, path, query, fragment))
@@ -189,12 +139,17 @@ def _url_escape_uri(url):
     (scheme, netloc, path, query, fragment) = urlparse.urlsplit(url)
     if scheme in ("http", "https"):
         # Without this, URLs with + in them break
-        path = urllib.quote(path, safe="/+")
+        path = urllib.quote(path, safe="/+%")
     return urlparse.urlunsplit((scheme, netloc, path, query, fragment))
 
 
 def url_join_unescaped_path(url, path):
-    return _url_escape_uri(urlutils.join(url, path))
+    (scheme, netloc, basepath, query, fragment) = urlparse.urlsplit(url)
+    path = urlutils.join(urllib.unquote(basepath), path)
+    if scheme in ("http", "https"):
+        # Without this, URLs with + in them break
+        path = urllib.quote(path, safe="/+%")
+    return urlparse.urlunsplit((scheme, netloc, path, query, fragment))
 
 
 def bzr_to_svn_url(url):
@@ -202,8 +157,12 @@ def bzr_to_svn_url(url):
 
     This will possibly remove the svn+ prefix.
     """
+    url = urlutils.split_segment_parameters(url.strip("/"))[0]
     if url.startswith("readonly+"):
         url = url[len("readonly+"):]
+        readonly = True
+    else:
+        readonly = False
 
     if (url.startswith("svn+http://") or
         url.startswith("svn+https://")):
@@ -214,10 +173,10 @@ def bzr_to_svn_url(url):
     # The SVN libraries don't like trailing slashes...
     url = url.rstrip('/')
 
-    return url
+    return url, readonly
 
 
-def svn_to_bzr_url(url):
+def svn_to_bzr_url(url, readonly=False):
     """Convert a Subversion URL to a URL understood by Bazaar.
 
     This mainly involves fixing file:// URLs on Windows.
@@ -226,6 +185,8 @@ def svn_to_bzr_url(url):
         # Subversion URLs have only two // after file: on Windows
         if url.startswith("file://"):
             url = "file:///" + url[len("file://"):]
+    if readonly:
+        url = "readonly+" + url
     return url
 
 
@@ -244,25 +205,29 @@ class SubversionProgressReporter(object):
             assert progress < 100000
             self._last_progress = 0
         changed = progress - self._last_progress
-        assert changed >= 0, "changed was %d (%d -> %d)" % (changed, self._last_progress, progress)
+        if changed < 0:
+            raise AssertionError("changed was %d (%d -> %d)" % (changed, self._last_progress, progress))
         self._last_progress = progress
         ui.ui_factory.report_transport_activity(self, changed, None)
 
 
-def Connection(url, auth=None):
+def Connection(url, auth=None, config=None, readonly=False):
     # Subvertpy <= 0.6.3 has a bug in the reference counting of the
     # progress update function
     if subvertpy.__version__ >= (0, 6, 3):
         progress_cb = SubversionProgressReporter(url).update
     else:
         progress_cb = None
-    assert type(url) == str
+    assert type(url) == str, "URL not string: %r" % url
     try:
-        ret = subvertpy.ra.RemoteAccess(url, auth=auth,
+        ret = RemoteAccess(_url_escape_uri(url), auth=auth,
                 client_string_func=bzrlib.plugins.svn.get_client_string,
-                progress_cb=progress_cb)
+                progress_cb=progress_cb,
+                config=config)
         if 'transport' in debug.debug_flags:
             ret = MutteringRemoteAccess(ret)
+        if readonly:
+            ret = ReadonlyRemoteAccess(ret)
     except subvertpy.SubversionException, e:
         msg, num = e.args
         if num in (subvertpy.ERR_RA_SVN_REPOS_NOT_FOUND,):
@@ -293,8 +258,10 @@ def Connection(url, auth=None):
 class ConnectionPool(object):
     """Collection of connections to a Subversion repository."""
 
-    def __init__(self, url):
+    def __init__(self, url, readonly=False):
+        self.start_url = url
         self.connections = set()
+        self.readonly = readonly
         self.auth_baton = create_auth_baton(url)
 
     def set_credentials(self, credentials):
@@ -308,6 +275,44 @@ class ConnectionPool(object):
         if password is not None:
             self.auth_baton.set_parameter(AUTH_PARAM_DEFAULT_PASSWORD, password)
 
+    def get_any(self):
+        try:
+            c = self.connections.pop()
+        except KeyError:
+            return Connection(self.start_url, self.auth_baton,
+                readonly=self.readonly)
+        else:
+            assert not c.busy, "busy connection in pool"
+            return c
+
+    def new(self, url):
+        # Nothing available? Just pick an existing one and reparent:
+        if len(self.connections) == 0:
+            return Connection(url, self.auth_baton, readonly=self.readonly)
+        c = self.connections.pop()
+        try:
+            c.reparent(_url_escape_uri(url))
+            return c
+        except NotImplementedError:
+            self.connections.add(c)
+            return Connection(url, self.auth_baton, readonly=self.readonly)
+        except:
+            self.connections.add(c)
+            raise
+
+    def get_parent(self, url):
+        # Check if there is an existing connection we can use
+        for c in self.connections:
+            assert not c.busy, "busy connection in pool"
+            assert not c.url.endswith("/")
+            if url == c.url or url.startswith(c.url+"/"):
+                self.connections.remove(c)
+                relpath = urlutils.relative_url(c.url+"/", url.rstrip("/")+"/")
+                if relpath == ".":
+                    relpath = ""
+                return c, relpath.rstrip("/")
+        return self.new(url), ""
+
     def get(self, url):
         # Check if there is an existing connection we can use
         for c in self.connections:
@@ -315,19 +320,7 @@ class ConnectionPool(object):
             if c.url == url:
                 self.connections.remove(c)
                 return c
-        # Nothing available? Just pick an existing one and reparent:
-        if len(self.connections) == 0:
-            return Connection(url, self.auth_baton)
-        c = self.connections.pop()
-        try:
-            c.reparent(_url_escape_uri(url))
-            return c
-        except NotImplementedError:
-            self.connections.add(c)
-            return Connection(url, self.auth_baton)
-        except:
-            self.connections.add(c)
-            raise
+        return self.new(url)
 
     def add(self, connection):
         assert not connection.busy, "adding busy connection in pool"
@@ -342,20 +335,23 @@ class SvnRaTransport(Transport):
 
     @convert_svn_error
     def __init__(self, url, from_transport=None, credentials=None):
-        bzr_url = url
-        self.svn_url = bzr_to_svn_url(url)
-        Transport.__init__(self, bzr_url)
-        host = urlutils.parse_url(bzr_url)[3]
+        self.svn_url, readonly = bzr_to_svn_url(url)
+        Transport.__init__(self, url)
+        host = urlutils.parse_url(url)[3]
         if host.endswith(".codeplex.com"):
             warn_codeplex(host)
         if from_transport is None:
-            self.connections = ConnectionPool(bzr_url)
+            self.connections = ConnectionPool(self.svn_url,
+                readonly=readonly)
             if credentials is not None:
                 assert isinstance(credentials, dict)
                 self.connections.set_credentials(credentials)
             # Make sure that the URL is valid by connecting to it.
             self.connections.add(self.connections.get(self.svn_url))
         else:
+            if readonly != from_transport.is_readonly():
+                raise AssertionError("readonly %r != %r (%r)" % (readonly,
+                        from_transport.is_readonly(), from_transport))
             self.connections = from_transport.connections
             if credentials is not None:
                 assert isinstance(credentials, dict)
@@ -366,6 +362,12 @@ class SvnRaTransport(Transport):
         from bzrlib.plugins.svn import lazy_check_versions
         lazy_check_versions()
 
+    def is_readonly(self):
+        return self.connections.readonly
+
+    def get_any_connection(self):
+        return self.connections.get_any()
+
     def get_connection(self, repos_path=None):
         if repos_path is not None:
             return self.connections.get(urlutils.join(self.get_svn_repos_root(),
@@ -373,6 +375,21 @@ class SvnRaTransport(Transport):
         else:
             return self.connections.get(self.svn_url)
 
+    def get_path_connection(self, path):
+        return self.connections.get_parent(
+            urlutils.join(self.svn_url, path))
+
+    def get_paths_connection(self, paths):
+        paths = [p.strip("/") for p in paths]
+        prefix = common_prefix(paths)
+        subpaths = [urlutils.determine_relative_path(prefix, p) for p in paths]
+        conn, relprefix = self.get_path_connection(prefix)
+        if relprefix == "":
+            relsubpaths = subpaths
+        else:
+            relsubpaths = [urlutils.join(relprefix, p) for p in subpaths]
+        return (conn, relsubpaths)
+
     def external_url(self):
         return self.base
 
@@ -389,7 +406,7 @@ class SvnRaTransport(Transport):
         # catch it in bzrdir.py
         raise NoSuchFile(path=relpath)
 
-    def stat(self, relpath):
+    def stat(self, path):
         """See Transport.stat()."""
         class StatResult(object):
             def __init__(self, svn_stat):
@@ -399,13 +416,10 @@ class SvnRaTransport(Transport):
                     self.st_mode = stat.S_IFREG
                 self.st_size = svn_stat['size']
                 self.st_mtime = svn_stat['time']
-        conn = self.get_connection()
+        conn, relpath = self.get_path_connection(path)
         try:
-            stat_fn = getattr(conn, "stat", None)
-            if stat_fn is None:
-                raise TransportNotPossible("stat not supported in this version")
             try:
-                return StatResult(stat_fn(relpath, -1))
+                return StatResult(conn.stat(relpath, -1))
             except NotImplementedError:
                 raise TransportNotPossible("stat not supported in this version")
         finally:
@@ -416,7 +430,7 @@ class SvnRaTransport(Transport):
 
     def get_uuid(self):
         if self._uuid is None:
-            conn = self.get_connection()
+            conn = self.get_any_connection()
             try:
                 return conn.get_uuid()
             finally:
@@ -432,7 +446,7 @@ class SvnRaTransport(Transport):
 
     def get_svn_repos_root(self):
         if self._repos_root is None:
-            conn = self.get_connection()
+            conn = self.get_any_connection()
             try:
                 self._repos_root = conn.get_repos_root()
             finally:
@@ -440,7 +454,7 @@ class SvnRaTransport(Transport):
         return self._repos_root
 
     def get_latest_revnum(self):
-        conn = self.get_connection()
+        conn = self.get_any_connection()
         try:
             return conn.get_latest_revnum()
         finally:
@@ -454,82 +468,41 @@ class SvnRaTransport(Transport):
         assert isinstance(limit, int)
 
         if paths is not None:
-            paths = [p.strip("/") for p in paths]
+            conn, paths = self.get_paths_connection(paths)
+        else:
+            conn = self.get_connection()
 
-        conn = self.get_connection()
-        try:
-            return conn.iter_log(paths, from_revnum, to_revnum, limit,
-                discover_changed_paths=discover_changed_paths,
-                strict_node_history=strict_node_history,
-                include_merged_revisions=include_merged_revisions,
-                revprops=revprops)
-        except AttributeError:
-            # Older subvertpy, fall back to iterator wrapper.
-            pass
-
-        from threading import Thread, Semaphore
-
-        class logfetcher(Thread):
-            def __init__(self, transport, conn, *args, **kwargs):
-                Thread.__init__(self)
-                self.setDaemon(True)
-                self.transport = transport
-                self.args = args
-                self.kwargs = kwargs
-                self.pending = []
-                self.conn = conn
-                self.semaphore = Semaphore(0)
-                self.busy = False
-
-            def next(self):
-                self.semaphore.acquire()
-                ret = self.pending.pop(0)
-                if isinstance(ret, Exception):
-                    raise ret
-                return ret
-
-            def run(self):
-                assert not self.busy, "already running"
-                self.busy = True
-                def rcvr(orig_paths, revision, revprops, has_children=None):
-                    self.pending.append((orig_paths, revision, revprops, has_children))
-                    self.semaphore.release()
-                try:
-                    try:
-                        self.conn.get_log(rcvr, *self.args, **self.kwargs)
-                        self.pending.append(None)
-                    except Exception, e:
-                        self.pending.append(e)
-                finally:
-                    self.pending.append(Exception("Some exception was not handled"))
-                    self.semaphore.release()
-                    self.transport.add_connection(self.conn)
-        fetcher = logfetcher(self, conn, paths, from_revnum, to_revnum,
-                limit, discover_changed_paths=discover_changed_paths,
-                strict_node_history=strict_node_history,
-                include_merged_revisions=include_merged_revisions,
-                revprops=revprops)
-        fetcher.start()
-        return iter(fetcher.next, None)
+        return conn.iter_log(paths, from_revnum, to_revnum, limit,
+            discover_changed_paths=discover_changed_paths,
+            strict_node_history=strict_node_history,
+            include_merged_revisions=include_merged_revisions,
+            revprops=revprops)
 
     def get_log(self, rcvr, paths, from_revnum, to_revnum, limit, discover_changed_paths,
                 strict_node_history, include_merged_revisions, revprops):
 
         if paths is not None:
-            paths = [p.strip("/") for p in paths]
-
-        conn = self.get_connection()
+            conn, paths = self.get_paths_connection(paths)
+        else:
+            conn = self.get_connection()
         try:
-            return conn.get_log(rcvr, paths,
-                    from_revnum, to_revnum,
-                    limit, discover_changed_paths, strict_node_history,
-                    include_merged_revisions,
-                    revprops)
+            try:
+                return conn.get_log(rcvr, paths,
+                        from_revnum, to_revnum,
+                        limit, discover_changed_paths, strict_node_history,
+                        include_merged_revisions,
+                        revprops)
+            except subvertpy.SubversionException, e:
+                msg, num = e.args
+                if num == subvertpy.ERR_RA_DAV_REQUEST_FAILED:
+                    return DavRequestFailed(msg)
+                else:
+                    raise e
         finally:
             self.add_connection(conn)
 
     def change_rev_prop(self, revnum, name, value):
-        conn = self.get_connection()
+        conn = self.get_any_connection()
         try:
             return conn.change_rev_prop(revnum, name, value)
         finally:
@@ -537,30 +510,29 @@ class SvnRaTransport(Transport):
 
     @convert_svn_error
     def get_dir(self, path, revnum, fields=0):
-        conn = self.get_connection()
+        conn, relpath = self.get_path_connection(path)
         try:
-            return conn.get_dir(path, revnum, fields)
+            return conn.get_dir(relpath, revnum, fields)
         finally:
             self.add_connection(conn)
 
     def get_file(self, path, stream, revnum):
-        conn = self.get_connection()
+        conn, relpath = self.get_path_connection(path)
         try:
-            return conn.get_file(path, stream, revnum)
+            return conn.get_file(relpath, stream, revnum)
         finally:
             self.add_connection(conn)
 
     def get_file_revs(self, path, start_revnum, end_revnum, handler):
-        conn = self.get_connection()
+        assert type(path) is str
+        conn, relpath = self.get_path_connection(path)
         try:
-            return conn.get_file_revs(path, start_revnum, end_revnum, handler)
+            return conn.get_file_revs(relpath, start_revnum, end_revnum, handler)
         finally:
             self.add_connection(conn)
 
     def list_dir(self, relpath):
         assert len(relpath) == 0 or relpath[0] != "/"
-        if relpath == ".":
-            relpath = ""
         try:
             (dirents, _, _) = self.get_dir(relpath, self.get_latest_revnum())
         except subvertpy.SubversionException, (_, num):
@@ -570,9 +542,9 @@ class SvnRaTransport(Transport):
         return dirents.keys()
 
     def check_path(self, path, revnum):
-        conn = self.get_connection()
+        conn, relpath = self.get_path_connection(path)
         try:
-            return conn.check_path(path, revnum)
+            return conn.check_path(relpath, revnum)
         finally:
             self.add_connection(conn)
 
@@ -613,7 +585,7 @@ class SvnRaTransport(Transport):
             return False
         if cap in self.capabilities:
             return self.capabilities[cap]
-        conn = self.get_connection()
+        conn = self.get_any_connection()
         try:
             try:
                 self.capabilities[cap] = conn.has_capability(cap)
@@ -628,16 +600,16 @@ class SvnRaTransport(Transport):
             self.add_connection(conn)
 
     def revprop_list(self, revnum):
-        conn = self.get_connection()
+        conn = self.get_any_connection()
         try:
             return conn.rev_proplist(revnum)
         finally:
             self.add_connection(conn)
 
     def get_locations(self, path, peg_revnum, revnums):
-        conn = self.get_connection()
+        conn, relpath = self.get_path_connection(path)
         try:
-            return conn.get_locations(path, peg_revnum, revnums)
+            return conn.get_locations(relpath, peg_revnum, revnums)
         finally:
             self.add_connection(conn)
 
@@ -684,7 +656,83 @@ class SvnRaTransport(Transport):
         return urlutils.join(self.base, relpath)
 
 
+class ReadonlyRemoteAccess(object):
+
+    busy = property(lambda self: self.actual.busy)
+    url = property(lambda self: self.actual.url)
+
+    def __init__(self, actual):
+        self.actual = actual
+
+    def check_path(self, path, revnum):
+        return self.actual.check_path(path, revnum)
+
+    def stat(self, path, revnum):
+        return self.actual.stat(path, revnum)
+
+    def has_capability(self, cap):
+        return self.actual.has_capability(cap)
+
+    def get_uuid(self):
+        return self.actual.get_uuid()
+
+    def get_repos_root(self):
+        return self.actual.get_repos_root()
+
+    def get_latest_revnum(self):
+        return self.actual.get_latest_revnum()
+
+    def get_log(self, *args, **kwargs):
+        return self.actual.get_log(*args, **kwargs)
+
+    def iter_log(self, *args, **kwargs):
+        return self.actual.iter_log(*args, **kwargs)
+
+    def change_rev_prop(self, revnum, name, value):
+        raise TransportNotPossible('readonly transport')
+
+    def get_dir(self, *args, **kwargs):
+        return self.actual.get_dir(*args, **kwargs)
+
+    def get_file(self, *args, **kwargs):
+        return self.actual.get_file(*args, **kwargs)
+
+    def get_file_revs(self, *args, **kwargs):
+        return self.actual.get_file_revs(*args, **kwargs)
+
+    def revprop_list(self, revnum):
+        return self.actual.revprop_list(revnum)
+
+    def get_locations(self, path, peg_revnum, revnums):
+        return self.actual.get_locations(path, peg_revnum, revnums)
+
+    def do_update(self, revnum, path, start_empty, editor):
+        return self.actual.do_update(revnum, path, start_empty, editor)
+
+    def do_diff(self, *args, **kwargs):
+        return self.actual.do_diff(*args, **kwargs)
+
+    def do_switch(self, *args, **kwargs):
+        return self.actual.do_switch(*args, **kwargs)
+
+    def reparent(self, url):
+        return self.actual.reparent(url)
+
+    def get_commit_editor(self, *args, **kwargs):
+        raise TransportNotPossible('readonly transport')
+
+    def rev_proplist(self, revnum):
+        return self.actual.rev_proplist(revnum)
+
+    def replay_range(self, *args, **kwargs):
+        return self.actual.replay_range(*args, **kwargs)
+
+    def replay(self, *args, **kwargs):
+        return self.actual.replay(*args, **kwargs)
+
+
 class MutteringRemoteAccess(object):
+    """Trivial RemoteAccess wrapper that mutters all activity."""
 
     busy = property(lambda self: self.actual.busy)
     url = property(lambda self: self.actual.url)
@@ -698,10 +746,7 @@ class MutteringRemoteAccess(object):
 
     def stat(self, path, revnum):
         mutter('svn stat -r%d %s' % (revnum, path))
-        stat = getattr(self.actual, "stat", None)
-        if stat is None:
-            raise NotImplementedError(self.stat)
-        return stat(path, revnum)
+        return self.actual.stat(path, revnum)
 
     def has_capability(self, cap):
         return self.actual.has_capability(cap)
